// ================================================================
//  Mega Games - 40+ Games... with a Two level Menu selection
//  for use with mini STEM Educational Game Platform
//  having an 8 LED 4 color bar display and a 4 Button module +Piezo audio out
//  see: https://www.instructables.com/member/RonM9

//  Dec 2021 Ronald R. Miller first 40+Mega_Game dev&qa completed
//  14 Jan 2022 Update

// include Library of Hardware Support functions
#include "Mini_STEM_Platform.h"

#define UNUSED_ADC ADC_a // A0 is OK for Nano, A7 for ProMicro or Nano, A3 for rp2040
#define BEEPPIN AUDIO_OUT
#define SPEAKERPIN AUDIO_OUT
#define SPEAKERNEG AUDIO_NEG
bool spkr = true; // speakers & pasive piezos can be AC driven for more volume (not active buzzers)
bool quiet = false; // assumes a resister between AUDIO_NEG & GND (w/o resister quiet=silent)

// the false 'Verbose' saves many hundreds of bytes of dynamic memory (but gives less Monitor messages)
#define Verbose false

int readySetGo(int maxDelay = 0);
bool btnPressed();
void taDah();
int noBtnDelay(int);
void Beep(unsigned short int);
void playNote(int, int);
void playNote(int);
void wait_BtnRelease();
void beepTone(int, int);
void dspColors();
void showLevel(byte level);
void genTone(int, int);
bool catchBtn();
bool scanBtnWait(int);
void genSndEffect(int , int , int , int , int  );
void setColor(byte);
void QuickDraw();
void Tug_a_War();
void Chicken();
void Hot_Hands();
void PIG();
void showColor(byte );


void TBD() {
  clearDisp();
  buZZ(40);
  Serial.println("This Selection is yet To Be Defined.");
}
void NA() {buZZ(20);}

/**
   Declare an array of pointers to functions returning void
   and initialize with desired game functions
*/
#ifdef RP2040
void (*games[9][9])() = {
  // 0   ------ Diagnostic Tools & Utilities     NOTE: [0][0] must by TBD
  {TBD, btnReporting, Color_lights, GlowBeepTest, DebounceTest, TBD, TBD, TBD, TBD},
  // 1   ------ Early Inline Games (Oldies & Goodies)
  {setup, PushIt, ShootingGallery, JumpMan_8, Skee_ball, Sea_Hunt_8, PIG, BiFunc, Flip_d},
  // 2   ------ Two Person Challenges
  {NA, pingPong, QuickDraw, Tug_a_War, Chicken, Hot_Hands, Spray, Joust, UpDownSide},
  // 3   ------ Spirted FUN
  {NA, Whack_Mole, SimonSays, Speed_Scan_r4, Telekinesis, PKE_Meter, Affinity_Meter, Forture_Teller, TBD},
  // 4   ------ Show off your Skills
  {NA, playThatTune, React_Now_r2, SeeIt_CatchIt, Balance, Shuffleboard, Memory, CodeBreaker, TBD},
  // 5   ------ Demos, Challenges, Activities, and Applets
  {NA, Support_Demo, Illumination_Challenge, Automated_Art, tuneLightOrgan, SoundEffects, Timer, Metronome, readVoltages},
  // 6   ------ Morse Code Practice & Exercise samples
  {NA, MC_freeForm, A2Z_Morse_Exercise, Morse_AB_test, MC_Relay_exercise, MC_Receive_Practice, Tx_Line_test, TBD},
  // 7
  {TBD, TBD},
  // 8
  {TBD, TBD}
};

#else
void (*games[9][9])() = {
  // 0   ------ Diagnostic Tools & Utilities     NOTE: [0][0] must by TBD
  //{TBD, btnReporting, Color_lights, GlowBeepTest, TBD, TBD, TBD, TBD, TBD},
  //{TBD, setup, debugTest, DebounceTest, TBD},  // some debug stuff
  {TBD, setup, TBD},   //  setup is ~ a reset restart (desired for Pro-Micro, to do serial printouts)

  // 1   ------ Early Inline Games (Oldies & Goodies)
  {NA, PushIt, ShootingGallery, JumpMan_8, Skee_ball, Sea_Hunt_8, PIG, BiFunc, Flip_d},
  //{TBD, TBD}, //  TBD in cell 1-8 indicate no more valid entries

  // 2   ------ Two Person Challenges
  //{NA, pingPong, QuickDraw, Tug_a_War, Chicken, Hot_Hands, Spray, Joust, UpDownSide},
  {TBD, TBD},

  // 3   ------ Spirted FUN
  {NA, Whack_Mole, SimonSays, Speed_Scan_r4, Telekinesis, PKE_Meter, Affinity_Meter, Forture_Teller, TBD},
  //{TBD, TBD}, //  TBD in cell 1-8 indicate no more valid entries

  // 4   ------ Show off your Skills
  {NA, playThatTune, React_Now_r2, SeeIt_CatchIt, Balance, Shuffleboard, Memory, CodeBreaker, TBD},
  //{TBD, TBD},

  // 5   ------ Demos, Challenges, Activities, and Applets
  //{NA, Support_Demo, Illumination_Challenge, Automated_Art, tuneLightOrgan, SoundEffects, Timer, Metronome, readVoltages},
  //{NA, Support_Demo, Illumination_Challenge, Automated_Art, tuneLightOrgan, NA, Timer, Metronome, readVoltages}, // alittle smaller for ProMicro
  {TBD, TBD},

  // 6   ------ Morse Code Practice & Exercise samples
  //{NA, MC_freeForm, A2Z_Morse_Exercise, Morse_AB_test, MC_Relay_exercise, MC_Receive_Practice, Tx_Line_test, TBD},
  //{NA, MC_freeForm, A2Z_Morse_Exercise, Morse_AB_test, MC_Relay_exercise, TBD, TBD, TBD},
  {TBD, TBD},

  // 7
  {TBD, TBD},
  // 8
  {TBD, TBD}
};
#endif

byte gameSets; // = whats filled in above (# of valid rows, without 'TDB' in cell [1])
byte entries[9];

// global game stuff
byte group=1;
byte player;
bool playing;
int turn;  // turn counter
byte ver;
byte level, goal;
int levelMax;
int dir;
int  rally;
unsigned int idleCnt = 0, loopCnt;

unsigned long t0, t1, t2;
unsigned long randSeed = 0;

// Game music stuff
//    1   2   3   4   5   6   7   8   9   10  11  12  13  14  15  16  17  18  19  20  21  22
//                                do  re  mi  fa  so  la  ti  do
//    C3  D3  E3  F3  G3  A4  B4  C4  D4  E4  F4  G4  A4  B4  C5  D5  E5  F5  G5  A5  B5  C6
const byte starttune[] = {8, 11, 8, 11, 8, 11, 8, 11, 12, 11, 10, 11, 12};
const int  duration[] = {100, 200, 100, 200, 100, 400, 100, 100, 100, 100, 200, 100, 500};
const byte cheer[]    = {8, 8, 12, 15, 12, 15};
const int  cheerDur[] = {100, 100, 100, 300, 100, 300};
const byte btnNote[] = {5, 6, 7, 8};
int effect;

// used by Simon Says
byte randomArray[50]; //Intentionally long to store up to 50 inputs (doubtful anyone will get this far)
byte inputArray[50];

const int basicKeyNotes[] =
{0, 131, 147, 165, 175, 196, 220, 247, 262, 294, 330, 349, 392, 440, 494, 523, 587, 659, 698, 784, 880, 988, 1047, 1175, 1319};
//    1   2   3   4   5   6   7   8   9   10  11  12  13  14  15  16  17  18  19  20  21  22  23  24
//                                do  re  mi  fa  so  la  ti  do
//    C3  D3  E3  F3  G3  A3  B3  C4  D4  E4  F4  G4  A4  B4  C5  D5  E5  F5  G5  A5  B5  C6  D6  E6

bool volHigh = false;

//   LED display stuff
int cursorPtr;
bool colorLeds[5];
const int colorSet[4][2] = { {1, 2},  {3, 4},  {5, 6},  {7, 8} };
byte cell[14];  // working buffer

/*** This interrupt drive 1 milisecond update routine does not work for all processors ...
 *   and with out it I am sure to keep the main code capable of working smooth even without it
//#include <avr/io.h>
//#include <avr/interrupt.h>
//#include <avr/sleep.h>

  volatile unsigned long myMillis=0;
  unsigned long msec;
  unsigned long usecs;
  unsigned int mySecs=0, secsIdle=0;
  //======================================================
  //Timer2 Overflow Interrupt Vector, called every 1ms (works with some AVR MCUs, eg: Nano)
  ISR(TIMER2_OVF_vect) {
  TCNT2 = 131;       // Reset Timer to 130 out of 255;  +1 fine adj' for my use
                     // + reduces the interval, resolution: 1 = 0.5secs/60secs
  myMillis++;
  if (myMillis%1000 == 0) { // every 1 second
    mySecs++;
    secsIdle++;
  }
  oneMilliUpdate();
  TIFR2 = 0x00;      // Timer2 INT Flag Reg: Clear Timer Overflow Flag
  };

  void oneMilliUpdate() {
  refreshDisp();
  }
***/

// ---------- Morse code support data structure ------------
int numWrds = 900; //or 500
#define IsPunct  (cinput>32 && !(toupper(cinput)>64 && toupper(cinput)<91)
bool prtChr, pAdj;

unsigned int avgPeriod, pAveraging;
unsigned int ditPeriod;
int farns, deFF;  // Farnsworth Factor, multiplier for std to Farnsworth inter char & word periods
bool pOpen = true; // expect an open ( else next should be close )

bool oneKey = true; // straight single contact key   (TBD: false=using a 2 contact paddle)
//bool batOp=false; // would-be indicator for battery Operation (w/o serial monitor)
char didaStr[10];

const char alpha[] = "ZABCDEFGHIJKLMNOPQRSTUVWXYZA";
const char nonAlpha[] = " 1234567890.,?/'-:\"+!=@$)&\n(_~";
//       1    2    3    4    5    6    7    8    9    0
const byte nonAlpha_dida[] = {0, 0x10, 0x18, 0x1C, 0x1E, 0x1F, 0xEF, 0xE7, 0xE3, 0xE1, 0xE0,
                              0x2A, 0xCC, 0x33, 0xED, 0x21, 0xDE, 0xC7, 0x2D, 0x15, 0xD4, 0xEE, 0x25, 0x76, 0xD2, 0x17, 0x0A, 0xE9, 0x0};
                            //  .    ,     ?     /     '     -     :     "     +     !     =     @     $     )     &     \n    (     _

void MC_setup() {
  //  ditPeriod= (60000)/(240*2);  // in msecs, for ~10WPM to start off  (240 in 60,000 ms is 5WPM)
  ditPeriod = (60000) / (240 * 2 + 120); // in msecs, for ~12WPM to start off  (240 in 60,000 ms is 5WPM)
  //  ditPeriod= (60000)/(240*3);      // in msecs, for ~15WPM to start off  (240 in 60,000 ms is 5WPM)
  //  ditPeriod= (60000)/(240*4);      // in msecs, for ~20WPM to start off  (240 in 60,000 ms is 5WPM)

  // Farnsworth calculations
  int cWPM = 18, sWPM = 10;
  float fp = 1000. * (60 / sWPM - 37.2 / cWPM) / 19.; // inter time period of Farnsworth timing
  farns = fp / (1000 * 1.2 / cWPM); // factor to convert std to Farnsworth unit time period
  deFF = farns;

  avgPeriod = 2 * ditPeriod;
  pAveraging = 4 * avgPeriod;

  if (btn1) {
    farns = 1; // disable Farnsworth effect if btn1 held down at power up.
    Serial.println("Standard inter-char & word timing used.");
  } else {
    Serial.print("Farnsworth ratio: "); Serial.print(sWPM); Serial.print("/"); Serial.println(cWPM);
  }

  //  Serial.print(" Keying rate (in WPM): "); Serial.print((1000.0*60.0) / (240.0*ditPeriod/5.0));
  //  Serial.print("  + Farnsworth spacing: x"); Serial.println(farns);

}

// the setup routine runs once on power up or when you press reset:
//=================================================================
void setup() {
  int aval, val;
  unsigned long gsum;  // keep gsum as uninitialized
  //  long asum;
  //  unsigned long dsum;

  // initialize serial communication at 9600 bits per second:
  Serial.begin(115200);
  Serial.println("Mega Games 40+ utilizing the mini LED STEM Platform");

  // determine the # of valid entries in games[][]
  for (int i = 0; i <= nleds; i++) {
    entries[i] = nleds; // herein a max of 8 entries
    for (int j = 1; j <= nleds; j++) {
      if (games[i][j] == games[0][0]) {
        entries[i] = j - 1; // set not fill to the limit
        break;
      }
    }
    Serial.print(i); Serial.print("  entries: "); Serial.println(entries[i]);
    if (entries[i] > 0) gameSets = i + 1;
  }
  Serial.print("  groups: "); Serial.println(gameSets);


  // ------------ setup the Mini Game/STEM platform (8LEDs, 4Btns, and Audio)
  gPlatform_setup();
  if (gameSets > 6) MC_setup(); // setup for Morse Code operations

  beep(50, 50); beep(50, 50); beep(150); // announce "it is alive!"

  // accumulate pseudo random values, due to HW input noise (Entropy), then randomize
  // used when random values need to be generated prior to (/without) ramdomly occuring user input
  //asum=dsum=0;
  t0 = micros();
  for (int i = 1; i <= 100; i++) { // work up a random seed value
    aval = analogRead(ADC_b) + analogRead(ADC_a) + 1;
    gsum = gsum + i * aval;
    //    asum = asum + aval;
    //    dsum += abs((asum/i)-aval);
    delayMicroseconds(500 + gsum % 1000); // average 1000 us
  }
  randSeed = (gsum + micros()) % 9999999; // here you could set a seed to a given value for repeated testing
  //  Serial.print("geo sum: "); Serial.print(gsum); Serial.print("   micros: "); Serial.println(micros()-t0);
  //  Serial.print("Aval avg:  "); Serial.print(asum/100); Serial.print("    Aval Delta Total:  "); Serial.println(dsum);
  Serial.print("Random Seed used: "); Serial.println(randSeed);
  randomSeed(randSeed); //Added to generate "fresh randomness" with the randomArray for the output function

  /****   currently managing without it
    //Setup Timer2 to fire every 1ms
    //------------------------------
    TCCR2B = 0x00;        //Disable Timer2 while we set it up
    TCNT2  = 130;         //Reset Timer Count to 130 out of 255
    TIFR2  = 0x00;        //Timer2 INT Flag Reg: Clear Timer Overflow Flag
    TIMSK2 = 0x01;        //Timer2 INT Reg: Timer2 Overflow Interrupt Enable
    TCCR2A = 0x00;        //Timer2 Control Reg A: Wave Gen Mode normal
    TCCR2B = 0x05;        //Timer2 Control Reg B: Timer Prescaler set to 128
  ***/
}

// the loop routine runs over and over again forever:
//=================================================================
void loop() { // Top Menu level, for selecting a activity
  byte func, maxFunc;

  //group = 1; // set at reset
  if (1>group || group>8) group = 1;
  func = 1;

  group = get_Option(gameSets - 1, group); // select Group of interest   (can adj volume with Btn4 while here)
  if (entries[group] == 0) {
    buZZ(75);
    Serial.print("SORRY, nothing in that group.");
    clearDisp();
    delay(700);
    return;
  }
  Serial.print(" Group: "); Serial.println(group);

  while (1) {
    pAdj = true;  // enable MC ditdah period adjustment
    prtChr = true; // MC chr prints as fetched

    glowEnable = false;
    clearDisp();
    while (Btn) {   // wait for no button
      if (btn4) buZZ(1);
      refreshWait(1);
      scanBtns();
    }

    // currently there are upto 8 activities/games supported
    maxFunc = entries[group];
    // indicate selectable functions
    for (int i = 1; i <= maxFunc; i++) {
      dim[i] = true;
    }
    brightOne = func;
    refreshDisp();

    //    while(Btn) {scanBtns(); refreshDisp();}  // wait here for button release
    while (! btn1) {
      refreshWait(2); // 2ms so are selection is extra sure footed, incase of bouncy switches
      scanBtns();
      //if (btn2Pressed) brightOne = (brightOne==1)?  maxFunc : brightOne-1;  // -- move left
      if (btn2Pressed) brightOne = (brightOne == 0) ?  maxFunc : brightOne-1; // -- move left Allow 0:debugging
      if (btn3Pressed) brightOne = (brightOne % maxFunc) + 1; // ++ moves right
      if (ESC || btn4) {
        boop(50);
        return;
      }
    }
    if (randSeed == 0) {
      randSeed = random(micros() / 4); // Add "more randomness" to random() calls, based on time btn pressed
      randomSeed(randSeed);       // for repeated testing, set randSeed=desired initial value
      Serial.print("random Seed:  "); Serial.println( randSeed);
    }

    func = brightOne;
    clearDisp();
    level = 0;
    while (Btn) scanBtns(); // wait for button release

    Serial.print(" FUNC: "); Serial.println(func);

    games[group][func]();   // call the selected function
    if (func == 0) func = 1;
    if (group == 0) return; // start all over if any diag type code was run

    pinMode(ADC_b, INPUT);  // be sure to have it reinstated
    digitalWrite(AUDIO_NEG, LOW); // will have no effect if it is set as INPUT
  }
}



// ========================   Game Loops   =============================

// ************************  Group 1  *******************************

// Various selection of mini Apps / Games / Challenges
//=============================================================
// from Menu16Games Set #1 (reworked for 8 instead of 12 LEDs & made dependant on 'nleds')  

void PushIt() { // PushIt loop
  // Nov 2017 reworked 'PushIt' operation to be useful with displays of < 12 Leds
  bool lastPressed, pressed = false, pushed = false;

  static int maxCnt = 2; // starts at 2 after power on
  int litPeriod, period, cnt, lastMaxCnt, savedCnt;
  bool isLit, incDone = false, decDue = false;

  clearDisp();
  refreshDisp();

  // Push-It init code
  // --------------------------
  litPeriod = 1000;
  period = litPeriod;
  cnt = 0; // =1 resulted in initially one too few flashes

  // read previous incarnation of 'max count' from EEPROM for the 'Push-It' game
  ////  savedCnt = EEPROM.read(eeCntAddr);
  ////  maxCnt = (savedCnt < 50)? savedCnt: 2; // be reasonable
  // ToBeDone ... ReWorked to work for AVR & RP2040, or not
  // for now 'maxCnt' is a static var

  showLevel(maxCnt);

  digitalWrite(ledOnBrd, HIGH);  //needs testing with RP2040
  t0 = millis();
  isLit = false;

  while (1) {
    pressed = btnPressed() && !btn4;
    if (!lastPressed && pressed)  {
      pushed = true;
    }

    // Flash from 1 to n times
    // Increasing the number every time the user has pressed the button on the first flash of the series
    // A missed attempt will reduce the count

    if ((millis() - t0) >= period) {  //    is this time period up ?
      if (cnt <= 1) {
        litPeriod = (nleds < 12) ? 800 : 1000;
        for (int i = 1; i < maxCnt; i++) {
          //good for 12 leds:    litPeriod = 16 * (litPeriod-100) / 20 +100;  //  P = 0.8*(P-100) + 100
          litPeriod = 15 * (litPeriod - 80) / 20 + 80; //  P = 0.75*(P-80) + 80
        }
      }

      if (isLit) {                       // about to go from LED on to OFF
        if (cnt == maxCnt + 1) { // on orig 'Push-It' this was if (cnt == 1) {
          //             !!!!!!!!!!!!!!!! ------------------------------------  time to judge
          lastMaxCnt = maxCnt;
          if (pushed) {       // he hit the mark
            maxCnt++;
            cell[lastMaxCnt] = 1;
            //            litLED=maxCnt;    // acknowledge with a new bright LED
            showLevel(maxCnt);  // now ack with 'showLevel' so we can discern levels above nleds
            refreshWait(500);
            incDone = true;
            if (maxCnt > nleds) {
              refreshWait(500);
              taDah();       // reached the highest level
              //              maxCnt=2;   // now, allow levels > nleds
            }
          }

          if (maxCnt != lastMaxCnt) {
            Serial.print("maxCnt = ");
            Serial.println(maxCnt);
          }
          if (savedCnt != maxCnt && (maxCnt == lastMaxCnt)) {  // dont write more often than needed
            savedCnt = maxCnt;
            // TBD            EEPROM.write(eeCntAddr, maxCnt);
          }

          cnt = maxCnt + 1;
        } else {  // isLit and not time to judge
        }
        period = litPeriod / 3;

      } else {                     // about to go from LED off to ON
        period = litPeriod;
        cnt++;
        if (cnt > (maxCnt + 1)) { // at maxCnt LED is off for one count
          cnt = 1;             // ===========  restart the cycle
          incDone = false;
          digitalWrite(ledOnBrd, HIGH); // needs testing with RP2040
        }
        //      Serial.print(cnt);
      }
      t0 = millis();
      isLit = not isLit;

      if (cnt > 0 && cnt <= maxCnt) {  // && cnt < maxCnt
        if (pushed && !incDone) {
          maxCnt--;
          if (maxCnt < 1) maxCnt = 1;
          cell[maxCnt] = 0;
          brightOne = 0;
          beep(100);    // had a miss & a strike
        }
      }
      lastPressed = pressed;
      pushed = false;
    }

    //-------------- UPDATE Display
    litLED = (maxCnt - 1) % nleds + 1;
    if (litLED == 1) litLED = 0; // so you can tell when it flashes
    for (int i = 1; i <= nleds; i++) {
      if (i <= maxCnt) { //
        dim[i] = true;
      } else {
        dim[i] = false;
      }
    }
    if (maxCnt > nleds) { // make it easier to see the fast flashes
      brightOne = 0;
      spriteTwo = 0;
      dim[1] = 0;
    }

    if (isLit && cnt < maxCnt + 1) spriteTwo = 1; // brighten led #1
    else spriteTwo = nleds + 1; //ledON(13); // No 'ON' lit put use up one's normal time, so dim leds don't flicker  ###TODO
    if (cnt == maxCnt + 1 && pushed) {
      spriteTwo = (maxCnt + 1);
    }
    if (spriteOne > 0) {
      refreshWait(500);
      spriteOne = 0;
    }

    refreshWait(1);
    if (ESC || btn4) {
      refreshWait(300);
      return;
    }

    if (btn2) { // re initial game level
      maxCnt = 2;
      cnt = 0;
    }
  }
}

// =====================================================================
void ShootingGallery() {  // a Shooting Arcade,  renamed from 'SootEmUp'
  int targs;
  bool playing = true;
  int cycleTime;
  int scanCell, targCell;
  //  int level=1;

  byte ver = getOption(2);  // version 1: sight moves & targets are stationary
  // version 2: sight stationary & targets move
  // ver 3&4 the targets can only be seen when under the sight, but is too difficult for me, so commented out below
  if (ver == 0) return;

  level = 1;
  while (1) {
    if (level > 10) level = 1;
    // setup game-level
    clearDisp();

    for (int i = 1; i <= nleds; i++) cell[i] = 1; // load up the targets
    if (ver == 1) {
      cell[3] = 0;   // leave gabs, seeing aid
      cell[6] = 0;
    }
    else        {
      cell[1] = 0;
      cell[5] = 0;
    }

    while (btnPressed()) {
      refreshWait(1); // wait for no button pressed
    }

    playing = true;
    while (playing) {
      scanCell = scanCell % nleds + 1;   // goes from 1 to nleds
      targs = 0;
      for (int i = 1; i <= nleds; i++) {
        if (cell[i]) targs++;
      }
      if (targs == 0) {
        //horray();
        taDah();
        level++;
        //playing=false; // restart game at next level
        break;
      }

      //================ UPDATE Display

      if (ver == 1) {
        for (int i = 1; i <= nleds; i++) {
          if (ver <= 2) dim[i] = cell[i];
        }
        litLED = scanCell;
      } else if (ver == 2) { // ver is 2
        for (int i = 1; i <= nleds; i++) {
          dim[i] = cell[(scanCell + i) % nleds + 1]; // causes a moving shifted display of the cells
        }
        //        if (dim[nleds/2]) litLED = nleds/2;
        //        else litLED=0;
        if (dim[nleds / 2]) brightOne = nleds / 2;
        else brightOne = 0;
        dim[nleds / 2] = true;
      }
      /***
            // you have to shot a few blindly, then be very keen as to when there is a target in your sights (when it goes bright)
            if (ver==3) {
              for (i=1; i<=nleds; i++) { dim[i] = false; }
              dim[scanCell]=true; // comment out for high contrast (off/bright)
              if (cell[scanCell]) brightOne = scanCell;
              else brightOne=0;
            } else if (ver==4){
              targCell = (scanCell+6)%nleds+1;
              if (cell[targCell]) brightOne = 6;
              else brightOne=0;
              dim[6]=true;
            }
      ***/

      //      cycleTime = 4 * (10-level) * (10-level) + 10;      // ~ 400-40
      //      cycleTime = ((10-level) * (10-level) * (10-level))/2 + 20;      // ~ 800-50 is OK
      cycleTime = 5 * (10 - level) * (10 - level) + 20;  // ~ 500-40 ?

      if (DispScanWait(cycleTime)) {
        if (btn1Pressed || btn4Pressed) {
          if (ver == 1 || ver == 3) targCell = scanCell;
          else        targCell = (scanCell + (nleds / 2)) % nleds + 1;
          if (cell[targCell] == 1) {
            cell[targCell] = 0; // got it
            beepTone(1200,8); beepTone(1600,16); // p-ing off the target
          } else {
            cell[targCell] = 1;
            buZZ(10);       // missed
          }
        }
        if (ESC) {
          refreshWait(300);
          return;
        }
        if (btn3Pressed) {    // step up the level
          level = (level >= 10) ? 1 : level + 1;
          announceLevel(level);
        }
      }

    }
  }
}

// =====================================================================
void JumpMan_8() { // Jump Man loop
  byte csum;
  bool jump, duck;
  bool jumping;
  bool jBtn, dBtn;
  bool triedLast, lastDBtn;
  int cycleTime;
  int objCnt;
  bool playing = true;
  byte ver = 1;

  //  blinkEnabled = true;

  if (nleds < 8) {
    ver = 1;
  }
  else {
    ver = getOption(3);  // version 1: will use button 1 or 4 to jump
    // version 2: add over head obsticles, to duck under (if you figure out how)
    // version 3: for Jump: Btn1 / Duck: btn2 or btn4
  }
  if (ver == 0) return;

  //  allColor=0;
  for (int i = 1; i <= nleds; i++) {
    cell[i] = 0;
  }

  level = 1;
  objCnt = 0;
  jump = duck = false;
  playing = true;
  while (playing) {

    if (cell[1] == 1) {
      if (jump) { // jump btn must be newly press (or in a multi-object jump
        // GOOD
        jumping = true;
        if (cell[2] == 0) objCnt++; // multi cell objects are credited only once
      } else { // BAD
        objCnt--;
        beep(25);
      }
    } else {
      jumping = false;
    }

    if (cell[1] == 2) {
      if (duck) { //
        // GOOD
        if (cell[2] == 0) objCnt++;
      } else { // BAD
        objCnt--;
        beep(25);
      }
    }
    if (objCnt >= 4) {
      objCnt = 0;
      level++;
      // acknowledge the Advancement
      beep(150); refreshWait(50); beep(350);
      // clear cell 1-(nleds/2)
      cell[1] = cell[2] = cell[3] = cell[4] = cell[nleds / 2 - 1] = cell[nleds / 2] = 0;
    }
    if (objCnt < 0) objCnt = 0;

    csum = 0;
    // shift obsticals left
    for (int i = 1; i < nleds; i++) {
      cell[i] = cell[i + 1];
      if (i > (nleds / 2)) csum += cell[i];
    }

    if (random(0, 6) >= 5) cell[nleds] = 1; // 1/6 of the time generate an obstical
    //    if (random(0,5)>=4) cell[nleds] = 1;  // 1/5 of the time generate an obstical
    else cell[nleds] = 0;
    if (ver > 1 && random(0, 6) >= 5 && cell[nleds] == 1 && cell[nleds - 2] != 2 && cell[nleds - 1] != 2)  cell[nleds] = 2;
    if (csum == 0 && cell[nleds] == 0) cell[nleds] = random(0, ver + 1); // if low on objs try to add something in

    //================ UPDATE Display
    spriteOne = 0;
    flashLed = 0;
    for (int i = nleds; i >= 1; i--) {
      dim[i] = (cell[i] != 0);
      if (cell[i] == 2) {
        flashLed = i;
      }
    }
    dim[1] = true;

    if (cell[1]) spriteOne = 1;

    // cycleTime = ((10-level) * (10-level) * (10-level))/2 + 20;      // ~ 800-50 is OK
    // cycleTime = (5 * (12 - level) * (12 - level)) + 30;  // ~
    if (level<7) cycleTime = (5 * (12) * (11 - level));
    else         cycleTime = (5 * (14-level) * (14-level));
    
    DispScanWait(cycleTime);  // refresh display while catching btn press
    // what is the player trying to do ?
    triedLast = jBtn;
    if (ver < 3) {
      jBtn = btn1 || btn4;
      duck = triedLast && !BtnPressed;
    } else {
      jBtn = btn1;
      lastDBtn = dBtn;
      dBtn = btn2 || btn4;
      duck = dBtn && !lastDBtn;
    }
    jump =  jBtn && (!triedLast || jumping);

    if (btn3Pressed) {    // step up the level
      level = (level >= 9) ? 1 : level + 1;
      announceLevel(level);
    }
    // provide for changing ver while playing
    if (btn3) {
      ver = ver % 3 + 1;
      level = 1; // would user rather not reset the level(speed) ?Hum
      beeps(50, ver); refreshWait(500);
    }

    // if 'ESC' ... Exit, but first report on level reached
    if (ESC) {
      showLevel(level);
      refreshWait(750);
      while (!btnPressed());
//      clearDisp();
//      refreshWait(250);
      return;
    }
    if (level > 12) level = 1; //   >=12 is 'plaid' speed
  }
}

void Skee_ball() { // Skee ball
  // three target "hoops", worth (1) (3) (2)  pts. The closest is the 1pt hoop
  // You are scored after each 5 balls (15 is the maximum score)
  // hold Btn1 to swing, and let go to throw the ball
  int player = 0;
  int i, cnt;
  byte spot, cell, balls, score;
  byte h1, h2, h3;  // hoops valued 1-3
  int travel, rate, thrust;
  bool rolling, newPressed, released, btnDown, wasDown;

  loopCnt = 0;
  level = 1;
  ver = 1;  //if (ver==0) return;
  glowEnable = true;

  clearDisp(); refreshWait(500);
  while (1) {
    // setup game-level
    wait_BtnRelease();   // wait for no button pressed
    spot = 1;
    cell = 0;
    balls = 0;
    score = 0;

    clearDisp();
    glow[nleds - 2] = true;
    dim[nleds - 3] = lit[nleds - 1] = true;

    h1 = nleds - 3; h3 = nleds - 2; h2 = nleds - 1;
    playing = true;
    while (playing) {
      spot = 1;       // where we are throwing from
      dimLED = 1;
      travel = 0;
      thrust = 0;
      rolling = false;
      turn = 1;   // # of rolled balls
      while (turn) {
        bool pressed = btnPressed();
        wasDown = btnDown; // keep B2&3 from wasing a 'release'
        btnDown = (btn1 || btn4);
        if (btnDown && !rolling) {
          thrust++;
          cell = 1;
          spriteOne = 1;
        }

        released = wasDown && (!btnDown) && BtnChanged;
        if (released) {   // the ball takes flight
          rolling = true;
          dimLED = 0;
          thrust = (3 * thrust) / (level + 2); // at higher levels targets are effectivly further, require more thrust
          //Serial.print("thrust: "); Serial.println(thrust);
          rate = (thrust + 750) / 15; //for thrust=7500 gives 100 which results in 10 spots in one second
          rate = rate / 4; // patch so we can update more often for smoother fast motion balls
        }

        if (rolling && (loopCnt % 50) == 0) { // was %200 before patch above
          if (thrust <= 30) {         // coming to a stop
            if ((nleds - 3) <= cell && cell <= (nleds - 1)) { // goes in one of the hoops
              score += (cell == h3) ? 3 : (cell == h2) ? 2 : 1;
              beep(40, 15);   // Tah
              if (cell == h2) beep(30);
              if (cell == h3)  {
                refreshWait(130);
                beep(200);   // Dah
              }
            } else {      // missed
              buZZ(50);
            }
            turn = 0;
            balls++;
            refreshWait(1000);
            cell = 0;
            spriteOne = 0;

            if (balls >= 5) {  // 5 ball set complete, give score
              showLevel(score);   // show the Score
              refreshWait(2000);
              if (score > 10) level++;
              playing = false;
              break;  // from this game-set
            }
            continue;
          }

          travel = travel + rate;
          thrust -= rate;
          cell = spot + travel / 100;
          if (cell >= nleds + 1) cell = (cell % nleds) + 1;
          spriteOne = cell;
        }

        //================ UPDATE Display
        loopCnt++;
        refreshWait(1);
        // if 'esc' button held down ... Exit
        if (ESC || (btn4 && btn1)) {
          return;
        }
      }
    }
  }
}

void Sea_Hunt_8() { // reworked for use with 8 instead of 12 LEDs
  int targCell;
  int subLoc, subDir;
  int player = 1; // first player is #1 second is #0
  int depChrg = 0;
  int depChgCnt, depChgLimit;
  bool sonarPing = false;
  byte ping;
  bool advance;
  byte n;

  ver = getOption(4); // version 1: Each volley is both a depth-charge and a sonar-ping
  // version 2: Head to head, taking turns, to see who sinks the sub (player1 uses Btn1 the other Btn4)
  // version 3: depth-charges and sonar-pings are launch separately (via Btn1 & Btn4)   3 depth-charges only, if enabled
  // version 4: the sub speed can be 1-3 cells per volley,                              6 depth-charges only
  // a limit to the number of depth charges for ver. 3 & 4 can be activated
  if (ver == 0) return;
  while (1) {
    // setup game-level
    clearDisp();
    refreshWait(750);

    // locate sub & its direction
    subLoc = random(1, nleds + 1);
    subDir = (random(0, 2) == 0) ? -1 : 1;
    depChgLimit = (ver < 4) ? 3 : 6;
    if (ver == 4) subDir = subDir * random(1, 4); // speed can be +/- 1,2,3
    //    Serial.print("subLoc: "); Serial.print(subLoc); Serial.print("    subDir: ");  Serial.println(subDir);

    wait_BtnRelease();   // wait for no button pressed
    targCell = nleds / 2;
    depChgCnt = 0;
    playing = true;

    while (playing) {
      //sonarPing=false;
      advance = false;
      if (btnPressed()) {

        // if 'esc' button held down  ... Exit
        if (ESC) {
          delay(300);
          return;
        }

        if (btn2Pressed) targCell--;
        if (btn3Pressed) targCell++;
        if (targCell <= 0) targCell = nleds;
        if (targCell >= nleds + 1) targCell = 1;
        if (ver == 1) {
          depChrg   = btn1Pressed || btn4Pressed;
          sonarPing = depChrg;
        } else if (ver == 2) {
          depChrg   = (player == 1 && btn1Pressed) || (player == 0 && btn4Pressed);
          sonarPing = depChrg;
        } else if (ver == 3 || ver == 4) {
          depChrg   = btn1Pressed;
          sonarPing = btn4Pressed;
          //                        activate the next line if you want to limit the # of depth charges
          if (depChrg) depChgCnt++;    // only tracked for ver 3&4
        }

        if (ver == 2 && ((btn4Pressed && player == 1) || (btn1Pressed && player == 0))) {
          beep(25);
        } else {
          if (depChrg) {
            advance = true;
            // drop depth-charge
            //deChrg = 20;
            spriteTwo = litLED = targCell;
            refreshWait(250);
            boop(30);               // sound of charge
            refreshWait(250);
            spriteTwo = litLED = 0;
            if (subLoc == targCell) {
              refreshWait(200);
              spriteOne = targCell;
              refreshWait(150);
              boOop(250);            // ka ...
              spriteOne = 0;
              dim[targCell - 1] = dim[targCell + 1] = true;
              Beep(500);             // ... Bang!
              dim[targCell - 1] = dim[targCell + 1] = false;
              refreshWait(750);
              if (ver == 2) winner(player);
              refreshWait(750);
              playing = false;
            } else {  // missed
              if (depChgCnt >= depChgLimit) {
                lost();   // player has lost, give the 'Raspberry
                playing = false;
              }
            }
          }
          
          if (sonarPing && playing) {      // take sonar reading
            advance = true;
            ping = dimIt = flashLed = 0;
            n = (nleds < 8) ? 1 : (nleds < 12) ? 2 : 3;
            for (int i = 0; i <= n; i++) {
              if ((targCell - i) > 0) {
                dim[targCell - i] = true;
                if ((targCell - i) == subLoc) ping = targCell;
              }
              if ((targCell + i) < nleds + 1) {
                dim[targCell + i] = true;
                if ((targCell + i) == subLoc) ping = targCell;
              }
              refreshWait(150);
              if (ping == targCell) beep(25);
              refreshWait(150);
              if ((targCell - i) > 0) dim[targCell - i] = false;
              if ((targCell + i) < nleds + 1) dim[targCell + i] = false;
              ping = litLED = flashLed = 0;
            }
          }
        }
        if (advance) {
          player = 1 - player;
          // the sub moves after being poked at
          subLoc = subLoc + subDir;
          if (subLoc < 1)  subLoc += nleds;
          if (subLoc > nleds) subLoc -= nleds;
          //Serial.print("subLoc: "); Serial.println(subLoc);
        }
      }

      if (ESC || (btn4 && btn1)) {
        delay(300);
        return;
      }
      //================ UPDATE Display
      spriteOne = spriteTwo = dimIt = litLED = flashLed = 0;

      //      spriteTwo = subLoc; // do this to debug
      flashLed = targCell;
      //dimLED = dimIt = targCell;   // too soften the flash
      litLED = dimIt = targCell;   // too soften the flash (removed dim+dimIt due to flicker)
      refreshWait(1);

    }
  }
}

// =====================================================================
void BiFunc() {  //  Binary Operator Functions
  int i, csum;
  int testCell, targCell;
  byte opr1, opr2, results;

  loopCnt = 0;

  ver = getOption(4);  // Funcunctions 1: AND   2: OR   3: XOR   4: ADD
  if (ver == 0) return;
  clearDisp();

  refreshWait(500);
  while (1) {
    // setup game-level
    ledsAll(0); refreshDisp();  // clear all 'lit[]' leds

    wait_BtnRelease();   // wait for no button pressed

    targCell = 4;
    playing = true;
    opr1 = opr2 = 0;
    while (playing) {
      if (btnPressed()) {
        if (btn2Pressed) targCell--;
        if (btn3Pressed) targCell++;
        if (targCell <= 0) targCell = 4;
        if (targCell >= 5) targCell = 1;
        if (btn2Pressed || btn3Pressed) loopCnt=0; // causes flash to indicate new pos.
        if (btn1Pressed) {
          opr1 = opr1 ^ (1 << (4 - targCell));
        }
        if (btn4Pressed) {
          if (opr1 == 0 || opr1 == 0xF) { // initial opr1, used by the player must != 0, 15
            beep(20);
            continue;
          }
          if (opr2 == 0) { // if we have no opr2 yet, pick one that != opr1
            while (opr2 == 0 || opr2 == opr1) opr2 = 1 + random(14);
          }
          switch (ver) {
            case 1: results = opr1 & opr2;  break;
            case 2: results = opr1 | opr2;  break;
            case 3: results = opr1 ^ opr2;  break;
            case 4: results = opr1 + opr2;  break;
          }
          //Serial.print("opr1: "); Serial.print(opr1); Serial.print("  opr2: "); Serial.print(opr2);
          //Serial.print("   results: "); Serial.println(results);


          if (opr1 == opr2) { // you got it
            winner(1);
            refreshWait(500);
            playing = false;
          } else {
            buZZ(150);  //boo
            targCell = 0;
            // show the Bi Operation results
            opr1 = (results & 0x0F);
          }
        }
      }

      //================ UPDATE Display
      loopCnt++;
      dimLED = litLED = 0;

      for (i = 0; i < 4; i++) {
        colorLeds[4 - i] = (opr1 >> i) & 1;
      }
      if ((loopCnt % 811) < 55) { // creats cursor effect
        colorLeds[targCell] = !colorLeds[targCell];
      }

      dspColors();  // takes up 1 msec
      refreshWait(1); // keep from scanning btns too fast

      if (ESC || (btn1 && btn4)) {
        buZZ(30);
        refreshWait(300);
        return;
      }
    }
  }
}

void Flip_d() {   // One dimentional, "Reversi" game. Reminiscent of 'Go'
  int targs, testCell, targCell;
  int csum;
  int lastCell, aiMove, hisFirstLast, hisLastFirst, myLastFirst, mySplit;
  byte r, x, myGrps, hisGrps, setSize;
  byte color, moves;

  loopCnt = 0;
  ver = getOption(2);  // version 1: player 1 vs Computer challenge
  // version 2: two person
  if (ver == 0) return;
  clearDisp(); refreshWait(1000);
  player = 0;
  while (1) {
    // setup game-level
    //clearDisp(); refreshDisp();
    clearDisp();

    for (int i = 1; i <= nleds / 2; i++) { // select half the leds, randomly
      r = random(nleds) + 1;
      while (dim[r]) {
        r = random(nleds) + 1;
      }
      dim[r] = true;
    }
    wait_BtnRelease();   // wait for no button pressed

    aiMove = 0;
    targCell = nleds / 2;
    playing = true;
    moves = 0;
    if (ver != 2) player = 0;
    while (playing) {
      idleCnt = 0; // defeat idle timeout while playing this comtemplated game
      if (btnPressed() || aiMove) {
        if (ESC) return;
        if (btn2Pressed) targCell--;
        if (btn3Pressed) targCell++;
        if (targCell <= 0) targCell = nleds;
        if (targCell >= (nleds + 1)) targCell = 1;
        if (ver == 1 && moves == 0 && btn4Pressed && player == 0) {
          player = 1;         // let the AI go first
          btn4 = false; //stop further btn event processing
        }
        if ((btn1Pressed && player == 1) || (btn4Pressed && player == 0)) {
          beep(30);
        }
        else if (btn1Pressed || btn4Pressed || aiMove)
        { // -------------------  make their move
          if (dim[targCell] != player) { // convert the other guys set to your side
            testCell = targCell;
            while (dim[testCell] != player && testCell <= nleds) {
              dim[testCell] = player;
              testCell++;
            }
            testCell = targCell - 1;
            while (dim[testCell] != player && testCell > 0) {
              dim[testCell] = player;
              testCell--;
            }
          } else {  // it is one of yours to be given up
            dim[targCell] = 1 - player;
          }
          csum = 0;
          for (int i = 1; i <= nleds; i++) {
            if (dim[i]) csum ++;
          }
          if (csum == 0 || csum == nleds) { // game over
            color = (csum == 0) ? 1 : 4;
            // acknowledge the "Win-ner!"
            beep(200);
            //ledsAll(0);
            //refreshWait(75);
            refreshWait(75);
            setColor(color); beep(500);

            setColor(0);
            refreshWait(200);
            playing = false;
          }
          moves++;
          player = 1 - player;

          dimFlash = litLED = 0;
          boop(20);
          refreshWait(750); // for viewing new state
        }

        //  ------------------------- support playing against the Computer
        lastCell = -1;
        hisGrps = myGrps = 0;
        aiMove = hisFirstLast = hisLastFirst = myLastFirst = mySplit = 0;
        if (playing && ver != 2 && player == 1) { // AI looks for a move
          for (int i = 1; i <= nleds; i++) { // tally the board position
            x = dim[i];
            if (x != lastCell) {
              if (x) myGrps++; else hisGrps++;
              if (x == 1 && hisFirstLast == 0) hisFirstLast = i - 1;
              if (x == 0) hisLastFirst = i;
              if (x == 1) myLastFirst = i;
              setSize = 1;
            } else {
              setSize++;
              if (x && setSize == 3) mySplit = i - 1;
            }
            lastCell = x;
          }
          // -------------- decide on move
          if (hisGrps == 1) aiMove = hisLastFirst;
          else if (myGrps == 1) aiMove = mySplit;
          else if (hisGrps == 2) aiMove = (dim[1] == 0) ? hisFirstLast : aiMove = hisLastFirst;
          else if (myGrps == 2) aiMove = (mySplit) ? mySplit : myLastFirst;
          else if (hisGrps == 3) aiMove = (dim[1] == 0) ? hisFirstLast : aiMove = hisLastFirst;
          else if (myGrps < hisGrps) aiMove = (mySplit) ? mySplit : myLastFirst;
          else {
            if (dim[1] == 0) aiMove = hisFirstLast;
            if (dim[nleds] == 0) aiMove = hisLastFirst;
          }
          if (aiMove == 0)  aiMove = random(nleds) + 1;
        }
      }


      //================ UPDATE Display
      //flashLed = (player==1)? targCell:0 ;
      loopCnt++;
      dimFlash = litLED = 0;
      if (player == 0) {
        if ((msCnt/100)%2==1) dimFlash = targCell; // 1/2 the effect of dimFlash
      }
      if  (player != 0 && loopCnt < 350) {
        litLED = targCell;
      }
      if (loopCnt > 600) loopCnt = 0;

      //refreshDisp();
      refreshWait(1);
      if (aiMove) {
        dimFlash = litLED = 0;
        refreshWait(1000);
        targCell = aiMove;
        flashLed = aiMove;
        refreshWait(1500);
        flashLed = 0;
      }

      // if 'esc' button held down or Fnc-1 ... Exit
      if (ESC) {
        refreshWait(300);
        return;
      }
    }
  }
}

// =====================================================================
void Spray() {
  byte  litCnt, fullP;
  short int paint, aim, msDelay;
  bool sprayIt;
  byte newGame, leadPlayer;
  int i;
  bool pressed;
  byte levelMax;

  byte player;

  clearDisp();

  levelMax = 10;
  loopCnt = 0;
  player = leadPlayer = 1;
  dir = 1;
  aim = -3;
  fullP = nleds / 2; // std units of paint

  ledsAll(0); refreshWait(500);
  paint = fullP + 1; // initial an extra unit of paint
  while (1) {
    // setup game
    level = 1;
    ledsAll(0); refreshDisp();
    loopCnt = 0;
    sprayIt = false;
    newGame = false;
    if (aim != -3) { // if not initial round
      paint = -1;
      newGame = true;
      if (dir == 1) {
        aim = nleds + 3;
        leadPlayer = 2;
      } else {
        aim = -2;
        leadPlayer = 1;
      }
    }

    wait_BtnRelease();   // wait for no button pressed
    refreshDisp();

    playing = true;
    while (playing) {

      //================ Advance Game Status
      loopCnt++;
      msDelay = (level <= levelMax / 2) ? 125 - 12 * level : 100 - 8 * level;
      if ((loopCnt % msDelay) == 0) {

        // ------ Is the game over ?
        litCnt = 0;
        for (i = 1; i < nleds + 1; i++) {
          litCnt += dim[i];
        }
        if ((paint != -1 && paint < fullP) && (litLED == aim) && ((player == 1 && litCnt == nleds) || (player == 2 && litCnt == 0))) { // ? all one color
          // winner
          winner(player);
          refreshWait(1000);
          playing = false;
          newGame = true;
        }

        aim = aim + dir;
        if ((aim <= -2) || (aim >= nleds + 3)) { // at the end, turn around
          dir = (aim <= -2) ? 1 : -1;
          if (paint < fullP) { // done his thing (paints been used), switch players
            paint = (paint < 0) ? fullP + 1 : fullP;
            player = 3 - player;
            if (player == leadPlayer) {
              level++;
              if (level > levelMax) level = 1;
            }
            refreshWait(330);
            spriteOne = (player == 1) ? 1 : nleds;
            beep(15);
            refreshWait(150);
            spriteOne = 0;
            refreshWait(330);
            if (newGame) {
              newGame = false;
              for (int n = 1; n <= nleds; n++) dim[n] = (player != 1);
            }
          }
        }
      }

      //================ UPDATE Display
      if (aim > 0 && aim < nleds + 1)  litLED = aim;
      else litLED = 0;
      refreshWait(1);

      //================ Process Keys
      if (btnPressed() || (loopCnt % msDelay) == 0) {
        // qualify button press,  p1 btn1, p2 btn4 or btn0
        pressed = ((player == 1) && btn1) || ((player == 2) && btn4);

        sprayIt = (paint && pressed);

        if (sprayIt) {
          paint--;
          if (aim > 0 && aim < nleds + 1) {
            // see if spot not already painted by him
            if (dim[aim] != (player == 1)) {
              dim[aim] = (player == 1);
            } else {
              if (paint == (fullP - 1)) beep(20); // boo, they missed right off
              paint = 0;
            }
          }
        }
      }

      if (btn3Pressed) {    // step up the level
        level = (level >= levelMax) ? 1 : level + 1;
        announceLevel(level);
      }
      if (ESC || (btn1 && btn4)) {
        refreshWait(300);
        return;
      }
    }
  }
}


//=============================================================
// mostly from Menu16Games Set #2
// ************************  Group 2  *******************************

// =====================================================================
void pingPong() { // Ping-Pong - Two player Game
#define STARTPERIOD 330;
  int litPeriod, period;
  int cnt = 0, maxCnt;
  int rally = 0;
  int dir = 1;
  byte color = 0; // 1 for left side, 4 for right
  bool isLit = false;

  bool lastSwung, swing = false;
  bool lastPosHit, hit = false;
  bool missed = false;
  int ver;

  ver = 2;  // for left:Btn1 / right:btn4
  //  ver = getOption(2); // version 1: use either Btn1 or 4 good for one player
  // version 2: for 2 players P1 must use Btn1, P2: btn4
  litPeriod = STARTPERIOD;
  period = litPeriod;
  maxCnt = nleds - 1;
  t0 = millis();

  while (1) {
    lastSwung = swing;
    scanBtns();
    if (ESC) return;
    swing =  btn1Pressed || btn4Pressed;
    if (ver == 2) {
      if (dir == 1) swing = btn4;
      else        swing = btn1;
    }
    hit = hit || swing;
    if (btn3Pressed) {    // step up the speed/level
      rally = rally + 1;
      litPeriod = STARTPERIOD;
      for (int i = 1; i < rally; i++) litPeriod = 9 * litPeriod / 10;
      cnt = (dir == 1) ? 0 : nleds - 1; // restart the rally
      isLit = false;
      //announceLevel(rally/2);
    }

    if ((millis() - t0) >= period) {  // is this time period up ?
      if (isLit) {                       // about to go from LED on to OFF
        period = litPeriod / 3;
        if (1) {
          missed = false;
          if (cnt >= maxCnt || cnt <= 0) { // ------ end of the line
            //litPeriod = 9 * litPeriod / 10;
            litPeriod = STARTPERIOD;
            for (int i = 1; i < rally; i++) {
              litPeriod = 9 * litPeriod / 10;   // * .9
            }

            //        !!!!!!!!!!!!!!!!  make judgement
            if (hit && !lastPosHit) {
              // -------------------  he hit it
              litLED = cnt + 1;
              //refreshWait(40);
              bop(50);  // paddle hitting ball
              litLED = 0;
              hit = false;
              rally++;
              if (cnt <= 0) dir = 1; else  dir = -1;
            } else {  // ------------  failed to hit it
              litLED = 0;
              clearDisp();
              color = (dir > 0) ? 1 : 4;
              // acknowledge the "Win-ner!"
              if (rally < 1 && !hit) {
                if (color == 1) {
                  color = 1;
                }
                else          {
                  color = 4;
                }
              } else {
                if (color == 1) {
                  brt[1] = 1;
                  beep(150);
                  refreshWait(100);
                  brt[2] = 1;
                  beep(500);
                }
                else          {
                  brt[nleds - 1] = 1;
                  beep(150);
                  refreshWait(100);
                  brt[nleds] = 1;
                  beep(500);
                }
              }

              missed = true;
              if (missed) {
                color = 0;
                refreshWait(1000);
                if (rally >= 2) {
                  showLevel(rally / 2); // give em one star for each set of back & forth
                  refreshWait(2000);
                  brightOne = 0;
                  refreshWait(200);
                }

                // re-init
                litPeriod = STARTPERIOD;
                period = litPeriod;
                rally = 0;

                // change direction
                if (color == 1) {
                  dir = 1;
                  cnt = 0;
                } else {
                  dir = -1;
                  cnt = nleds - 1;
                }
              }
              clearDisp();
            }
          } // end time to judge
        }

      } else {                     // about to go from LED off to ON
        period = litPeriod;
        lastPosHit = hit;
        hit = false;
        cnt = cnt + dir;  // goto NEXT cell
      }
      t0 = millis();
      isLit = not isLit;
    }

    //-----------  UPDATE Display
    for (int i = 1; i <= nleds; i++)  dim[i] = 0;
    dim[cnt + 1] = 1;
    if (color) showColor(color);
    refreshDisp();
    delay(1);
    if (ESC) return;
  }
}

void QuickDraw() {
  int ret, who;
  bool playing;

  clearDisp(); refreshWait(750);
  setColor(5); refreshWait(500);    // indicates we have selected the game

  while (1) {
    // setup game-level
    setColor(0); refreshWait(1000);

    playing = true;
    while (playing) {

      while (!btnPressed()) {   // pause till player gives go ahead, with two Leds dim flashing
        dimFlash = nleds - (msCnt % 2);
        refreshWait(1);
      }
      dimFlash = 0;

      while (Btn) {    // wait for NO button pressed
        scanBtns();
        refreshWait(1);
        if (ESC) return;
      }

      refreshWait(700);
      ret = readySetGo(2000); // There is a Random time between yellow and 'Draw!' green
      t0 = millis();  // start timing on  Button pushed
      if (ret == 0) {
        while (!btnPressed()) {}  // Wait for the First press
        t2 = millis() - t0;
        Serial.print("Reaction time = ");  Serial.print(t2); Serial.println("ms");
        if (btn2) {
          refreshWait(500);
          if (t2 > 500) {
            Beep(50);
            break;
          }
          showLevel(t2 / 20); // show 1 LED for each 20ms
          refreshWait(2000);
          clearDisp();
        }
        // if 'esc' button held down or Fnc-1 ... Exit
        if (ESC || btn3) {
          refreshWait(300);
          btn3 = 0;
          return;
        }

        if (btn1Pressed && btn4Pressed) { // TIE
          winner(-1);
        }
        if (btn1Pressed || btn4Pressed) { // we have a winner
          winner(btn1Pressed);
        }

      } else {        // there has been a false start, shame on him
        who = (btn1) ? 1 : (btn4 ? 4 : 0);
        setColor(who); beep(200);
        setColor(0);
        scanBtnWait(1000);   // then try again
      }
    }
  }
}

// =====================================================================
void Tug_a_War() {
  int ret;
  int player1Cnt, player2Cnt;
  int winLen, bowLoc, loc;  // center Bow location

  //byte ver = getOption(2);  // version 1: could be Fixed time between Set ... Go!
  // version 2: could be Random time between yellow and green
  clearDisp(); refreshWait(750);
  setColor(5); refreshWait(500);    // indicates we have selected the game

  while (1) {
    // setup game-level
    winLen = (2 * (nleds / 2 + 2));
    clearDisp(); refreshWait(1000);

    while (!btnPressed()) {   // pause till player gives go ahead, with two Leds dim flashing
      dimFlash = nleds - (msCnt % 2);
      refreshWait(1);
    }
    dimFlash = 0;

    while (btnDown()) {
      refreshDisp();  // wait for NO button pressed
      delay(1);
      if (ESC) return;
    }
    refreshWait(700);
    playing = true;
    while (readySetGo(400)) ;  // false start returns 1 else 0
    refreshWait(250);  // allows the 'Go' green to be seen
    // and allow players to get going, so we are measuring their rate of button pressing not their reaction time
    clearDisp();
    player1Cnt = player2Cnt = 0;
    while (playing) {

      scanBtns();
      if (ESC) {
        delay(300);
        return;
      }

      if (btn1Pressed) {
        player1Cnt++;
        if ((player1Cnt - player2Cnt) >= winLen) {
          winner(1);
          break;
        }
      }
      if (btn4Pressed) {
        player2Cnt++;
        if ((player2Cnt - player1Cnt) >= winLen) {
          winner(2);
          break;
        }
      }

      //================ UPDATE Display
      if (btnPressed) clearDisp(); // clear all leds light settings

      bowLoc = (nleds / 2) + (player2Cnt - player1Cnt) / 2; // calculate the Location of the Bow tied on the line
      loc = bowLoc - 1; if (0 < loc && loc <= nleds) dim[loc] = true;
      loc = bowLoc;   if (0 < loc && loc <= nleds) spriteOne = loc;
      loc = bowLoc + 1; if (0 < loc && loc <= nleds) spriteTwo = loc;
      loc = bowLoc + 2; if (0 < loc && loc <= nleds) dim[loc] = true;

      refreshWait(1);  // BTW: the use of (2) creates a race condition where a fraction of the time
      // with RP2040 dim[] LED updates are ineffective ??!! ...resolved with a patch
    }
    if (ESC) {
      delay(300);
      return;
    }
  }
}

// =====================================================================
void Chicken() {
  int ret;
  int p1Loc, p2Loc, p1jumped, p2jumped;
  int winningPlayer;
#define MOVE_FREQ 250
  int move_freq, moveCnt;
  bool ledFlip;

  byte ver = 1;
  //byte ver = getOption(2);  // version 1:
  // version 2:
  move_freq = MOVE_FREQ;
  // indicates we have selected this game
  for (int i = 0; i < nleds; i++) {
    dim[i + 1] = true; dim[nleds - i] = true;
    DispScanWait(move_freq / 2);
    dim[i + 1] = false; dim[nleds - i] = false;
  }

  while (1) {

    scanBtns();
    while (btn4) {
      scanBtns();
      refreshWait(1);
      if (ESC) return;  // only exit opertunity
    }

    while (!(btn1 && btn4)) {   // till both players press down, dim flashing prompt LEDs
      scanBtns();
      ledFlip = ((msCnt / (2 * move_freq)) % 2) == 0;
      dim[1] = ledFlip;
      dim[nleds] = !ledFlip;
      spriteOne = btn1;
      spriteTwo = (btn4) ? nleds : 0;
      refreshWait(1);
      if (btn3Pressed) {                // at this time press Btn3 to increase &
        move_freq = 4 * move_freq / 5;              // recalculate speed
        bop(30);
      }
    }

    // setup game-level
    loopCnt = 0;
    p1Loc = p2Loc = 0;
    p1jumped = p2jumped = 0;
    winningPlayer = 0;

    bop(100);
    refreshWait(random(150, 350)); // ready state to be watched
    clearDisp();

    moveCnt = 0;
    playing = true;
    while (playing) {
      loopCnt++;
      moveCnt = (moveCnt + 1) % move_freq;

      if (p1Loc > 0) {
        if (!btn1 && p1jumped == 0) { // if released log it
          p1jumped = loopCnt;
        }
        if (moveCnt == 0) p1Loc++;  // move it ahead
        if (p1Loc > nleds) winningPlayer = 1; // he reach the other end
      } else {
        // if btnpressed start blip on its way
        if (btn1) p1Loc = 1;
      }

      if (p2Loc > 0) {
        if (!btn4 && p2jumped == 0) { // if released log it
          p2jumped = loopCnt;
        }
        if (moveCnt == 0) p2Loc--;
        if (p2Loc == 0) winningPlayer = 2;
      } else {
        if (btn4) p2Loc = nleds; // if btnpressed start blip on its way
      }

      // display update
      clearDisp();   // clear out old states
      spriteOne = p1Loc;     //  note an 'litLED' and a 'dimLed'+'bonus' are the same brightness
      //dim[p2Loc]=true; bonus=p2Loc;
      spriteTwo = p2Loc;

      refreshDisp();

      // if blips collide or both released
      if ((p1Loc >= p2Loc && p2Loc != 0) || (p1jumped && p2jumped)) { // if blips collide or both released
        //Serial.print(p1jumped); Serial.print("  "); Serial.println(p2jumped);
        if (p1jumped == p2jumped) winningPlayer = -1; // its a tie
        else winningPlayer = (p1jumped > p2jumped) ? 1 : 2; // its over. Who won ?
      }
      if (winningPlayer != 0) winner(winningPlayer); // game will restart

      refreshWait(1);
      scanBtns();

      //  not good here     if (ESC) return;
      if (btn1) escCnt = msCnt; // reset cnt for ESC, so B4 must be held alone
    }
  }

}

// =====================================================================
void Hot_Hands() {
  int ret;
  int ledSet;
  int p1Trys, p2Trys;
  int player = 1;
  int who;
  bool escGo;

  clearDisp(); refreshWait(750);
  ledsAll(15); refreshWait(500);    // indicates we have selected the game

  byte ver = 1;
  //  byte ver = getOption(2);  // version 1: They take turns trying to slap the other
  //TBD: if desired:
  // version 2: Free for all. Either can slap at will
  
  while (1) {
    // indicate who's turn it is to try and slap it to the other
    who = (player == 1) ? 1 : 8;
    ledsAll(who); DispScanWait(500);
    // setup game-level
    ledsAll(0); DispScanWait(1000);

    while (btnPressed()) {
      refreshDisp();  // wait for NO button pressed
      delay(1);
    }

    playing = true;
    while (playing) {

      while (!(btn1 && btn4)) {
        scanBtns();
        if (ESC) {  // double how long before b4->Exit
          if (escGo) return;
          escGo = true;
          escCnt = msCnt;
        }
        ledSet = (btn1) ? 1 : 0;
        ledSet = (btn4) ? 8 : ledSet;
        ledsAll(ledSet);
        refreshDisp();
        delay(1);
      }
      escGo = false;
      ledsAll(9); // both Red & Blue leds on

      while (btn1 && btn4) { // Wait for the First release
        refreshWait(1);
        scanBtns();
      }
      if ((player == 1 && !btn4) || (player == 0 && !btn1 )) { // wrong player released first
        beep(25);
      } else {
        p1Trys = 0; p2Trys = 0;
        while (playing) {
          if (! (btn1 || btn4)) {  // if both have released, the attempt has Failed
            ledsAll(0);  refreshDisp();
            buZZ(100);
            playing = false;
            refreshWait(500);
          }

          if (btn1Pressed) p1Trys++;
          if (btn4Pressed) p2Trys++;

          if (p1Trys > 1) {
            // if the other player's button is still down they Win
            if (btn4) winner(1);  // resets 'playing'
            else      winner(2);
          }

          if (p2Trys > 1) {
            if (btn1) winner(2);
            else      winner(1);
          }

          // update display
          ledSet = (btn1) ? 1 : 0;
          ledSet = (btn4) ? ledSet + 8 : ledSet;
          ledsAll(ledSet);
          refreshDisp();
          delay(1);
          scanBtns();
          // to 'esc' double how long before b4->Exit  see above
        }
      }
    }
    player = 1 - player;
    ledsAll(0); refreshWait(1000);
  }
}

// =====================================================================
void PIG() {    // 'PIG' basketball,  3 misses and you're done
  int player = 0;
  int csum;
  byte shot, spot, dribbleCnt, score, pigCnt;
  int travel, rate, thrust;
  bool inAir, newPressed, released, btnDown;
  unsigned int lastLCnt;
  byte targCell;

  loopCnt = 0;

  ver = getOption(2);  // version 1: rotate shot 1,2,3   count how many made before U have 'PIG'
  // version 2: 2 player, player picks shot, first to 'PIG' loses
  if (ver == 0) return;

  clearDisp(); refreshWait(500);
  targCell = nleds - 1;


  while (1) {
    // setup game-level
    wait_BtnRelease();   // wait for no button pressed
    shot = 0;
    spot = 1;
    score = pigCnt = 0;

    playing = true;
    while (playing) {

      clearDisp(); refreshDisp();
      shot = shot % 3 + 1; // 1= the first spot is the 3pt. line, 2= free throw, 3= lay-up
      // w/12LEDS:      if (ver==1) spot = shot+shot-1;
      if (ver == 1) spot = shot;
      turn = shot;
      dim[targCell] = true;
      dimFlash = spot;

      dribbleCnt = 0;
      lastLCnt = 0;
      travel = 0;
      thrust = 0;
      inAir = false;
      turn = shot;
      while (turn) {
        btnPressed();
        //       btnDown = (Btn || btn1 || btn4); old backwards support of 'Btn'
        btnDown = (btn1 || btn4);
        if (btnDown && !inAir) {
          if (BtnPressed) {
            dimFlash = 0;
            dribbleCnt++;
            if (ver == 1 && shot == 3 && dribbleCnt > 1) { // coming in for a lay-up
              spot++;
              if (lastLCnt > 0) thrust += 40000 / (loopCnt - lastLCnt); // add momentum to thrust
              lastLCnt = loopCnt;
            }
          }

          if (dribbleCnt < 3 || shot == 3) {
            brightOne = spot;
            if (BtnPressed) beepTone(170,50);  // bounce the ball
          }
          if (dribbleCnt == 3) thrust++;
        } else {
          brightOne = 0;
        }

        released = (!btnDown) && BtnChanged;
        if (released && dribbleCnt >= 3) { // the ball takes flight
          inAir = true;
          //          Serial.print("thrust: "); Serial.println(thrust);
          //         rate=(thrust+1000)/20;  //for thrust=1000 gives 100 which results in 10 spots in one second
          rate = (thrust + 750) / 15; //for thrust=7500 gives 100 which results in 10 spots in one second
        }

        if (inAir && (loopCnt % 200) == 0) {
          if (thrust <= 30) { // end of ball flight
            refreshWait(150);
            if (litLED == nleds - 1) {
              winner(1);
              playing = true; // reset, due to winner()
              score++;
              //              continue; // seems to be a bug in compiler, this went to the else clause, Boo-Hiss
            } else { //missed
              beep(20);
              pigCnt++;
            }
            turn = 0;
            refreshWait(500);
            continue;
          }

          travel = travel + rate;
          thrust -= rate;

          litLED = spot + travel / 100;
          if (litLED > nleds) litLED = (2 * nleds) - litLED;
          if (litLED < 1) thrust = 0;
        }

        if (pigCnt >= 3 && ver == 1) { // =3 for PIG  would-be  =5 for HORSE
          showColor(5); refreshWait(100);
          clearDisp();  refreshWait(600);
          if (score > 0) {
            boOop(50);
            showLevel(score);
            refreshWait(2000);
          }
          else buZZ(150, 500); // & with no baskets made!
          playing = false;
          break;  // from this turn
        }

        //================ UPDATE Display
        loopCnt++;
        if (dribbleCnt == 0) {
          dimFlash = spot;
          if (ver == 2 && btn2Pressed) {
            if (--spot < 1) spot = 1;
            shot = 0;
          } else if (ver == 2 && btn3Pressed) {
            if (++spot > nleds - 2) spot = nleds - 2;
            shot = 0;
          }
          dimFlash = spot;
        }

        refreshDisp();
        delay(1);

        // if 'esc' button held down or Fnc-1 ... Exit
        if (ESC || (btn4 && btn1)) {
          delay(300);
          return;
        }
      }
    }
  }
}

void Joust() {
  // ====================================     Shoot Out
  //                When he's in your sights ... fire!, Competitively with two players
  // Challenge...
  // hit with btn2, hold till all dim, switch to btn3, hold till all are off, & release leaving no dims
  byte hits1, hits4;
  bool faceoff, flip, doingChallenge;
  byte speed;

  spriteOne = 1;
  spriteTwo = nleds;
  loopCnt = 0;
  hits1 = hits4 = 0;
  faceoff = flip = false;
  doingChallenge = false;
  speed = 1;

  while (1) {
    loopCnt++;
    refreshWait(1);
    scanBtns();
    if (ESC) break;

    if (loopCnt % (600 - speed * 125) == 0) {   // consider influence over time, at a given cadence
      spriteOne++; // ++ go right
      spriteTwo--; // -- go left
      if (spriteOne > nleds)  spriteOne = 0;
      if (spriteTwo < 0)      spriteTwo = nleds;
      if (spriteOne == 0 && random(4) != 0) spriteOne = 1; // 1/4 of the time Skip!
      if (spriteTwo == 0 && random(4) != 0) spriteTwo = nleds;
      // fidget support ..
      //          if (btn2) dim[spriteOne] = (flip)? !dim[spriteOne] : 0;
      //          if (btn3) dim[spriteTwo] = (flip)? !dim[spriteTwo] : 1;
      if (btn2 && flip) dim[spriteOne] = !dim[spriteOne];
      if (btn3 && flip) dim[spriteTwo] = !dim[spriteTwo];
    }

    if (spriteTwo == spriteOne || spriteTwo == spriteOne + 1) { // opponents are face-to-face or on the same spot
      faceoff = true;
      if (btn1Pressed && !btn4) {
        spriteTwo = 1;
        beep(120);
        hits1++;
      } else if (btn4Pressed && !btn1) {
        spriteTwo = nleds;
        beep(120);
        hits4++;
      }
      if (hits1 == 3 || hits4 == 3) { // achieved the Wining # of hits
        beep(60); beep(60); beep(60); refreshWait(500);
        winner(hits1 == 3);
        refreshWait(1000);
        break;
      }
    } else {    // check to penalize miss placed shots
      faceoff = false;
      if (btn1Pressed && hits1 > 0) hits1--;
      if (btn4Pressed && hits4 > 0) hits4--;
    }

    // support fidget challenge to transform background
    if (btn1Pressed) doingChallenge = false;
    if (btn2Pressed || btn3Pressed) doingChallenge = true;
    if (btn2Pressed && faceoff) flip = !flip; // set flip=true only if it is not already
    if (btn3Pressed && faceoff) flip = !flip; // so must release one before the other can enable it
    if (!(btn2 || btn3)) flip = false;
    if (btn4Pressed && doingChallenge) {
      for (int i = 1; i <= nleds; i++) dim[i] = 0; // clear background
      speed = (speed + 1) % 5; // shift speed
    }
  }
}

// =====================================================================
void UpDownSide() { // Upside-Downside
  // land a ball in each posible "target" spot to flip it
  // to Up(on) for the Left side or Down (0ff) for the right side.
  // hold Btn1 (player1) / Btn4 (player 2) to swing, and let go to throw
  int player = 1;
  int i, cnt, r;
  byte spot, score, btn, target;
  int travel, rate, thrust, dTrv;
  bool rolling, newPressed, released, btnDown;
  byte ver, pass;
  bool setOn;
  bool turn;
  byte missed[2 + 1];

  ver = getOption(2);
  if (ver == 0) return;
  level = 1;
  missed[1] = 0; missed[2] = 0;

  clearDisp();
  refreshWait(500);

  while (1) {
    // setup game-level
    wait_BtnRelease();   // wait for no button pressed
    score = 0;
    loopCnt = 0;
    clearDisp();
    //for (i=1; i<=nleds; i++) dim[i]=i%2;  // lite every other
    //    dim[r=random(nleds/2)+1]=1;   dim[r+nleds/2]=1;  // lite random 4 symeterically
    //    dim[r=(r+random(nleds/2-1))%(nleds/2)+1]=1;   dim[r+nleds/2]=1;
    dim[r = random(nleds / 2) + 1] = 1;    // lite random 2 of 4
    dim[r = (r + random(nleds / 2 - 1)) % (nleds / 2) + 1] = 1;
    if (random(2)) {
      dim[5] = dim[1];  // repeat on other side
      dim[6] = dim[2];
      dim[7] = dim[3];
      dim[8] = dim[4];
    }
    else           {
      dim[5] = dim[4];  // reflect "
      dim[6] = dim[3];
      dim[7] = dim[2];
      dim[8] = dim[1];
    }

    playing = true;
    while (playing) {
      refreshWait(600);
      spot = (player == 1) ? 1 : 8;;    // where we are throwing from
      btn = (player == 1) ? 1 : 4;
      //dimFlash=spot;
      r = random(nleds);
      for (i = 1; i <= nleds; i++) { // find a target for this player
        target = (i + r) % nleds + 1;
        if (dim[target] == (player == 2)) break;
      }
      if (missed[player] && random(2)) target = missed[player]; // 1/2 the time repeat a missed target
      missed[player] = 0;
      travel = 0;
      thrust = 0;
      rolling = false;
      turn = true;   // turn in progress
      while (turn) {
        bool wasDown = btnDown;
        bool pressed = btnPressed();

        if (!rolling) {
          lit[spot] = (loopCnt % 1250) > 800 && (loopCnt % 1250) < 860;
          lit[target] = (loopCnt % 1250) > 1000;
          if (target == spot && (loopCnt % 1250) > 1200) lit[target] = 0; // reduce flash where needed
        }
        btnDown = (btnNum == btn);
        if (ESC && (player == 1 || btn1)) return; // uses ESC while its P1's turn (or b1+b4)
        if (btnDown && !rolling) {
          thrust++;
          spriteOne = spot;
        } else {
          spriteOne = 0;
        }

        released = (!btnDown) && wasDown && BtnChanged;
        if (released) {       // the ball takes flight
          rolling = true;
          lit[spot] = false;
          lit[target] = false;
          //Serial.print("thrust: "); Serial.println(thrust);
          rate = (thrust + 750) / 15; //for thrust=7500 gives 100 which results in 10 spots in one second
          rate = rate / 4; // patch so we can update more often for smoother fast motion balls
        }

        if (rolling && (loopCnt % 50) == 0) { // was %200 before patch above

          if (thrust <= 30) {         // coming to a stop
            //Serial.print(" pass "); Serial.print(pass);
            //Serial.print("   LED "); Serial.println(litLED);
            if (target == litLED && ver == pass) { // they hit it !
              setOn = !dim[litLED];
              dim[litLED] = setOn;
              if (setOn) beep(70);
              else bop(90);
            } else {
              missed[player] = target;
              buZZ(20);
            }

            cnt = 0;
            for (i = 1; i <= nleds; i++) cnt += dim[i];
            if (cnt == 0 || cnt == nleds) { // game over
              winner(cnt != 0);
              playing = false; // reset, due to winner()
            }

            turn = false;
            refreshWait(1000);
            litLED = 0;
            continue;
          }

          travel = travel + rate;
          thrust -= rate;
          dTrv = travel / 100;
          spot = (dTrv % nleds) + 1;
          if (player != 1) spot = (nleds + 1) - spot;
          litLED = spot;

          pass = 1;
          if (dTrv >= nleds) {
            pass = 2;
          }
        }

        //================ UPDATE Display
        loopCnt++;
        if (spot == target && !rolling) loopCnt++; // extra flash where needed

        refreshWait(1);
        if (btn1 && btn4) {
          return; // both to Exit
        }
      }
      player = 3 - player; // toggles 1,2,1,2 ...
    }
  }
}

// ***************************  Group 3  *******************************
// ================= Spirted Fun and Psyche-Ometers ===========================

// --- utility for 'Whack_Mole' to light up any combination of colors of Leds
void ledsAll(byte ledColors) {
  for (int i = 0; i < 4; i++) {
    colorLeds[i + 1] = (1 == (1 & (ledColors >> i)));
  }
  dspColors();
}

void Whack_Mole() {
  byte i, rnum;
  byte mloc, numMoles;
  byte pinState = 0;
  int hitCnt, wait4it;
  int maxCnt;
  int need, twos2go, threes2go;
  bool hit;
#define MOLE colorLeds

  ver = 1;  // standard real world behavior; there is only one malet and it can only hit one mole at a time
  // an idea for a version #2 would be to allow simutaeous multiple button pusses, you could illiminate sets of moles faster.

  clearDisp();
  clearDisp(); refreshWait(500);
  ledsAll(15); refreshWait(500);    // indicates we have selected the game
  clearDisp(); refreshWait(1500);
  if (ESC) {
    refreshWait(300);
    return;
  }
  t0 = millis();
  hitCnt = 0;
  maxCnt = 1600;  // how long a set of moles are exposed (in wouldbe millisecs-ish)
  need = 0;
  refreshWait(1);
  //Serial.print("miliSec Count: ");  Serial.println(millis() - t0);
  while ((millis() - t0) < (30 * 1000)) {  // is this time period up ?
    digitalWrite(ledOnBrd, mySecs & 1); // DEBUG flash
    //Serial.print("miliSec Count: ");  Serial.println(millis() - t0);
    if (maxCnt > 400) maxCnt -= 40;  // decreases mole exposure as game goes on
    loopCnt = maxCnt;
    for (i = 0; i <= 4; i++) MOLE[i] = 0;

    if (need == 0) {
      need = nleds; // -------  need a set of nleds 'random' #'s with distribution of ~ 50% 1s, 33% 2s, and 17% 3s.
      twos2go = 4;
      threes2go = 2;
    }
    rnum = random(0, need);
    if (rnum < threes2go) {
      threes2go--;
      numMoles = 3;
    } else if (rnum < (twos2go + threes2go)) {
      twos2go--;
      numMoles = 2;
    } else {
      numMoles = 1;
    }
    need--;

    mloc = random(1, 5);
    MOLE[mloc] = 1;
    if (numMoles > 1) {
      mloc = ((mloc + random(0, 3)) % 4) + 1;
      MOLE[mloc] = 1;  // add in second mole
      if (numMoles > 2) { // add in a third mole
        mloc = ((mloc + random(0, 2)) % 4) + 1;
        if (MOLE[mloc] == 1) mloc = (mloc % 4) + 1;
        MOLE[mloc] = 1;
      }
    }

    dspColors();
    wait4it = random(50, 250); // preset wait time between sets of moles
    while (0 < loopCnt--) {
      //      dspColors();
      refreshWait(1);
      scanBtns();
      hit = false;
      if ((btn1 + btn2 + btn3 + btn4) == 1 || ver == 2) {
        if (btn1 && MOLE[1]) {
          MOLE[1] = 0;
          hit = true;
        }
        if (btn2 && MOLE[2]) {
          MOLE[2] = 0;
          hit = true;
        }
        if (btn3 && MOLE[3]) {
          MOLE[3] = 0;
          hit = true;
        }
        if (btn4 && MOLE[4]) {
          MOLE[4] = 0;
          hit = true;
        }
        pinState = 1 - pinState;
        if (!hit && ver != 2) digitalWrite (BEEPPIN, pinState); // make noise if missing, without causing delays
      } else {
        digitalWrite (BEEPPIN, LOW);
      }
      dspColors();
      if (hit) {
        hitCnt++;
        if ((MOLE[1] + MOLE[2] + MOLE[3] + MOLE[4]) == 0) {
          wait4it = 20;
          break;
        }
        if (ver == 1) wait_BtnRelease(); // to insure no double btn presses are taken
      }
      if (ESC) {
        refreshWait(300);
        return;
      }
    }
    if (ESC) {
      refreshWait(300);
      return;
    }
    digitalWrite (BEEPPIN, LOW);
    refreshWait (wait4it);
    wait_BtnRelease();  // must have malet off the moles before proceding
  }
  //Serial.print("max Count time allowed: ");  Serial.println(maxCnt);
  //Serial.print("Hit Count: ");  Serial.println(hitCnt);
  // ----------------  Show score. Show 1 led for each 10 hits
  showLevel(hitCnt / 10);
  refreshWait(2000);
  while (!btnPressed());
}

// ===================== Simon Says support subrountines ====================
void playIntro() {
  int note, dur;
  for (int thisNote = 0; thisNote < 13; thisNote ++) {
    // play the next note:
    note = starttune[thisNote];

    if (note == 8)       setColor(1);
    else if (note == 11) setColor(2);
    else if (note == 12) setColor(3);
    else if (note == 10) setColor(4);

    dur = duration[thisNote];
    playNote(note, dur);

    // stop for the next note:
    setColor(0);
    refreshWait(25);
  }
}

void checkPlayerInput() { //Function for allowing user input and checking input against the generated array
  bool btnPushed;
  byte  btn;

  for (int x = 0; x <= turn; x++)
  { // for each one in the current sequence wait for a button to be pushed
    // if it is the correct one we keep looping through the sequence

    btnPushed = false;
    while (! btnPushed) {
      refreshWait(1);
      scanBtns();
      if (ESC) {
        refreshWait(300);
        return;
      }
      btn = (btn1) ? 1 : ((btn2) ? 2 : ((btn3) ? 3 : ((btn4) ? 4 : 0))); // logical button # (1-4)

      if (BtnChanged && btn != 0) {
        btnPushed = true;
        setColor(btn);
        playNote(btnNote[btn - 1], 100);
        refreshWait(100);  // insures minimum LED illumination
        inputArray[x] = btn;
        //          Serial.print("  btn pushed: ");  Serial.print(btn);
        wait_BtnRelease();  // much better this way
        setColor(0);
        if (inputArray[x] != randomArray[x]) { //Checks value input by user and checks it against
          delay(300);
          fail();                              //the value in the same spot on the generated array
          //The fail function is called if it does not match
          return;
        }
      }
    }  // end while

  }
  refreshWait(500);

  if (ver == 1 && turn >= 3) {
    setColor(5);
    // took too long:     for (int thisNote = 0; thisNote < 6; thisNote ++) {
    for (int thisNote = 0; (thisNote < 6 && thisNote <= turn); thisNote ++) {
      // play the next note:
      playNote(cheer[thisNote], cheerDur[thisNote]);
      refreshWait(25);
    }
  }

  turn++; //Increments the turn count, also the last action before starting the output function over again
}

int getGameBtn() {
  int btn = 0;
  while (btn == 0) {
    refreshWait(1);
    scanBtns();
    if (ESC) {
      return -1;
    }
    btn = (btn1) ? 1 : ((btn2) ? 2 : ((btn3) ? 3 : ((btn4) ? 4 : 0))); // logical button # (1-4)
  }
  setColor(btn);
  playNote(btnNote[btn - 1], 100);
  wait_BtnRelease();
  setColor(0);
  if (ESC) return -1;
  return btn;
}

void fail() { //Function used if the player fails to match the sequence
  for (int y = 0; y <= 2; y++)
  { //Flashes lights for failure
    setColor(5);  // all on
    playNote(5, 300);
    delay(200);
    setColor(4);
    //playNote(1, 300);  // was
    //boOop(300);  // low freq. tone with Vibrado
    buZZ(300);   // my fav
    delay(200);
  }
  delay(400);
  if (turn > 3) { // let them know how far they got
    showLevel(turn);
    refreshWait(1500);
  }
  clearDisp();
  refreshWait(100);
  turn = 0; //Resets turn value so the game starts over without need for a reset button
}

// ========================================== Simon Says main GAME loops
void SimonSays() { // Simon Says

  ver = 1;  // version 1: normal play
  // version 2: 2 players, taking turns, they add a new lite at the end of their turn,
  //           the first to mess-up loses.
  playIntro();

  turn = 0;
  for (int level = 0; level <= 99; level++)
  {
    if (ver == 1 || turn == 0) refreshWait(1000);

    // Serial.print("\n Turn: "); Serial.print(turn);  // Some serial output to follow along
   
    // fill up the array to be matched by the player
    if (ver < 2 || turn == 0) {
      randomArray[turn] = random(1, 5); //Assigning a random number (1-4) to the randomArray[turn count]
      if (ESC) {
        refreshWait(300);
        return;
      }
    } else {
      randomArray[turn] = getGameBtn(); // let user provide
      refreshWait(500);
      if (ESC) {
        refreshWait(300);
        return;
      }
      beep(50); refreshWait(50); beep(150); // thankYou
      refreshWait(500);
    }

    for (int x = 0; x <= turn; x++)
    {
      byte btn = randomArray[x];  // logical button/led # (1-4)
      setColor(btn);
      if (ESC) {
        refreshWait(300);
        return;
      }
      playNote(btnNote[btn - 1], 100);
      refreshWait(400);
      setColor(0);
      refreshWait(100);
    }
    checkPlayerInput();
    if (ESC) {
      refreshWait(300);
      return;
    }
  }

}

//----------------------------------------------------------------------
void Speed_Scan_r4() {  // hit it on the right spot to speed things up to a winning max
  // misses slows things down, and can put you even further from winning
  int onMSecs, stRate = 150;
  int hits = 0, goal = 9;

  litLED = 1;
  onMSecs = stRate;
  while (true) {
    refreshWait(onMSecs);   // light one LED (moving blip) for a bit
    btnsNow();   // get a full state update (including debounce)
    // allow use of left or right most button
    if (BtnPressed && (litLED == 5 || litLED == 6)) {
      beep(100);
      hits++;
      if (hits == goal) {
        beep(100, 50); beep(100, 50); beep(250); // you did it
        hits = 0; // start over
        refreshWait(1000);
        clearDisp();
      }
    }

    // added for support of a missed penalty
    if (BtnPressed && !(litLED == 5 || litLED == 6)) {
      refreshWait(300);
      hits--;     // slow down update rate
      clearDisp();
    }

    onMSecs = stRate - (15 * hits); // more hits he get faster the blip scans
    for (int i = 1; i <= hits; i++) dim[i] = true; // display as, dim, the hit progress
    litLED = (litLED % nleds) + 1;    // advance to next LED
    if (ESC) break;
  }
}

void Telekinesis() {
  // ========================================    Telekinesis
  // uses ADC input, and works best with 2+ inches of wire attached to it
  unsigned int x;
  int pos, lastPos;
  bool btnOccured;
  int n = 0, cnt = 0, cntMax = 0;
  int dir[4];
  int maxDir, maxSum;
  unsigned int sum;
  int tst[4];
  bool pause;

  pos = nleds / 2;
  goal = nleds;
  clearDisp();
  btnOccured = false;
  while (1) {
    cnt = 0;
    sum = 0;
    maxSum = 0;
    tst[0] = tst[1] = tst[2] = tst[3] = 0;

    for (int i = 0; i < 4; i++) dir[i] = 0;
    while (cnt++ < 25) {    // consider influence over time
      refreshWait(20);

      x = analogRead(UNUSED_ADC); // read only every 2-5 msecs  (A0|A6 now A7 its available on ProMicro)
      //        Serial.print("Analog Val:  "); Serial.println(x);  // testing
      x += analogRead(ADC_b); // patch for greater reading

      x = (7 * x) / 11 + cnt; // adj for greater balanced results
      sum += (x % 256); // statistics keeping
      tst[x % 4]++;
      dir[x % 4]++;
      if (dir[x % 4] > maxSum) {
        maxSum = dir[x % 4];
        maxDir = x % 4;
      }
      if (btn4Pressed) pause = true;
      scanBtns();
      btnOccured = btnOccured || (BtnPressed && !btn4Pressed) ;
    }
    if (debugPrt) Serial.print("Mean: "); Serial.println(sum / 100);
    if (debugPrt) Serial.print("  4bit Cnts: "); Serial.print(tst[0]); Serial.print(" "); Serial.print(tst[1]); Serial.print(" ");
    if (debugPrt) Serial.print(tst[2]); Serial.print(" "); Serial.println(tst[3]);

    while (btn4Pressed || pause) {  // implement pause func
      if (btn4Pressed) pause = !pause;  //toogle in&out of pause
      if (!pause) {
        setColor(5);  // announce resumption
        beep(30);
        setColor(0);
      }
      scanBtns(); refreshWait(1);
      if (ESC) return;
    }

    sum = 0;
    lastPos = pos;
    if (maxDir == 0) pos++;      // go right
    else if (maxDir == 3) pos--; // go left
    if (pos < 0) pos = nleds - 1;
    pos = pos % nleds;
    spriteOne = pos + 1;
    if (lastPos != pos) {
      if (btnOccured) dim[pos + 1] = !dim[pos + 1]; // if predicted invert the Led we are now at
      btnOccured = false;
      n = 0;
      for (int i = 1; i <= nleds; i++) n += dim[i];
      if (n == goal) { // if they have set or cleared them all congradulate them
        taDah();
        goal = (goal == 0) ? nleds : 0;
      }
    }
    if (ESC) break;
    secsIdle=0; // disable idle timeout
  }
}

// ======================== ghostbuster's type P.K.E. meter
void PKE_Meter() {  // ----- psycho-kinetic scope Detecting Psychokinetic/ParaNormal Energy
  float baseline;   //a small % value (e.g. 10) used to scale RNG rating
  float PKE_level;  // displayed scaled level of PKE
  // Mathematically normal quiescent values:
  float runningMean = 127.5, runningGMean = 113.2, runningDelta = 85.33, runningBChg = 4.0;
  float rateRNG, lastRNG;  // rating RNG randomness (0.0=True Random,  1.0=totaly Not random)
  byte chgd;
  unsigned int x, lastX = 127;
  int seqDelta, geoMean;
  long bitChgSum = 0;
  bool reading;  // true while taking readings
  int cnt;
  //  int mycnt=1;

  ver = get_Mode(2); // 1: linear  2: logrithmic output display
  if (ver == 0) return;
  // start off with a display sweep to Max level & back to level 1
  for (int i = 1; i <= nleds - 1; i++) {
    dispPKE(i);
    refreshWait(120);
  }
  for (int i = nleds - 1; i >= 1; i--) {
    dispPKE(i);
    refreshWait(120);
  }

  // set level 1 = to a standard minimal % of non randomness
  baseline = 12.0 / 100.0; // set empirical (w/ 12 LEDS I used 10%?)
  lastRNG = baseline;
//  Serial.print("  Non-Randomn baseline: "); Serial.println(100 * baseline);

  // sweep from max to level 1 at start
  beep(40);
  clearDisp();
  reading = true;
  while (reading) {
    // Make Analog readings to generate a random value 0-256
    // using HRNG(2) which is about twice as solidly random as is HRNG(1),straigt ADC in.
    // and therefore less sensitive to a false NRP. I think HRNG(1) has too little signal to noise.
    // in Mega_Game proj. HRNG(2)'s randomness is software amplifies as there are no diodes on Ain
    x = HRNG(2) & 0xFF; // take an 8bit reading, was using HRNG(3)
    // HRNG(0) gives 'random()' #s with ~ constant randomness thus ~ 0 display output
    
    for (int i=0; i<25; i=i+5) { // sampling rate, has stabiity side effects (is evey 25 ms)
      refreshWait(5);
      if (PKE_level>4 && random(3)<(PKE_level-4)) digitalWrite ( BEEPPIN, random(2));  // more reading noise
    }
    
    // Determin a running randomness rate
    chgd = x ^ lastX;
    bitChgSum = 0;
    for (int i = 0; i < 8; i++) {
      bitChgSum += (1 & (chgd >> i));
    }
    seqDelta = abs((int)lastX - (int)x);
    geoMean = sqrt(x * lastX);  //   geometric mean
    lastX = x;

    // track these values used to evaluate randomness
    runningMean  = (9.0 * runningMean + x) / 10.0;
    runningGMean  = (9.0 * runningGMean + geoMean) / 10.0;
    runningDelta = (9.0 * runningDelta + seqDelta) / 10.0;
    runningBChg  = (9.0 * runningBChg + bitChgSum) / 10.0;
    // combine their deviations from their Norms
    rateRNG  = abs((runningMean / 127.5) - 1.0);
    rateRNG += abs((runningGMean / 113.2) - 1.0);
    rateRNG += abs((runningDelta / 85.33) - 1.0);
    rateRNG += abs((runningBChg / 4.0) - 1.0);
    rateRNG = rateRNG / 4; // expected range 0.0 - 1.0

    //       rateRNG = (rateRNG + 3*lastRNG) /4; // smooth it out a little
    rateRNG = (rateRNG + 7 * lastRNG) / 8; // smooth it out a little
    lastRNG = rateRNG;
    //        if (debugPrt && (cnt++ % 20)==0) {  // print Randomness evaluation numbers
    //          Serial.print("  mean: "); Serial.print(runningMean);
    //          Serial.print("  GeoMean: "); Serial.print(runningGMean);
    //          Serial.print("  Seq.Delta: "); Serial.print(runningDelta);
    //          Serial.print("  Avg Bits Chg'd: "); Serial.print(runningBChg);
    //          Serial.print("  Non-Randomn Percent: "); Serial.println(100 * rateRNG);
    //        }

    if ((myMillis / 50) % 8 == 0) {
      //  rateRNG=(++mycnt/100.);
      //  if (mycnt==100) mycnt=1;
      //  Serial.print("  Non-Randomn Percent: "); Serial.println(100 * rateRNG);
      //Serial.print("  ADC a & b: "); Serial.print(analogRead(ADC_a)); Serial.print("  "); Serial.println(analogRead(ADC_b));
    }
    //rateRNG=(mycnt/100.);

    PKE_level = rateRNG / baseline;
    if (ver == 2) {
      //            given logb(x) = ln(x)/ln(b)
      //PKE_level = log(PKE_level)/log(1.5);  // was 1.4, went to 1.5 to reduce sensitivity
      PKE_level = (nleds + 2) - log(100 * (1.0 - rateRNG)) / log(1.6); // ~ a reverse orientation of a log scale
    }

    dispPKE(PKE_level);
    if (PKE_level>1 && random(3)<(PKE_level-1)) { // make sig. noise proportional to PKE_level
      digitalWrite ( BEEPPIN, random(2));
      refreshWait(1);
      digitalWrite ( BEEPPIN, random(2));
    }
                        
    scanBtns();
    if (btn1Pressed && ver == 1) { // recalabrate
      wait_BtnRelease(); // wait for no button
      //Serial.print("  Non-Random Percent: "); Serial.println(100 * rateRNG);
      //          baseline = (baseline+rateRNG)/2;  // set level 1 halfway to current level
      baseline = rateRNG / 2; // set current level to be = display level 2
      //Serial.print("  new baseline: "); Serial.println(100*baseline);
    }
    if (btn4) return;
  }
}

// ----------- Hardware based Random Number Generator  -----------------
int HRNG(byte algo) {    // --------------------------------------------
  // 2021 update 3 -> 4 algos, with new #2 with Ain reading being Enhanced by a second reading
  int sensorValue;
  byte y, yRev, s;  // use of a byte restricts results to 8 bits (0-255)
  char c;
  unsigned int x;
#define DETECTOR ADC_a  // not sure if 2 pass defines will work
//  byte DETECTOR=ADC_a;

  //  const byte Ain = A2;  // push buttons' input line
  //  const byte Din = 16;  // the digital equivalent to the above analog pin

  // read ADC value
  // with the next line's analog read of a line with a pull-up on it, the DETECTOR line's Randomness is:
  //   Non-Randomn Percent: ~40,     while w/o it is:  ~15
  sensorValue = analogRead(DETECTOR);

  switch (algo) {
    case 0:   // ------------  use the random function  ---------------
      //  calculated Pi: 3.15   mean: 127.12   Avg Bits Chg'd: 4.00  dist: 1.07    Random Mark missed by: 0.95 %   Running avg Err: 0.94 %
      x = random(256);
      break;

    case 1:   // Use a direct Analog Read
      // calculated Pi: 3.00   mean: 130.35   Avg Bits Chg'd : 3.09  dist: 1.19    Random Mark missed by: 20.23 %   Running avg Err: 20.88 %
      x = sensorValue;  // direct Analog Read (use lower bits 8 as above, or 10 of them)
      break;

    case 2: // Amplify Ain randomness with another read, offset
      x = sensorValue;  // direct Analog Read (uses lower 8 bits) mean:140  geoMean:90  seq.D:120 avgBitsChgd:4.2  NRP:16  erratic
      // w/o external back biased diode pair attached to the Ain, following is used to amplify the randomness
      //x = (analogRead(ADC_b)+sensorValue)/2; // its quiescent levels: ~ mean:120  geoMean:115  seq.D:65  avgBitsChgd:3.3  NRP:12
      x = (x << 4) ^ analogRead(ADC_b);
      //x = (x << 4) ^ analogRead(DETECTOR);    // ~ same #s but using only one pin input

      break;

    case 3:  // reverse read bit order and add in 2 more reads
    // with + length of wire looks close to as good as case #0
    // w/ 2 readings, Pi: 3.12  mean: 127.70  Avg Bits Chg'd: 3.91  dist: 1.06    Random Mark missed by: 2.65 %  Running avg Err: 2.96 %
    case 4:  // Combine this Hardware RNG value and Pseudo SW RandomNumber
      y = sensorValue;
      //          reverse the bits
      yRev = 0;
      for (int i = 0; i <= 7; i++) {
        yRev = yRev << 1;
        yRev += y % 2; y = y >> 1;
      }

      sensorValue = analogRead(DETECTOR);
      // with double reading Avg Bits Chg'd: 3.44 -> 3.65
      sensorValue = sensorValue + analogRead(DETECTOR);  // ' '  &  improved err (-2%)

      y = yRev + sensorValue;    // Avg Bits Chg'd: 3.44   w/no wire, to 4.0 with wire
      x = y; // the byte 'y' kept results to 8 bits
      break;

  }

  if (algo == 4) { // use for best of both worlds
    y = y + random(256);
    x = y;
  }

  return (x % 1024); // x is an Integer, but in most cases only has 8 bits in it (& not >1023)
}

void dispPKE(byte level) {  // --------------------------------------------
  byte midLED;

  if (level >= nleds) {
    level = nleds;  // over-scale makes clicking sound
    beep(4);
  }
  clearDisp();
  midLED = nleds / 2;
  if (level == 0) {
    dim[midLED] = dim[midLED + 1] = true;
    return;
  }
  // set bright the level LEDs
  if (level <= midLED) {
    spriteOne = midLED + level;
    spriteTwo = midLED + 1 - level;
  } else {  // level>midLED overflows and form a trail of dim leds
    spriteOne = nleds - (level - midLED);
    spriteTwo = level - (midLED - 1);
    for (int i = spriteOne; i <= nleds; i++) dim[i] = true;
    for (int i = 1; i <= spriteTwo; i++) dim[i] = true;
  }
}

// ---------------------------------------------------------------------
void Affinity_Meter() {
  //  =============== Temperament / Affinity / Strength / Love Meter
  // Testee holds connect to ADC_a&b in one hand and grnd in the other. (internal pull-up is used)
  byte lSide, rSide;
  const float runningXMean = 512; // for running mead of read Analog value (range 0-1023);
  float ravgX;
  int lastLED;
  int x;
  float sumX;
  int n, cnt, cntMax;
  bool ackd, t;
  //  const int xvals[] = {0,26,75,119,178,246,375,525,646,742,805,853,889,1024}; // used with 12 LEDs
  // for use with 8 LEDs
  const int xvals[] =   {0, 80, 150, 220, 290, 390, 525, 750, 32000}; // focus on usefull range. w/40K pullup
  // Affinity reading of 2-4 (a person's net impedence 5-12K) are just right, not too low & not up tight .

  pinMode(ADC_b, INPUT_PULLUP); // Nano: 40K to Vcc (3.6-5v)  RP2040: 50K to 3.3v
  while (1) {
    n = cnt = cntMax = 0;
    ackd = false;
    clearDisp();
    beep(25);
    for (int i = 1; i <= nleds; i++) dim[i] = true;
    refreshWait(1000);
    lSide = 0; rSide = nleds;
    ravgX = runningXMean;
    litLED = ravgX / 100 + 2; // put it in the ball park
    //      sumX = 0.0;
    while (1) {
      if (lSide <= rSide) { // initially sweep Leds in on the 'x' pos
        if ((litLED - lSide) > (rSide - litLED)) dim[lSide++] = false;
        else dim[rSide--] = false;
        if (lSide <= rSide) litLED = 0;
        sumX = ravgX;
        n = 1;      // delays locking an average in
        cnt = 0;
      } else {
        if (cnt > 4 && !ackd) { // tell em that's it
          ackd = true;
          beep(50, 50); beep(50);
        }
      }
      for (int i = 0; i < 10; i++) { // get trending value
        brightOne = litLED;
        DispScanWait(50);  // meanwhile upd display & preserve button presses
        if (btn1 || btn4) break;
        x = analogRead(ADC_a); // reads ADC_a for inviro value 0-1023
#ifdef RP2040
       x = x >>2; // reduce to 10 bits (from 12)
       x = x + (x/5); // scale up 20% due to 50K pullup as opposed to 40K
#endif
        if (ravgX == 0.0) ravgX = x;
        ravgX = (5.0 * ravgX + x) / 6.0;
      }
      sumX += ravgX;
      n++;
      x = sumX / n;
      if (debugPrt) {Serial.print(" sumX/n = X: "); Serial.println(x);}

      litLED = 0;
      while (xvals[++litLED] < x) {}  // show current tracked value

      cnt = (litLED == lastLED) ? cnt + 1 : 0; // cnt the settling
      lastLED = litLED;

      if (debugPrt) {Serial.print(" ravgX: "); Serial.print(ravgX);}
      //        scanBtns();  // scanned above
      if (btn4) return;
      if (btn1) break;   // go again
    }
  }
}

// ==========================================  Conjure like a Magic-8-Ball or Forture teller
// Magic-8-Ball: hold down btn1 ... ponder question, let go.  Red: NO,  YEL: Maybe,  Grn: YES  Blue: Try again
// use btn2 for 1/6 out comes ~ like a dice, useful for determining one's forture or options in games.
// use btn3 for 1/8 could be used like Tarot or Horoscope card telling or a Game/Forture wheel
//
// ideas: Sound of dice rattling could be added while B2 held; after b3: the sound of fortune wheel clicker as it rolls to stop.
void Forture_Teller() {
  int cnt, maxCnt;
  int mySecs;
  byte ans, num, keyCode;
  byte priorBtn;
  static byte dicePat[] = {0x10, 0x42, 0x2A, 0xA5, 0xD3, 0xE7};
  int steps, st;
  while (1) {
    clearDisp();
    while (!Btn) {
      scanBtns();
      mySecs = msCnt / 1000;
      litLED = mySecs % 2; // toggles 0/1 per second
      if (mySecs%4>1) litLED*=nleds; // also toggles light between ends (red-grn)
      refreshWait(1);
    }
    if (ESC || btn4) break;

    t0 = millis();  // start timing on Button pushed,
    // then use human originated action Entropy to directly effect the resulting answer
    litLED = 0;
    cnt = 0; maxCnt = 200;

    while (Btn) { // ----------- wait for release
      priorBtn = btnNum;
      scanBtns();
      keyCode = priorBtn;
      if (ESC || btn4) return; // time to leave
      if (ans > 0) cnt += 2; // posible responses ... lasts 1/3 as long as dark
      if (cnt++ > maxCnt) { // produces random potenial appearing responses
        cnt = 0;  // prepare for new count with a new max
        maxCnt = (500 + 100 * random(10));
        if (keyCode>1) maxCnt=maxCnt/3;
        num = 2 + 2 * keyCode; //kc=1: 4  =2: 6  =3: 8
        ans = (ans) ? 0 : random(0, num + 1);
        if (ans == 0) clearDisp();
        else {
          if (keyCode == 1)      showColor(ans);
          else if (keyCode == 2) set_LEDs(dicePat[ans - 1], 1);
          else if (keyCode == 3) dimLED = ans;
        }
      }
      refreshWait(1);
    }

    clearDisp();
    if (keyCode == 1) {          // select one of the 4 colors
      refreshWait(400 + random(600));  // make them wait alittle
      // this also influences, randomly, the results
      ans = (millis() - t0) % 4; // answer is determined by their action
      showColor(ans + 1);
    } else if (keyCode == 2) {   // select one of the 6 colors , can be used as die for for game dice
      for (int i=1; i<3; i++) { // Roll the die
        set_LEDs(dicePat[random(0, 6)], 1);
        refreshWait(100 + 200*i );
        clearDisp(); refreshWait(50);
      }
      ans = (millis() - t0) % 6; // answer is determined by their action
      set_LEDs(dicePat[random(0, 6)], 2); // lite a given pattern, similar to dice, for 1-6
    } else {                     // btn3 was used, 1 of N leds
      ans = (ans + millis() - t0) % nleds;
      steps = random(ans + 4) + (3 * nleds);
      st = 60 - steps; // step time
      for (int i = 0; i < steps; i++) { // Spin around like a roulette wheel
        ans = ans % nleds + 1;
        litLED = ans;
        refreshWait(st);
        st += i;
      }
      brightOne = litLED;
    }

    while (1) {     // await the next Request
      scanBtns();
      refreshWait(1);
      if (ESC) return;     // ESC when done
      if (Btn) break;      // go again
    }
  }
}

// ***************************  Group 4  *******************************
// ------------------- Show off your Skills ----------------------------

void  playThatTune()  { // replay a set of notes played for you.
  // starts with one then 2, 3 ...
  // each LED spot is like a key on a keyboard (Low to left, high to the right)
  // b2 < > b3     b4 to play    b1 to rehear the taget set note series

  byte ninx, kinx, n, reps, nreps, errs, maxNotes;
  int dur, offset, xfact;
  byte note[21];  // should be much more than enough

  // ver = getOption(4);  // the higher version # the closer the notes are to each other
  int ver = 1;
  if (ver == 0) return;

  // xfact=(ver==4)? 50 : 100*(4-ver);
  xfact = 50*(5-ver) +25;
  offset = (xfact > 150) ? 0 : (xfact < 150) ? 250 : 150;
  refreshWait(500);
  for (n = 1; n <= nleds; n++)  {
    litLED = n;
    genTone(offset + xfact * n, 350); refreshWait(50); // run through the scale of notes
  }

  litLED = 0;
  level = 1;
  nreps = 0;
  while (1) {
    refreshWait(1000);
    clearDisp();
    maxNotes = level;
    reps = (level < 4) ? level : 3;
    errs = 0;
    for (n = 1; n <= maxNotes; n++) {
      note[n] = random(nleds) + 1;
    }
    if (note[1] == note[2]) continue; // try again

    for (n = 1; n <= maxNotes; n++)  {
      genTone(offset + xfact * note[n], 500);  // play the Tune
      refreshWait(100);
    }
    ninx = 1;      // note index
    kinx = nleds / 2; // key selection inx

    while (ninx <= maxNotes) {
      scanBtnWait(1);  //  refresh disp + scanBtns + delay
      if (ESC) return;
      if (btn1Pressed) {  // replay notes to refresh their memory
        ///volHigh = !volHigh;
        // before first try or ver=1: replays are free else needed replays count as errors
        if (!(ninx == 1 || ver == 1))  errs++;
        for (n = 1; n <= maxNotes; n++) {
          dur = (n == ninx) ? 750 : 500;
          genTone(offset + xfact * note[n], dur); refreshWait(100);
        }
      }
      // user to select note 1-8 with btn2 < & > btn3
      if (btn2Pressed) kinx = (kinx == 1) ?  nleds : kinx - 1; // --
      if (btn3Pressed) kinx = (kinx % nleds) + 1; // ++
      if (btn4Pressed) {  // they select the note related to the current lit Led
        genTone(offset + xfact * kinx, 500);
        refreshWait(100);
        if (note[ninx] == kinx) { // they got it Right
          ninx++;
        } else {                // they got it Wrong
          beep(50);
          errs++;
        }
      }
      litLED = kinx;
      if (Btn) wait_BtnRelease();
    }

    // completed a rep at level
    playNote(3); playNote(6); // taHah audio only version
    if (errs == 0) nreps++;
    if (nreps == reps) {
      showLevel(level); // acknowledge completed level
      level++;
      nreps = 0;
    }
  }
}

//----------------  React_Now - Swat Quickly as LEDs light up
void React_Now_r2() { // attemp to hit the button when the first LED lights up
  // 1st LED is lit for 100 msecs, additional LEDs lite every 20ms
  // challenge: if one pushes & holds before the first LED they get a wrong 'win'; how to fix ?
  //  bool btn = !PRESSED_STATE;
  int min_Period = 100;
  //  int btnInput;
  int LEDptr;

  //  btnInput = button[0];   // use the first button on the left of the four
  while (true) {
    //    scanBtns();
    // insure that the button is not already pressed
    while (Btn && !ESC) scanBtns();
    if (ESC) return;

    // insure all LEDs are off
    clearDisp();

    scanWait(random(2000, 5000));      // wait for 2 - 5 seconds
    lit[1] = 1;
    refreshDisp();
    beep(10); // takes 2x msecs
    refreshWait(min_Period - 20 - 20); // account for the 20ms beep time and 20ms in for loop

    for (LEDptr = 1; LEDptr <= 8; LEDptr++) {
      //      digitalWrite(lites[litLED], ON_STATE);
      //      refreshWait(20);
      lit[LEDptr] = true;
      refreshWait(20);
      scanBtns();
      if (BtnPressed && !ESC) { // they pressed the button
        brightOne = LEDptr;
        refreshWait(1000);
        break;
      }
    }
    if (LEDptr > 8) beep(100);
    //    scanBtns();
    if (ESC) break;
  }
}

// --------------- See It ... Catch It !   (aka Capture & Release)
// Try capturing all 4 fleeting blips (later releasing them) one at a time, without fault.
// A minimum of 4 button presses is required for a ferfect score of 4.
// When done (all captured or released) your score is given (sum of hits, misses + failed oppertunities).
void SeeIt_CatchIt() {    // attempt to hit assosiated button when an LED lights up
  int time_given = 450; // within given time
  byte n;
  byte cnt = 0, lcnt;
  byte goal = 4;
  byte tally; // progress towards goal

  int ver = 1; // =1 for lenient, =2 for strickly the right & only the right btn pressed (blocking multi-presses)
  clearDisp();  // insure all LEDs are off
  while (true) {
    //refreshWait(random(1000,4000));       // wait for 1 - 4 seconds
    do {
      wait_BtnRelease();
      Serial.print(" "); Serial.println(cnt);
      standby(random(1000, 4000));  // refreshing Disp for 'msec' or btn press event
      if (Btn && lcnt == cnt) cnt++; // penalize unless they just had one
      if (ESC) return;
    } while (Btn);

    //n = random(4);    // pick an LED
    n = (random(1, 4) + n) % 4;   // pick an LED, other than the last one picked
    if (tally == 3 && (goal != 0) == lit[2 * n + 1]) n = random(0, 4); // if its already whats wanted try for another
    brightOne = 2 * n + 1;
    refreshWait(time_given);
    lcnt = cnt;
    scanBtns();   // updates btn[] emeadiately (for ver=2 should call stableBtns())
    // did they catch it ?
    if (btn[n + 1] && !(ver == 2 && (btns || !BtnChanged))) {
      cnt++; lcnt = cnt;
      lit[2 * n + 1] = !lit[2 * n + 1];
      beep(100, 100); boop(400); // ta Da!
      while (Btn) scanBtns(); // stay until released
    } else {
      if (Btn || (lit[brightOne] == (goal == 0))) cnt++; // wrong one or an oppertunity waisted
    }
    brightOne = 0;

    tally = lit[1] + lit[3] + lit[5] + lit[7];
    if (goal == 0) tally = 4 - tally;
    if (goal == (lit[1] + lit[3] + lit[5] + lit[7])) { // completed the goal
      showLevel(cnt);  // hoe many did it take
      refreshWait(500);
      if (cnt == 4) {
        brightOne = 1;  // indicate best Win!
        beep(150, 100);
        brightOne = 0;
        beep(500);
      }
      goal = (goal == 0) ? 4 : 0;
      time_given = time_given - (time_given - 200) / 5; // increase chalenge
      cnt = 0;

      refreshWait(500);
      clearDisp();
      dimFlash = 1;
      //while (!btnPressed()) refreshWait(1); // wait to start a new Go.
      do {          // wait to start a new Go.
        scanBtns(); refreshWait(1);
      } while (!(BtnChanged && Btn != 0));
      dimFlash = 0;
      if (goal == 0) {
        lit[1] = lit[3] = lit[5] = lit[7] = 1;
      }
      refreshWait(500);
    }
    scanBtns();
    if (ESC) break;
  }
}

// ---------------------------------------------------------------------
void Balance() {
  int grade, manAdj;
  int msDelay;
  int uptrend, dntrend;
  unsigned short int pos, ball;  // short is for RP2040 
  int dspFrame, lstFrame;
  byte ledScale;
  int x;
  // old bug: when 'pos' crosses 0 it skips over the 'nled' (8th) LED
  
  ledScale = (nleds < 10) ? 40 : 30; //  make it easier ... (larger number slows action down
  loopCnt = 0;

  while (1) {
    // setup game
    clearDisp();
    wait_BtnRelease();   // wait for no button pressed

    pos = (nleds / 2) * ledScale;
    grade = 0;
    playing = true;
    while (playing) {
      // ----------------  determine input adjustment Rate
      if (loopCnt % 26 == 0 || BtnPressed) {
        manAdj = -(2 * btn1 + btn2 + (btn1 && btn2)); // for b1:-2 b2:-1 both:-3
        manAdj += (2 * btn4 + btn3 + (btn4 && btn3));
        grade +=  manAdj * 1;
      }
      if (loopCnt % 10 == 0) {
        pos += grade;
        while (pos>=65280) pos += grade; // patch to fix wrap-around problem crossing 0
        ball = (pos / ledScale);
        dspFrame = ball / nleds;
        if (lstFrame != dspFrame) Beep(30);
        lstFrame = dspFrame;
      }

      //----------------- UPDATE Display
      loopCnt++;
      clearDisp();
      ball = ball % nleds;
      lit[ball + 1] = 1;

      // set update timing
      msDelay = (nleds < 10) ? 5 : 4; // slow it down some for nleds<10
      scanBtns();
      if (Btn) refreshWait(msDelay);
      else noBtnDelay(msDelay);
      if (loopCnt % 26 == 0) { // every 100-125 mses
        //  ------------------  check tilt-incline magnitude and apply it
        // determine grade
        if (uptrend > 5) grade += 1; // increase incline
        if (dntrend > 5) grade -= 1; // decrease incline
        uptrend = dntrend = 0;
      } else {
        x = analogRead(UNUSED_ADC); // read only every 2-5 msecs
        x += random(1000);        // use both real & psudo world
        if ((x % 5) == 1) ++uptrend;
        if ((x % 5) == 3) ++dntrend;
      }
      if (btn4 && btn1) {
        return;
      }
    }
  }
}

// =====================================================================
void Shuffleboard() { // Inline (Shuffleboard)
  // Land a ball in each posible "target" spots to the right.
  // Hold Btn1 to swing, and let go to throw.
  //Land a puck on each possible "target" spots to the right. If already occupied, it gets knocked out.
  // Hold Btn1 then let it go to launch your puck. Longer you hold further your 'puck' goes.
  int player = 0;
  int i, cnt;
  int lastCnt;
  int travel, rate, thrust;
  bool rolling, newPressed, released, btnDown;
  byte ver;
  byte row, lastS;

  //  ver=get_Option(2,1);
  ver = getOption(2);
  if (ver == 0) return;
  loopCnt = 0;
  level = 1;

  clearDisp();
  refreshWait(500);

  while (1) {
    // setup game-level
    wait_BtnRelease();   // wait for no button pressed
    cnt = 0;

    clearDisp();
    playing = true;
    while (playing) {
//      spot = 1;       // where we are throwing from
      dimFlash = 1;  // prompt to wined up & throw
      travel = 0;
      thrust = 0;
      rolling = false;
      turn = 1;   // # of rolled balls
      while (turn) {
        bool pressed = btnPressed();
        if (btn4 && btn1) return; // uses btn4+Btn1 to exit
        btnDown = (btn1 || btn4);
        if (btnDown && !rolling) {
          thrust++;
          spriteOne = 1;
        } else {
          spriteOne = 0;
        }

        released = (!btnDown) && BtnChanged;
        if (released) {   // the ball takes flight
          rolling = true;
          dimFlash = 0;
          //Serial.print("thrust: "); Serial.println(thrust);
          rate = (thrust + 750) / 15; //for thrust=7500 gives 100 which results in 10 spots in one second
          rate = rate / 4; // patch so we can update more often for smoother fast motion balls
        }

        if (rolling && (loopCnt % 50) == 0) { // was %200 before patch above
          if (thrust <= 30) {         // coming to a stop
            //            if (!dim[litLED] && !(ver==2 && !row1)) { // landed on a spot
            if (!dim[litLED] && !(ver == 2 && row != 2)) { // landed on a spot
              dim[litLED] = true;
              boop(40);
            } else {      // hit a laying ball
              boop(20); beep(20);
              if (ver == 1 || row == 2) dim[litLED] = false;
            }
            lastCnt = cnt;
            cnt = 0;
            for (i = 2; i <= nleds; i++) cnt += dim[i];
            if (cnt == nleds - 1) {
              winner(1);
              playing = false; // reset, due to winner()
            } else if (cnt == 0 && lastCnt) {
              lost();
              playing = false;
            }

            turn = 0;
            refreshWait(1000);
            litLED = 0;
            continue;
          }

          travel = travel + rate;
          thrust -= rate;
          litLED = travel / 100;        // (note: w/ISR use there could be a race condition)
          row = litLED/nleds + 1;
          //          if(litLED!=lastS) {lastS=litLED; Serial.print(litLED); Serial.print("  "); Serial.println(row);}
          litLED  = (litLED % nleds) + 1; // keep it in proper range 
        }

        //================ UPDATE Display
        loopCnt++;
        refreshWait(1);
        // if 'esc' button held down ... Exit
        if (btn4 && btn1) {
          return;
        }
      }
    }
  }
}

void Memory() {
  byte mem[nleds + 2], ledType[4], sndType[4];
  byte shw[nleds + 2]; // whats showing
  byte sndOffset, ledOffset;
  int i, j, item, tries;
  byte s1, s2, loc, sum, ledState;
  bool firstSel;
  unsigned long randSeed;

  // for use with < 8 leds a ver4 could rotate the whole field 1 cell after each attempted match up.
  //                              this adds to the other wise fairly easy to become very difficulty.

  byte ver = getOption(3);  // ver 1: unique Illumination and Audio
  // ver 2: Illination Ony, No audio
  // ver 3: Audio Only, No Illumination
  if (ver == 0) return;

  randSeed = micros() / 4; // note millis() did not do the job
  randomSeed(randSeed); // make the challenge always different
  Serial.print("random Seed:  "); Serial.println( randSeed);
  mem[nleds + 1] = 0;
  glowEnable = true;
  while (!ESC) {
    clearDisp();
    wait_BtnRelease();
    firstSel = true;
    loc = 1;
    flashLed = 1;
    tries = 0;
    for (i = 0; i <= nleds + 1; i++) {
      mem[i] = 0;
      shw[i] = 0;
      glow[i] = 0;
    }
    item = 0;  // valid items 1-4

    for (i = 1; i < nleds; i++) { // -------------------  place items into slots
      while (mem[i] != 0) i++;
      if (i > nleds) break;
      item++;
      mem[i] = item;
      do {  // find a spot for its match
        j = random(i + 1, nleds + 1);
        if ((i == 1 || i == 3) && j == i + 1) j = random(i + 1, nleds + 1); // reduce chance of matches 1&2 and 3&4
      } while (mem[j]);
      mem[j] = item;
    }
    ledOffset = random(4);
    sndOffset = random(4);

    for (i = 1; i <= nleds; i++) {
      Serial.print(i);
      Serial.print("  sound:  "); Serial.print(mem[i]);
      Serial.print("  light:  "); Serial.println(mem[i]);
    }
    Serial.println();

    // --------- player(s) try to match up the items, testing thier Memories
    while (!ESC) {
      if (btn1 && btn4) return; // fast exit

      if (Btn) {
        if (btn2Pressed) loc = (loc == 1) ?  nleds : loc - 1; // -- move left Allow 0:debugging
        if (btn3Pressed) loc = (loc % nleds) + 1; // ++ moves right
        if (BtnPressed) flashLed = loc;
        if (btn1Pressed) {    // wait for slot selection
          if (shw[loc]) {
            buZZ(15); // already choosen
          } else {
            // its the First slot selection
            if (firstSel) {
              firstSel = false;
              tries++;
              s1 = loc;
              shw[loc] = mem[loc];
              flashLed = 0;
              if (ver != 3) { // ---------  show the selected item
                switch ((mem[loc] - 1 + ledOffset) % 4)
                {
                  case 0: dim[s1] = 1;
                    break;
                  case 1: lit[s1] = 1;
                    break;
                  case 2: brt[s1] = 1;
                    break;
                  case 3: glow[s1] = 1;
                }
              } else {
                brt[s1] = 1;
              }
              if (ver != 2) {
                switch ((mem[loc] - 1 + sndOffset) % 4)
                {
                  case 0: boop(75);
                    break;
                  case 1: beep(75);
                    break;
                  case 2: Beep(90);
                    break;
                  case 3: buZZ(60);
                }
              }
              refreshWait(250);
            }

            // its the Second slot selection
            else {
              firstSel = true;
              s2 = loc;
              shw[loc] = mem[loc];
              flashLed = 0;
              if (ver != 3) { // ---------  show the selected item
                switch ((mem[loc] - 1 + ledOffset) % 4)
                {
                  case 0: dim[s2] = 1;
                    break;
                  case 1: lit[s2] = 1;
                    break;
                  case 2: brt[s2] = 1;
                    break;
                  case 3: glow[s2] = 1;
                }
              } else {
                brt[s2] = 1;
              }
              if (ver != 2) {
                switch ((mem[loc] - 1 + sndOffset) % 4)
                {
                  case 0: boop(75);
                    break;
                  case 1: beep(75);
                    break;
                  case 2: Beep(90);
                    break;
                  case 3: buZZ(60);
                }
              }
              refreshWait(250);

              // Do we have a match ?
              if (mem[s1] == mem[s2]) {
                refreshWait(250);
                taDah();  // YES - give very good ack ... keep them lit
                if (ver == 3) {
                  dim[s1] = 1;
                  dim[s2] = 1;
                }
                refreshWait(50);

                // Do we have a winner ?
                sum = 0;
                for (i = 1; i <= nleds; i++) {
                  if (shw[i]) sum++;
                }
                if (sum == nleds) {
                  taDah();
                  refreshWait(250);
                  showLevel(tries);
                  Btn = 0;
                  while (!Btn) scanBtnWait(1);  // wait for next button press
                  break;  // start over
                }

              } else {
                refreshWait(500);
                buZZ(15); // NO - give raspberry ... un lite them
                refreshWait(1500);   // let them ponder to remember
                shw[s1] = 0;
                shw[s2] = 0;
                flashLed = loc;
              }
              if (ver == 3) {
                brt[s1] = 0;
                brt[s2] = 0;
              }
            }
          }

          // update whats displayed
          if (ver != 3) {
            for (i = 1; i <= nleds; i++) {
              dim[i] = lit[i] = brt[i] = glow[i] = 0;
              if (shw[i]) {
                ledState = ((shw[i] - 1 + ledOffset) % 4);
                switch (ledState)
                {
                  case 0: dim[i] = 1;
                    break;
                  case 1: lit[i] = 1;
                    break;
                  case 2: brt[i] = 1;
                    break;
                  case 3: glow[i] = 1;
                    break;
                }
              }
            }
          }
        }
      }

      refreshWait(1);
      scanBtns();
    }
  }
}

void CodeBreaker() {
  byte code[5];
  int i;
  byte key, good, sum;
  int tries;
  byte keys[5], entry[5], used[5];
  int waiting;

  // with <8 Leds, the ongoing status reporting will need rethinking

  while (1) {
    clearDisp();
    for (i = 4; i >= 0; i--) { // start 'flash screen'
      //      allColor=i;
      setColor(i);
      refreshWait(200);
    }
    keys[1] = keys[2] = keys[3] = keys[4] = 0;
    for (int i = 1; i <= 4; i++) {
      code[i] = random(4) + 1;
      keys[code[i]]++;  // keep cnt of # of each keys (btns) used in the code
    }
    Serial.print(" Code:  ");
    for (int i = 1; i <= 4; i++) {
      Serial.print(code[i]);
      Serial.print("  ");
    }
    Serial.println();

    tries = 0;
    while (!ESC) {
      used[1] = used[2] = used[3] = used[4] = 0;
      entry[1] = entry[2] = entry[3] = entry[4] = 0;
      tries++;;
      good = 0;
      // -------- get a 4 key attempt
      for (key = 1; key <= 4; key++) {
        waiting = 0;
        while (!Btn) {    // wait for next button press
          scanBtnWait(1);
          if (++waiting > 10000) { // if nothing for 10 secs, get their attention
            for (i = 4; i >= 0; i--) {
              setColor(i);
              refreshWait(100);
            }
            waiting = 0;
          }
        }
        entry[key] = btnNum;
        setColor(btnNum);
        if (btn1Pressed) phoneTone(1);
        if (btn2Pressed) phoneTone(4);
        if (btn3Pressed) phoneTone(7);
        if (btn4Pressed) phoneTone(10); // would be the '0' on the dial
        if (code[key] == entry[key]) good++;
        used[btnNum % 5]++;
        setColor(0);
        wait_BtnRelease();
        if (ESC) return;
      }
      Serial.print(" Good: "); Serial.print(good);

      // -------- Check if it is right
      if (good == 4) {
        winner(1);
        showLevel(tries);
        refreshWait(2000);
        break;
      }

      // -------- report how well they did
      sum = 0;
      for (key = 1; key <= 4; key++) {
        if (keys[key] > used[key]) sum += used[key];
        else sum += keys[key];
      }
      Serial.print("  keys used: "); Serial.println(sum);
      delay(150);
      if ((sum + good) == 0) buZZ(200);
      else {
        showLevel(good);
        for (i = nleds; sum > 0; i--) { // to the right show # of correctly selected keys
          lit[i] = true;
          sum--;
        }
        refreshWait(3000);
        clearDisp();
      }
      //refreshWait(1);
      refreshWait(1);
      scanBtns();

      wait_BtnRelease();
      if (ESC) return;
    }
  }
}

// ***************************  Group 5  *******************************
// ----------- Demos, Challenges, Activities and Applets ------------------
void  Support_Demo()  {   // Demonstrates use of set of Hardware support functions, plus Scales of Leds and audio levels
  // delays & Display Refresh timing is critical to proper display, as a changing myMillis is used to fill the display
  // when using interupt driven display updates this is not an issue.
  byte lvl, c, maxLvl, LedL;
  int i;
  bool waiting;
  byte n, lastN, lastC, nLit;
  unsigned int tCnt;
  unsigned int ledTime;
  bool WelcomeScreen = true;
  byte cD, cL, cB, cDF, cFl, cBl, offset, cMax; // for phase 4

  //  demostrate the usefulness of set_LEDs()
  for (i = 0; i < 8; i++) {
    refreshWait(100);
    set_LEDs(1 << i, 1);
  }
  for (i = 0; i < 8; i++) {
    refreshWait(100);
    set_LEDs(1 << i, 2);
  }
  for (i = 0; i < 8; i++) {
    refreshWait(100);
    set_LEDs(1 << i, 3);
  }
  for (i = 0; i < 8; i++) {
    refreshWait(100);
    set_LEDs(1 << i, 0);
  }
  refreshWait(500);

  //    escCnt=ESC=0;
  ESC = 0;
  while (!ESC) {

    clearDisp();
    lvl = 0;
    wait_BtnRelease();
    waiting = true;
    WelcomeScreen = true;
    //------------------------ Phase 1 --------------------------------------------
    if (Verbose) Serial.println(" Demonstrates the (Mini_STEM_Platform) Standard Game sounds with lights ");
    if (Verbose) Serial.println("   Btn1: Boop-Dim,  Btn2: Beep-Lit, Btn3: BuZZ-Bright  ");
    while (!btn4) {
      scanBtns();

      if (WelcomeScreen && !BtnPressed) {  // display splash display until a new button presses
        for (i = 1; i <= nleds; i++) {
          dim[i] = true;  // set all LEDs dim
          lit[i] = ((msCnt / 333 + i) % 2); // flash odd & even LEDs in opposition
        }
        //      refreshWait(1);
      } else WelcomeScreen = false;

      if (BtnPressed) {
        clearDisp();
        for (i = 1; i <= nleds; i++) { // set all LEDs to a given level
          if (btn1Pressed) dim[i] = true;
          if (btn2Pressed) lit[i] = true;
          if (btn3Pressed) brt[i] = true;
          if (btn4Pressed) clearDisp();
        }
      }

      // produce sound while btn1-4 is held
      while (Btn) {
        if (btn1) boop(10);
        if (btn2) bop(10);
        if (btn3) beep(10);
        if (btn4) {
          buZZ(700);
          break;
        }
        scanBtns();
      }

      refreshDisp();
      delay(1);
      if (ESC) break;
    }

    wait_BtnRelease();
    //------------------------ Phase 2 --------------------------------------------
    if (Verbose) Serial.println(" Step thru Expanded Game sounds with lights. Based on additions utils in this sketch. ");
    while (!btn4) {
      refreshWait(1);
      scanBtns();
      if (ESC || (btn4 && btn1)) return;
      if (waiting && !Btn) {
        // ------------- create a 'Waiting' display
        n = (myMillis / 1000) % nleds;
        if (n != lastN) { // brighten up the privious dim LED, for 2x action
          litLED = dimLED;
          lastN = n;
        }
        //c = (n&1)? 1 : nleds;                 // toogle between 1st and last Led
        //c = (n/(nleds/2)) + 2*(n%4);            // Step thru odd then even Leds
        c = (nleds / 2) * (n & 1) + (n >> 1);    // pop back&fore lower-upper section
        //c = (n&0xFC) + ((n<<1)&2) + ((n>>1)&1);  // reverse the lower two bits, ~ skippy travel
        //dimLED=c+1;
        dimLED = c + 1;
      } else {
        dimLED = 0;
        waiting = false;
        // lvls: 0=off,  1=dim,  2=lit,  3=brt,  4=running flash
        if (btn2Pressed) lvl = (lvl == 0) ?  4 : lvl - 1; // -- down
        if (btn3Pressed) lvl = (lvl + 1) % 5;         // ++ up
        if (BtnPressed && lvl == 0) clearDisp();
        dim[myMillis % nleds + 1] = (lvl == 1); // note this is dependant on Disp Updates, to fill over time
        lit[myMillis % nleds + 1] = (lvl == 2);
        brt[myMillis % nleds + 1] = (lvl == 3);
        if (lvl == 4) {
          for (i = 1; i <= nleds; i++) { // -------------  flash odd & even LEDs in opposition
            brt[i] = (((myMillis / 200) % 2) == (i & 1));
          }
        }

        if (Btn && !btn4) {   // TBD: ? in this block no need to test for 'BTN' ?
          if (Btn && lvl == 1) genTone(150, 28);
          if (Btn && lvl == 2) boOop(30); // 250Hz boop tone with Vibrado
          //if (Btn && lvl==3) beep(20);
          if (Btn && lvl == 3) LOUDbeep(19); // extra Loud beep
          if (Btn && lvl == 4) Beep(20); // mixed freq. beep
          //if (Btn && lvl==0) buZZ(41);
          if (Btn && lvl == 0) {
            boop(4);  // not as harsh as 'buZZ'
            beep(1);
            bop(2);
          }
        }
        //            refreshWait(1);
      }
    }

    wait_BtnRelease();
    //------------------------ Phase 3 ------------------------------------------------
    if (Verbose) Serial.println(" Step thru a gradient scale of 24 tones and 16 levels of lit ");
    clearDisp();
    lvl = 0;
    maxLvl = 24;
    if (!ESC) wait_BtnRelease();
    if (ESC) return;
    waiting = true;
    while (!btn4) {
      refreshWait(1);
      scanBtns();
      if (waiting && !Btn) {
        if (myMillis % 50 == 1) { // upate every ~50 ms
          nLit = (nLit + 1) % 32;
          if (nLit == 0) nLit++;
          levelOn[0] = (nLit <= 16) ? nLit : (16 - (nLit - 16)); // goes 1-16-1 & repeats
          for (i = nleds; i > 0; i--) { // Fade-Wave for wait banner
            levelOn[i] = levelOn[i - 1]; // scroll light levels to the right
          }
        }
      } else {
        if (waiting) {
          clearDisp();   // clears at the hardware level too, now
          //refreshDisp();
          waiting = false;
        }
        if (btn2Pressed) lvl = (lvl == 0) ?  maxLvl : lvl - 1; // -- down
        if (btn3Pressed) lvl = lvl % maxLvl + 1; // ++ up  (lvl is 1-maxLvl)
        //            levelOn[myMillis%nleds+1]=lvl;
        levelOn[ledTime % nleds + 1] = (16 * lvl + 8) / maxLvl; // spread accross 16 levels,

        LedL = (lvl - 1) % nleds + 1;
        if (btn2Pressed || btn3Pressed) {
          blinkLed =  (LedL > 0) ? (LedL - 1) % nleds + 1 : 0;
        }

        if (BtnPressed && !btn4)  playNote(lvl, 200);
        //while (btn1) {playNote(lvl,200); scanBtns();}  // 100 for less notable breakup
        // this faster test reduces tone breakup (but is not in the spirit of HW support include)
        while (digitalRead(button[0]) == PRESSED_STATE) {
          playNote(lvl, 200);
        }

        ledTime++;
      }
    }

    wait_BtnRelease();
    //------------------------ Phase 4 --------------------------------------------
    if (Verbose) Serial.println(" Move potential Cursors/Sprites accross a background 'display' field");
    if (Verbose) Serial.println("   Dim-Flash,Flash,Blink / Dim,Lit&Bright LEDs");
    clearDisp();
    dim[5] = dim[6] = dim[7] = dim[8] = true; // background field
    //byte cD,cL,cB,cDF,cFl,cBl, offset, cMax;
    cDF = 0; cFl = 18; cBl = 16; cD = 14; cL = 12; cB = 10; // init cursors in a spacial relationship
    cMax = 20;
    offset = 0;

    while (!btn4) {
      refreshWait(1);
      scanBtns();
      if (BtnPressed) {
        if (btn2Pressed) offset = (offset - 1 + cMax - 1) % (cMax - 1); // move cursors left
        if (btn3Pressed) offset = (offset + 1) % (cMax - 1); // move cursors right
        if (btn2 || btn3) {
          dimIt = ((cD + offset) % cMax <= nleds) ? (cD + offset) % cMax : 0;
          litLED = ((cL + offset) % cMax <= nleds) ? (cL + offset) % cMax : 0;
          brightOne = ((cB + offset) % cMax <= nleds) ? (cB + offset) % cMax : 0;
          dimFlash = ((cDF + offset) % cMax <= nleds) ? (cDF + offset) % cMax : 0;
          flashLed = ((cFl + offset) % cMax <= nleds) ? (cFl + offset) % cMax : 0;
          blinkLed = ((cBl + offset) % cMax <= nleds) ? (cBl + offset) % cMax : 0;
        }
      }
    }

    if (!ESC) wait_BtnRelease();
  }
}

void Illumination_Challenge() { // ---------  DYI LED Illumination Pattern setup, &...
  // Challenge: recreation of Led illumination pattern as flawlessly as possible
  int n;
  bool equal;
  long pattern;
  byte patcell[14];
  int divr, state;
  bool trying, init;
  byte lastBtn;

  glowEnable = true;
  while (1) {
    init = true;
    cursorPtr = 1;
    lastBtn = 2; // so cursor shows
    clearDisp();
    pattern = random(10001, 2147483647L);
    Serial.print("Pattern Number: "); Serial.println(pattern);
    divr = 2;
    for (n = 0; n <= nleds; n++) {
      cell[n] = 0;
      divr += n;
      patcell[n] = (pattern / divr) % 6; // all psuedo random setting are dirived from a single pattern number
      if (patcell[n] > 3 ) patcell[n] = (pattern / (divr + 1)) % 6; // reduces freq of arc & glow
    }

    trying = true;
    while (trying) {
      refreshWait(1);
      //scanBtns();
      // ------------------- button reporting, Debuging
      if (btnPressed()) {
        lastBtn = btnNum;
        if (DEBUG) {
          if (btn2Pressed) {
            for (int i = 1; i <= nleds; i++) {
              Serial.print(i); Serial.print("  pat: ");  Serial.print(patcell[i]);
              Serial.print("  cell: ");  Serial.println(cell[i]);
            }
          }
        }

        if (btn1Pressed) {  // --------- View/Test the desired Illumination Pattern
          equal  = true;
          for (n = 1; n <= nleds; n++) if (cell[n] != patcell[n]) equal = false;
          if (equal) {
            taDah();
            refreshWait(1000);
            while (btn1) {
              scanBtns();
              refreshWait(1);
            }
            trying = false;
          } else {
            if (!init) boop(25);
          }
        }
        // ------------------ LED change support
        if (btn2Pressed) cursorPtr = (cursorPtr == 0) ?  nleds : cursorPtr - 1; // --  0-nleds
        if (btn3Pressed) cursorPtr = ((cursorPtr + 1) % (nleds + 1)); // ++
        if (btn4Pressed) cell[cursorPtr] = (cell[cursorPtr] + 1) % 6;
        if (btn4) init = false;
      }

      // ---------------  update the display
      //clearDisp();
      for (n = 1; n <= nleds; n++) {
        state = (btn1) ? patcell[n] : cell[n];
        dim[n] = (state == 1);
        lit[n] = (state == 2);
        brt[n] = (state == 3);
        arc[n] = (state == 4); // added arc & glow Sept 2021
        glow[n] = (state == 5);
      }
      blinkLed = (lastBtn == 2 || lastBtn == 3) ? cursorPtr : 0;

      if (ESC) return;
    }
  }
}

void Automated_Art() {
  // ========================================  Automated Art (would-be 'LIFE')
  long pattern;
  int dir[4];
  int spritePtr;    // sprite that is not seen
  int deltaOne, deltaPtr;
  unsigned int beet;
  bool gMute;
  byte speed;
  int spdCnt[] = {1000, 500, 250, 100};
  bool pause;

  deltaPtr = 0;
  loopCnt = beet = 0;
  gMute = false;
  speed = 1;

  while (1) {
    loopCnt++;
    refreshWait(1);
    scanBtns();
    if (ESC) break;

    if (btn1Pressed || deltaPtr == 0) {
      pattern = random(10001, 2147483647L);
      randomSeed(pattern);
      spriteOne = random(nleds) + 1;
      spritePtr = random(nleds) + 1;
      deltaOne = random(1, nleds);
      deltaPtr = random(1, nleds);
      Serial.print("Pattern Number: "); Serial.print(pattern); Serial.print(" +D: "); Serial.print(deltaOne); Serial.print(" -D: -"); Serial.println(deltaPtr);
    }
    if (btn2Pressed) {  // toggle genetic Mutation
      gMute = !gMute;
      if (gMute) beep(50);
    }
    if (btn3Pressed) {  // speed shift
      speed = (speed + 1) % 4;
    }
    while (btn4Pressed || pause) {  // pause func
      if (btn4Pressed) pause = !pause;  //toogle in&out of pause
      scanBtns(); refreshWait(1);
      if (ESC) return;
    }

    for (int i = 0; i < 4; i++) dir[i] = 0;
    if (loopCnt % spdCnt[speed] == 0) {      // consider influence over time
      Serial.print(" ON: "), Serial.print(spriteOne);  Serial.print(" OFF: "), Serial.println(spritePtr);
      lit[spriteOne] = true; // leave a 'on' trail
      lit[spritePtr] = false; // turn em off
      spriteOne += deltaOne; // ++ go right
      spritePtr -= deltaPtr; // -- go left
      if (spriteOne > nleds)  spriteOne = spriteOne - (nleds + 1);
      if (spritePtr < 0)      spritePtr = spritePtr + (nleds + 1);

      //was         if (gMute && (++beet)>(nleds*nleds) && spritePtr==0) {  // time to alow a mutation
      if (gMute && (++beet) > (4 * nleds)) { // time to alow a mutation
        if (random(2))  deltaOne = random(1, nleds);
        else            deltaPtr = random(1, nleds);
        beet = 0;
      } else {      // wrap-around spriteOne now, spritePtr will after one more beet, so that we see upto a nleds^2 sequence
        if (spriteOne == 0) spriteOne = deltaOne;
      }
    }
  }
}

void  tuneLightOrgan()  {     // setup a series of notes for replay and evaluation
  // option 1: choose 15 piano notes G3-G5,   number of notes is up to the count of LEDs
  //        2: select Frequencies from a range of 47  (131 - 13,190 Hz)
  byte note, notes[nleds + 2]; // should be much more than enough
  byte lvl, ninx, n, maxNote, numN, n0 = 4;
  byte playCnt;
  int freq, dur;

  ver = getOption(2);
  if (ver == 0) return;

  maxNote = (ver == 1) ? 15 : 47; // number of notes in the scale used
  refreshWait(500);
  freq = 130;
  for (n = 0; n < 15; n++)  { // give a sample of notes
    litLED = n % nleds + 1;
    notes[litLED] = 0;
    if (ver == 1) {
      playNote(n0 + n + 1, 200);  // run through the scale of notes
      refreshWait(50);
    }
    else genTone(freq, 150);
    freq = 14L * freq / 10;
  }

  refreshWait(1000);
  clearDisp();
  ninx = 1;      // note index
  lvl = 0;
  while (1) {
    if (ESC || (btn4 && btn1)) return;
    if (btn1Pressed) {  // ----------------------  play notes at desired tempo
      t0 = millis(),
      wait_BtnRelease();
      dur = millis() - t0;
      Serial.print("Duration of each note: "); Serial.println(dur);
      //        volHigh = !volHigh;
      clearDisp();
      numN = nleds;
      while (!notes[numN]) numN--;  // prepare to play only the provided notes
      for (n = 1; n <= numN; n++) {
        levelOn[n] = (16 * notes[n] + maxNote - 1) / maxNote;
        if (ver == 1) {
          note = (notes[n] > 0) ? n0 + notes[n] : 0;
          playNote(note, dur);
          refreshWait(dur / 4);
        } else {
          note = notes[n];
          if (note < 25) freq = basicKeyNotes[note];
          else freq = basicKeyNotes[note - 23] * 10;
          genTone(freq, dur);
        }
      }
    }

    // ---------------------------  user to change note/light level 0-N with btn2 < & > btn3
    if (btn2Pressed) lvl = (lvl == 0) ?  maxNote : lvl - 1; // -- freq.+illumination
    if (btn3Pressed) lvl = (lvl + 1) % (maxNote + 1);       // ++ freq.+illumination
    notes[ninx] = lvl;
    levelOn[ninx] = (16 * lvl + maxNote - 1) / maxNote;
    while (btn2 || btn3) {
      if (ver == 1 && lvl != 0) playNote(n0 + lvl, 120);
      if (ver == 2) {
        if (lvl < 25) freq = basicKeyNotes[lvl];
        else freq = basicKeyNotes[lvl - 23] * 10;
        genTone(freq, 120 + lvl); // give high freqs added dur
      }
      btnsNow();   // get a full state update (despite debounce)
      // with ver 2 auto step thru freqs
      if (ver == 2 && btn2) lvl = (lvl == 0) ?  maxNote : lvl - 1;
      if (ver == 2 && btn3) lvl = (lvl + 1) % (maxNote + 1);

      if (btn2 && btn3) {  // reset to null note
        lvl = 0;
        wait_BtnRelease();
      }
      if (!Btn) {
        Serial.print("Note: ");  // B2-3 released
        Serial.println(lvl);
      }
    }

    if (btn4Pressed) {  //-------------  move point of interest 1-nleds
      ninx++;
      if (ninx > nleds) ninx = 1;
      lvl = notes[ninx];
    }

    // update our custom cursor
    spriteOne = dimIt = litLED = dimFlash = 0;
    if (myMillis % 1800 < 300 || btn4) {
      if (notes[ninx] <= (2 * maxNote / 3) ) spriteOne = ninx;
      else                dimIt = ninx;
    }

    refreshWait(1);
    scanBtns();
  }
}

// ============= Sound effect functions in support of Sirens ======================
void  KnightRider() {  // KITT-2000 grill Light display
  // back-N-forth fading light, taking up 25-45% of display
  // 3/4 sec per sweep across 8 lights with a 3 led dimming tail
  // every fouth sweep it goes voooo - woomm
  int loopCnt = 0;
  int hotSpot, inx;
  int maxCnt = 100;
  int pace = 100;
  byte leds[nleds + 2]; // 8 map to display other are out of range
  int dir = 1;

  // setup game-level
  for (int i = 1; i <= nleds; i++) {
    leds[i] = 0;
  }

  hotSpot = -1;
  while (!Btn) {
    if (hotSpot == nleds - 1 && dir == -1) genTone(100, pace);
    else if ((hotSpot == nleds - 4 || hotSpot == nleds - 3) && dir == -1) genTone(175, pace);
    else refreshWait(pace);

    loopCnt = 0;
    hotSpot = (hotSpot + 1) % (nleds + 4) - 1;
    hotSpot += dir;
    if (hotSpot >= (nleds + 2)) dir = -1;
    if (hotSpot <= -1) dir = 1;
    for (int i = 1; i <= nleds; i++) {
      if (leds[i] > 1) leds[i] -= 3;
      else leds[i] = 0;
      if (i == hotSpot) leds[i] = 13;
    }

    for (int i = 1; i <= nleds; i++) levelOn[i] = leds[i]; // update display data structure
    refreshDisp();
    delay(1); // then try again
    scanBtns();
  }
}

// ----------------------------------------------------
void  Sweep_around() {  // like a electrocardiogram EKG, beep monitor
  int loopCnt = 0;
  int locOffSet, inx;
  int maxCnt = 150;
  byte midpt;
  byte leds[8 + 2]; // 8 map to display other are out of range
  int extra = 3; // =3 scan restart after sig goes off the end, good for EKG monitor
  // =0 head of trace wraps around directly
  // =5 a couple pause beets before retracing

  midpt = nleds / 2;
  // setup game-level
  for (int i = 1; i <= nleds; i++) {
    leds[i] = 0;
  }

  locOffSet = -1;
  while (!Btn) {
    if (maxCnt == loopCnt++) {
      loopCnt = 0;
      locOffSet = (locOffSet + 1) % (nleds + extra);
      for (int i = 1; i <= nleds; i++) {
        inx = i + locOffSet;
        if (inx > nleds + extra) inx = inx - (nleds + extra);
        if (inx <= midpt) leds[i] = (midpt - inx) * 3 + 1;
        else            leds[i] = 0;
      }
    }

    for (int i = 1; i <= nleds; i++) levelOn[i] = leds[nleds - i + 1]; // update display data structure
    //   refreshDisp();
    if (levelOn[4] > 7 && loopCnt % 3 == 0) {
      levelOn[4] = 16;  // a live signal pulse
      beep(1);
    }
    else refreshWait(1);
    scanBtns();
  }
}

// ---------------------------------------------------
// recreation of various well known Riffs, Sirens and a catch all of other Sound Effects
void SoundEffects() {
  //int effect; now global
  int dir, lvl, c;
  bool tomove, waiting;
  bool newEffect = true;
  byte n, lastN, lastC;
  int f;
  unsigned long msec;
  byte rnum, keyN = 14;
  int cnt;
  byte t[] = {2, 4, 3, 0, 7, 5, 8, 1, 0, 0, 0, 0};
  bool spkP, spkN, tmp;
  bool enbleWait = true;
  int dur=400;  // for 150 BPM

  ESC = 0;
  effect = 1; // start with 6, for British Hilo siren
  int numE = 18;
  wait_BtnRelease();
  while (!btn4) {
    switch (effect) {
      case 1:   // make a little Whitish Noise while waiting to procede to ... other Demo sound effects
        if (newEffect && Verbose) Serial.println("Whitish Noise");
        //enbleWait=false;  late find: with no 1 msec refreshWait 1n loop, MUCH better white noise can be made (use ~10x delays)

        // shortest implementation
        //              digitalWrite ( BEEPPIN, random(2));

        // Loud  48db implementation
        digitalWrite ( AUDIO_NEG, LOW);
        digitalWrite ( BEEPPIN, random(2));
        delayMicroseconds(random(1000));
        digitalWrite ( AUDIO_NEG, random(2));
        delayMicroseconds(random(100));

        // Loudest 52db, but pitchy
        //               digitalWrite ( AUDIO_NEG, LOW);
        //               digitalWrite ( BEEPPIN, random(2));
        //               delayMicroseconds(random(20,160));
        //               digitalWrite ( AUDIO_NEG, HIGH);
        //               digitalWrite ( BEEPPIN, random(2));
        //               delayMicroseconds(random(20,160));

        // best sounding ~= White noise
        //              digitalWrite ( BEEPPIN, random(2));
        //              delayMicroseconds(random(20,200));
        //              digitalWrite ( AUDIO_NEG, random(2));
        //              delayMicroseconds(random(20,200));

        digitalWrite (lites[0], random(2) || random(2));  // only lit 25% as ON_STATE is 0
        break;

      case 2:  // Four note ditti, lights and sounds (reminiscent of melody from "Close Encounters")
        if (newEffect && Verbose) Serial.println("Four note ditti, lights and sounds (reminiscent of melody from 'Close Encounters'");
        n = (myMillis / 650) % 4;
        if (n != lastN) {
          //Serial.print("n = ");  Serial.print(n);
          lastN = n;
          if (n == 2) {
            clearDisp();
            refreshWait(650);
          }
          n = 3 - n;      // results in a pattern more like used by Simon
          c = ((n & 1) << 1) + (n >> 1); // reverse the two bits
          //Serial.print("   c = ");  Serial.println(c);
          setColor(c + 1);
          playNote(8 + 2 * c, 250);
        }
        break;
      case 22:  // was tobe Four note ditti, lights and sounds (part of melody "Close Encounters")
        if (newEffect && Verbose) Serial.println("Four note ditti, lights and sound (of melody in 'Close Encounters'");
        genTone(522, 200);
        if (catchBtn()) break;
        genTone(1044, 400); //
        if (catchBtn()) break;
        genTone(830, 400); //
        if (catchBtn()) break;
        genTone(312, 800); //
        if (catchBtn()) break;
        delay(800);
        break;
      case 3:   //  The 5 note Sequence from "Close Encounters of the Third Kind"
        if (newEffect && Verbose) Serial.println("The 5 note Sequence from 'Close Encounters of the Third Kind'");
        // the divider changes the octave
        ledsAll(4);  //  wound-be Violet
        genTone(3729 / 4, 400); // B-
        if (catchBtn()) break;
        ledsAll(1);  //           Red
        genTone(4186 / 4, 600); // C
        if (catchBtn()) break;
        ledsAll(3);  //  wound-be Orange
        genTone(3322 / 4, 900); // A-
        if (catchBtn()) break;
        ledsAll(2);  //           Yellow
        genTone(1661 / 4, 600); // A-/2
        if (catchBtn()) break;
        ledsAll(15);  //           White
        genTone(2489 / 4, 1600); // E-
        clearDisp();
        scanBtnWait(1600);
        break;
      case 4:
        if (newEffect && Verbose) Serial.println("Cylon Centurion helmet scan (1978)");
        // ----------------------   a Cylon Centurion helmet scan (also interesting would be Kit 2000 grill display)
        //sweeps a group of LEDs back&forth with a Wiz-a-Wom ...     ~1sec to cross
        // every 1/6 sec move lite, 1st right then left accross display, & appears to overrun
        // hold for 1 counts on the ends
        // with sweeping noise
        msec = millis(); // use instead of myMilli so as not to be interupt ISR dependant
        if (tomove && (msec % (1000 / (nleds)) < 100))  { // cadence to result in ~1sec across displ
          tomove = false;
          //if (!(cnt==0 || cnt==nleds+1)) {  // 1 cnt pause on ends
          if (!(cnt < 1 || cnt > nleds)) { // setup 2 cnt pause on ends, special to work with only 4 leds  !!!
            lit[cnt] = lit[cnt + dir] = true;
            //refreshWait(50);          // allow for dim tail
            lit[cnt - dir] = false; //
          }
          cnt = cnt + dir;
          if (cnt < 0) {
            cnt = 1;
            dir = 1;
          }
          if (cnt > (nleds + 1)) {
            cnt = nleds;
            dir = -1;
          }
          //                if (cnt<-1) {cnt=1; dir=1;}               // for use to work with 4 leds  !!!
          //                if (cnt>(nleds+2)) {cnt=nleds; dir=-1;}   //  "
          refreshWait(1);
        }
        if (msec % (1000 / (nleds)) > 100) tomove = true;
        // produce sound of WhiZ-a-woom
        if ((dir == 1 && cnt == 1)         || (dir < 0 && cnt == nleds))      genTone(150, 20); // Wiz
        if ((dir == 1 && cnt == 2)         || (dir < 0 && cnt == (nleds - 1)))  genTone(200, 20); // -a-
        if ((dir == 1 && cnt > 2 && cnt < 5) || (dir < 0 && cnt > (nleds - 4) && cnt < (nleds - 1)))  genTone(100, 20); // woom
        break;

      case 5:
        if (newEffect && Verbose) Serial.println("KITT-2000 grill display scan (1982)");
        KnightRider();  // KITT-2000 grill display scan
        break;
      case 6:    //  British Hilo siren
        if (newEffect && Verbose) Serial.println("British Hilo siren");
        beepTone(1600, 600);
        beepTone(600, 600);
        break;
      case 7:    // EU Ambulence implementation
        if (newEffect && Verbose) Serial.println("EU Ambulence (uses Cords)");
        playCord(1550, 1600, 1650, 600);
        playCord(600, 630, 660, 600);
        break;
      case 77:    // alternate EU Ambulence implementation
        if (newEffect && Verbose) Serial.println("alternate EU Ambulence implementation");
        playCord(1400, 1800, 2400, 600);
        playCord(500, 650, 800, 600);
        break;
      case 8:    //  Police siren
        if (newEffect && Verbose) Serial.println("Police siren");
        genSndEffect(1500, 4000, 10, 10, 100);
        genSndEffect(4000, 1500, 10, 10, 270);
        break;
      case 9:    //  FireTruck wail
        if (newEffect && Verbose) Serial.println("FireTruck wail");
        //genSndEffect(500,2500,10,10,1100);
        //genSndEffect(2500,500,10,4,3300);
        genSndEffect(500, 2500, 10, 20, 1100);
        genSndEffect(2500, 500, 20, 1, 3300);
        break;

      case 10:   // futuristic effect, think flamboyant flying saucer. simplied for code size
        if (newEffect && Verbose) Serial.println("Flying Saucer");
        cnt = (cnt + 1) % 32;
        n = (cnt < 16) ? cnt : 31 - cnt;
        litLED = n % nleds + 1;
        genTone(400 + 100 * n, 100);
        break;

      case 11:       // Dialing a touch-Tone telephone
        if (newEffect && Verbose) Serial.println("Dialing a touch-Tone telephone");
        for (int i = 0; i < sizeof(t); i++) {
          litLED = t[i];
          if (litLED) phoneTone(litLED);
          else        refreshWait(70);
          refreshWait(90 + random(90));
        }
        break;

      case 12:
        if (newEffect && Verbose) Serial.println("Telephone Busy Signal");
        genSndEffect(1500, 1500, 10, 20, 1000); // busy signal-ish ..
        if (scanBtnWait(500)) break;
        genSndEffect(1500, 1500, 20, 10, 1000);
        if (scanBtnWait(1000)) break;
        genSndEffect(1500, 1500, 1, 10, 1000); // would-be ..
        if (scanBtnWait(500)) break;
        genSndEffect(1500, 1500, 10, 1, 1000);
        scanBtnWait(2000);
        break;

      case 13:       // Cardiac EKG signal monitor
        if (newEffect && Verbose) Serial.println("Cardiac EKG signal monitor");
        Sweep_around();
        break;

      case 14:    // Wandering Walk about the piano keys, akin to Free (avant-garde) Jazz
        if (newEffect && Verbose) Serial.println("Wandering Walk about the piano keys (Free Jazz)");
        litLED = 0;
        rnum = random(6);
        if (keyN < 14) rnum++; // tend to go back towards middle
        if ((rnum == 0 || rnum == 6) && random(2)) rnum = random(1, 6); // 1/2 the odds for the extreams
        keyN = (keyN - 3) + rnum;  // means +/- 0-3 keys
        if (keyN < 6) keyN = 6 + (6 - keyN); // bounce back into range
        if (keyN > 24) keyN = 24 - (keyN - 24);
        if (lastN == keyN) refreshWait(random(2) * 30); // 1/2 time extend note else make two separtate
        lastN = keyN;
        litLED = keyN % nleds + 1;
        dur = (random(2))? dur : 400*(random(1,3));
        playNote(keyN, dur);
        Serial.println(keyN);
        break;
      case 145:    // random piano notes
        if (newEffect && Verbose) Serial.println("random piano notes");
        litLED = 0;
        keyN = random(7, 24);
        if (keyN < 8) keyN = keyN - random(7);
        else if (keyN > 22) keyN = keyN + random(7);
        else litLED = ((keyN - 8) / 2) + 1;
        playNote(keyN, 400);
        break;

      case 15:
        if (newEffect && Verbose) Serial.println("Special Game Effect #16");
        genSndEffect(50, 250, 10, 10, 1500);
        if (scanBtnWait(250)) break;
        catchBtn();
        genSndEffect(250, 50, 10, 10, 1500);
        if (scanBtnWait(500)) break;
        catchBtn();
        //genSndEffect(2400,2400,10,10,200);
        genSndEffect(8000, 1000, 10, 10, 1600);
        scanBtnWait(1000);
        break;
      case 16:
        if (newEffect && Verbose) Serial.println("Special Effect #11");
        setColor(1);
        genSndEffect(2000, 6000, 20, 20, 500);
        delay(0); if (catchBtn()) break;   // ?
        setColor(4);
        genSndEffect(1000, 900, 10, 10, 300);
        refreshWait(500);
        setColor(0);
        scanBtnWait(1000);
        break;
      case 17:
        if (newEffect && Verbose) Serial.println("Special Effect #18");
        setColor(1);
        genSndEffect(5000, 5000, 20, 20, 100);
        if (scanBtnWait(500)) break;
        setColor(2);
        genSndEffect(2000, 1000, 20, 20, 500);
        if (catchBtn()) break;
        setColor(0);
        refreshWait(20);
        setColor(1);
        genSndEffect(5000, 5000, 20, 20, 100);
        if (catchBtn()) break;
        setColor(2);
        genSndEffect(2000, 1000, 20, 20, 500);
        setColor(0);
        scanBtnWait(300);
        break;

      case 18:
        if (newEffect && Verbose) Serial.println("Special Effect #19");
        // use of tritone span 11:F4-14:B5 (or 18:F5-21:B6)for police siren
        //playNote(11, 125); playNote(12, 125); playNote(13, 125); playNote(14, 125);
        genSndEffect(basicKeyNotes[18], basicKeyNotes[21], 10, 10, 400);
        refreshWait(200);
        genSndEffect(basicKeyNotes[18], basicKeyNotes[21], 20, 20, 400);
        refreshWait(400);
        break;
      default:
        delay(200);
        buZZ(100);
    }

    if (!BtnPressed) scanBtns();
    if (btn1Pressed) effect = 1;
    if (btn2Pressed) effect = (effect == 1) ? numE : effect - 1; // step <.> up/down the effects
    if (btn3Pressed) effect = (effect % numE) + 1;
    if (Btn) clearDisp();
    if (BtnPressed) {
      Serial.print("Effect: ");
      Serial.println(effect);
    }

    newEffect = false;
    if (btn2 | btn3) {
      enbleWait = true;
      newEffect = true;
      setColor(5);    // indicate we are moving on the next Effect
      wait_BtnRelease();
      setColor(0);
    }

    if (BtnPressed) scanBtns();
    if (enbleWait) refreshWait(1);
  }
}

void  Timer()  { //  ===============================   Timer ... flash & ticks secs and trumpets minutes (+ Stop-Watch functions)
  int  pos, minCnt, secs, n;
  bool toggle, timing, paused;
  unsigned int mCnt;
  unsigned long mics;
  int trimAdj;

#ifdef RP2040
  trimAdj = 13; // 16.6 shortens interval by 1/60th (making 1sec difference in a minute)
#else
  trimAdj = 168; // imperical rate adjustment, re given device, for timing accuracy sake (MCU,overhead,V,temp. ?dependant)
#endif

  //w/ISR  unsigned long mStart;

  int ver = 1;  // ver 1 uses delay() and is adjusted for @(mCnt%500 == 0) below
  // ver 2 uses a 1 millisecond ISR interurpt routine
  while (1) {
    clearDisp();
    //      xdimEnabled = true;
    //w/ISR      mStart=myMillis;
    mCnt = 0;
    minCnt = 0;
    pos = 1;
    toggle = false;
    paused = false;
    timing = true;
    while (timing) {
      secsIdle = 0; // block Idle timeout
      if (ver == 2) {
        //w/ISR          while (mCnt == (myMillis-mStart)) {}  // wait for ISR to change 'myMillis'
        //w/ISR          mCnt=myMillis-mStart;
      } else {  // ver 1 1 ms wait while doing screen refresh
        //delay(1);         // WB more accurate (~61 sec minute) but lose dimming and second's ticks
        refreshWait(1);    // slowser timing (~64 sec minute)
        // but is adjusted for below @(mCnt%500 == 0)
        mCnt++;
      }
      //refreshDisp();

      scanBtns();
      if (ESC) return;
      if (btn1Pressed) {
        timing = false;   // Restart
        break;
      }
      //w/ISR        if (btn2Pressed) ver=2;     // switch to ver=2 using the ISR
      if (btn4Pressed) paused = !paused;  // toggle in/out of a paused state
      if (paused) {
        wait_BtnRelease();
        while (paused) {
          // display time so far by way of a combination of bright and flashing lites
          secs = mCnt % 10000 / 1000; // seconds into this 10 sec period
          if (0 == secs) {
            spriteOne = pos;
            noBtnDelay(1000);
          } else {
            for (byte s = 1; s <= secs; s++) {
              spriteOne = pos;  noBtnDelay(50);
              spriteOne = 0;  noBtnDelay(200);
            }
          }
          spriteOne = 0;
          noBtnDelay(1000);   // wait upto  1 sec, given no buttons pressed
          //            if (btnState>0) {
          if (btn1Pressed || ESC) {
            timing = false;   // Restart or Exit
            break;
          }
          if (Btn) {
            paused = false;
            wait_BtnRelease();
          }
        }
      }
      if (ESC) return;

      if (mCnt % 500 == 0) { // once a second lite then noLite then adv the sprite
        toggle = !toggle;
        if (toggle) {
          spriteOne = 0;
        } else {
          //            pos = pos%nleds + 1;
          spriteOne = pos;
        }
      }

      //  -------------  Adjust for your MCU and setup      !!!!!!!!!!!!!!!!!!!!!!!!!
      // imperical rate adjustment,  mCnt+=16.6 shortens interval by 1/60th (making 1sec difference in a minute)
      if (ver == 1 && mCnt % 1000 == 500)  mCnt += trimAdj;

      if (mCnt % 10000 == 0) {  // little beep every 10 seconds
        spriteOne = pos;  // be sure it is lit
        //          if (mCnt!=60000) {
        beep(20);
        mCnt += 20;     // adj for time spent
        //          }
        pos = pos % 6 + 1; // 6*10 for 60secs / Min.
        //na          idleCnt=0;  // keep from timing out
      }
      if (mCnt >= 60000) { // every Minute
        //w/ISR          mStart=myMillis;
        clearDisp();
        mCnt = 1;
        minCnt++;   // inc Minute count
        Loud = true;
        for (n = 1; n <= minCnt; n++) {
          dim[n] = true;
          spriteOne = (n - 1) % nleds + 1;
          beep(75); refreshWait(125);
          mCnt += 200;  // account for time spent
        }
        Loud = false;
        litLED = spriteOne; // add extra brightness on the Led of the current Minute
        pos = 1 + mCnt / 10000;
      }
    }
  }
}


void Metronome() {  // ================ Emulates a metronome. Mode selection: 1/4-4/4 time.
  // specs: AVR: ~ Beat time Deviation: 4ms, BPM +/-2ms     RP2040: beat Devi: 1ms,  BPM +/-1ms
  int bpm = 100; // Range goes 20-240, Starts at 100bpm, good for "row row ..." or CPR
  byte count, beats = 0, bcnt;
  int ams;
  int st, pos = 1, dir = 1;
  unsigned long stime = 0;
  bool bpmChg = true;
  int hiv = 0, lowv = 99; // high & low values

  count = get_Mode(4); // select 1/4 2/4 3/4 or 4/4 time

  while (!ESC) {

    if (BtnPressed || bpmChg) {
      if (btn1Pressed) {
        bpm = (bpm % 10 == 0) ? bpm - 10 : 10 * (bpm / 10); // dec -10
        if (bpm < 20) bpm = 240;
      }
      if (btn2Pressed) bpm = bpm - 1;
      if (btn3Pressed) bpm = bpm + 1;
      if (btn4Pressed) {
        bpm = (bpm % 20 == 0) ? bpm + 20 : 20 * (bpm / 20 + 1); // inc +20
        if (bpm > 240) bpm = 20;
      }

      st = (60000 / bpm) / nleds - 1; // coarse LED step time, as there are overhead delays: -1 & fine adj later on the fly
      stime = millis();
      bcnt = 0; t0 = millis(); // reset test timing, as bpm has chgd
      pos = 1, dir = 1;
      hiv = 0; lowv = 99;
      if (bpm == (20 * int(bpm / 20)) && bpm >= 40 && bpm <= 180) {
        spriteTwo = (bpm - 20) / 20;   // when at multiple of 20 HighLight it
      } else spriteTwo = 0;

      Serial.print("bpm="); Serial.print(bpm);
      Serial.print("  StepTime="); Serial.println(st);
    }
    bpmChg = false;
    if ((msCnt - lastChg) > 1500)  spriteTwo=0;

    if ((millis() - stime) >= st) { // time to move needle
      stime = millis();
      pos = (pos + dir) % (nleds + 2);

      if (pos == 0 || pos > nleds) { // ------- end of each beat period
        ++bcnt;
        ams = (bcnt * 60000L / bpm) - (millis() - t0); // fine adj. as needed sb: 0-9 due to process rounding
        if (ams >= 0) delay(ams); // note: beat-to-beat time remains the same within ~1msec, at least with RP2040
        stime = millis(); // reset step timer
        ///// diag/testing ...
        if (ams > hiv && bcnt > 0) hiv = ams;
        if (ams < lowv && bcnt > 0) lowv = ams;
        /////

        if (bcnt >= bpm) { // ---- have done #BPM for a minute

          ////////// for diag/calc timing testing ...
          Serial.print(bpm); Serial.print("bpm,  deviation(ms):"); Serial.print((hiv - lowv) / 2.0); Serial.print(",   millisecs #BPM used as a 'minute': ");
          Serial.println(millis() - t0); // sb: 60,000 will be +/-1
          // auto -- for testing
          //if (bpm>20) bpm--;
          //bpmChg=true;
          hiv = 0; lowv = 99;
          //////////

          bcnt = 0;
          t0 = millis();
        }
      }

      if (pos > nleds) {
        pos = nleds;
        dir = -1;
      } else if (pos == 0) {
        pos = 1;
        dir = 1;
      }
      dimLED = pos;
      if ((pos == nleds / 2) || (pos == (nleds / 2 + 1))) brightOne = dimLED;
      else brightOne = 0;
      // crossing the middle tick ... "beat"
      if ((dir == 1 && pos == (nleds / 2 + 1)) || (dir == -1 && pos == (nleds / 2))) {
        beats++;
        if (beats == count) {
          bop(25);  // audio time must be <29
          beats = 0;
        }     // try to make a 'tick' sound
//        else beep(12);
//        else {bEEp(3); bop(2,1);beep(4);}  // (was 3 2,2 6)
        else beepTone(2000,8);
      }
    }

    refreshWait(1);
    scanBtns();
    secsIdle = 0; // thwart idle timer
  }
}

// ------- used by readVoltages()
long readVcc() {

#if defined(RP2040)
  return (0); // does not support or unaware of how
#else
  // Read 1.1V reference against AVcc
  // set the reference to Vcc and the measurement to the internal 1.1V reference
  //choose the right register settings for the processor in use
#if defined(__AVR_ATmega32U4__) || defined(__AVR_ATmega1280__) || defined(__AVR_ATmega2560__)
  ADMUX = _BV(REFS0) | _BV(MUX4) | _BV(MUX3) | _BV(MUX2) | _BV(MUX1);
#elif defined (__AVR_ATtiny24__) || defined(__AVR_ATtiny44__) || defined(__AVR_ATtiny84__)
  ADMUX = _BV(MUX5) | _BV(MUX0);
#elif defined (__AVR_ATtiny25__) || defined(__AVR_ATtiny45__) || defined(__AVR_ATtiny85__)
  ADMUX = _BV(MUX3) | _BV(MUX2);
#else
  ADMUX = _BV(REFS0) | _BV(MUX3) | _BV(MUX2) | _BV(MUX1);  // this one for Nano, ATmega-328
#endif

  delay(2); // Wait for Vref to settle typical: +/-.03 (1) is good @ +/-.06
  ADCSRA |= _BV(ADSC); // Start conversion
  while (bit_is_set(ADCSRA, ADSC)); // measuring

  delay(2); // settling
  uint8_t low  = ADCL; // must read ADCL first - it then locks ADCH
  uint8_t high = ADCH; // unlocks both

  long result = (high << 8) | low;

  result = 1125300L / result; // Calculate Vcc (in mV); 1125300 = 1.1*1023*1000
  //  result = 1126400L / result; // Calculate Vcc (in mV); 1126400 = 1.1*1024*1000
  //  result = 2619000L / result; // Calculate Vcc for ProMicro??; 2619000 = 2.56*1023*1000

  return result; // Vcc in millivolts
#endif
}

void readVoltages() {  // B1: for battery voltage (or USB-Vcc volts),   B2: ADC_a,   B3 ADC_b
// Volts are given as number of LEDs lit for whole 'V's, beep for 'period' and LEDs counted off for tenths of a volt. 
  long Vcc = 0;
  int V, d;
  float maxV=5.0, v;
  int Ain = 0;
  int fullScale=1024;
//  bool pullUp=false;

#ifdef ProMicro
  Ain = ADC_a;
#endif
#ifdef RP2040
  maxV=3.3;
  fullScale=4096;
#endif

  while (!btn4) { // cycle till exit
    clearDisp();
    refreshWait(500);
//if (btn2Pressed) {
//  if (pullUp) pinMode(A0, INPUT_PULLUP);
//  else pinMode(A0, INPUT);
//  pullUp=!pullUp;
//}
    if (btn1) Ain = 0;  // btn1-3 will switch which analog line is read
    if (btn2) Ain = ADC_a;
    if (btn3) Ain = ADC_b;
    if (Ain == 0) {
      Vcc = readVcc(); // supply voltage in milliVolts
      v = Vcc / 1000.0;
      // print out the voltage (Vcc) value
      Serial.print("Vcc = "); Serial.print(Vcc / 1000); Serial.print("."); Serial.print(Vcc % 1000);
      // USB supplied Vcc should be > 4.5 volts; LiIon battery supplied should be <4.0
      // a Vcc reading of 4.0-4.5 indicated some kind of issue
      if (Vcc < 4300) {
        v = (Vcc + 300) / 1000.0; // +300 for the 0.3 volts across the 1N5817 diode
        Serial.print("    Battery ~= "); Serial.print(v);
      }
      Serial.println();
    } else if (Ain == ADC_a) {
      v = maxV * analogRead(ADC_a) / fullScale;
      Serial.print("ADC-a voltage = "); Serial.println(v);
    } else {
      v = maxV * analogRead(ADC_b) / fullScale;
      Serial.print("ADC-b voltage = "); Serial.println(v);
    }
    V = int(v); // form V.d  V=whole Volts, d= 1/10ths
    d = int((v - V) * 10.0);
    if (V >= 1) {
      for (int i = 1; i <= V; i++) {
        brt[i] = true;
      }
    }
    refreshWait(350);
    beep(30);
    showLevel(d);

    DispScanWait(1000); // repeat every 1 second +
    if (Btn) buZZ(40); // ack you noted the btn press
  }

}

// =====================================================================
// ***************************  Group 6  *******************************
// ----------- Morse Code Practice & Exercise samples ------------------

//==================  funtional building blocks to support Morce Code I/O
// ---------- Morse code support functions ------------
byte fetchMorseChar(bool echo) {
  if (oneKey) return (fetchMorse_1Key(echo)); // uses One button keying, based on duration of presses
  //  else        return(fetchMorse_2Key(echo));  // uses basic Two button keying
  else        return (fetchMorse_22Key(echo)); // uses basic Two button keying improved p timing
  //  else        return(fetchMorse_2A_Key(echo));  // uses Two button with fixed timing & Auto keying
}

byte fetchMorse_1Key(bool echo) { // get one char (ditdah set) using single button keying (straight CW key)
  byte mChar = 0;
  int  p = 0;
  unsigned long pt;
  byte dit;
  bool toggle = false;
  static bool avgdit = true;
  static int minDit = 0, maxDah = 0;
  static byte ditCnt = 0, dahCnt = 0;

  // update timing used prior to each whole character
  avgPeriod = pAveraging / 4; // the running average maintains 4*resolution desired
  ditPeriod = avgPeriod / 2; // avgPeriod=2*ditPeriod  aka  the average between dit and dah period

  while (!(btn1 || btn3 || btn4)) {
    scanBtns();  // wait for button press
    refreshWait(1);
  }
  while (btn1) {
    p = 0;
    pt = millis();  // time the period
    while (btn1) {
      dit_dah(2);   // output audio while key pressed (done for 2 sounds better than for 1)
      scanBtns();
    }
    p = millis() - pt;

    //    dit = (p<avgPeriod);
    dit = (p < (9 * avgPeriod / 10)); // makes 1.8 ditPeriod the break point between a dit & a dah
    if (dit) ditCnt++; else dahCnt++;
    if (mChar == 0) { // setup first bit
      mChar = (dit) ? 0x01 : 0xFE;
    } else {    // shift in other bits
      mChar = (mChar << 1) | dit;
    }

    // track min. dit & max. dah timing
    if (dit && minDit == 0) minDit = p;
    if (dit && minDit > p) minDit = p;
    if (!dit && maxDah < p) maxDah = p;

    /***    // add into the running average 'pAveraging' (but not ones from left field)
      avgdit=dit;
        if (avgdit && dit && (2*p>ditPeriod)) {
          pAveraging = (7*pAveraging/8) + p;
          avgdit = !avgdit; // average in = # of dits & dahs
          Serial.print("diT  "); Serial.println(p);
        }
        if (!avgdit && !dit && p<(5*ditPeriod)) {
          pAveraging = (7*pAveraging/8) + (p/3);
          avgdit = !avgdit;
          Serial.print("dahT "); Serial.println(p/3);
        }
    ***/
    p = 0;
    while (Btn == 0) { // wait for additional dit-dahs (or other)
      scanBtns();
      delay(1);
      //      if (++p>(7*ditPeriod/3)) {  // require only 2.3 (3 expected be the average)
      if (++p > (2 * ditPeriod)) {
        break;          // done with this char
      }
    }
  }
  if (btn3 || btn4) return 0;
  if (echo) { // echo the dits & dahs
    set_didaStr(mChar, didaStr);
    Serial.print("  ");
    Serial.print(didaStr); Serial.print(" ");
  }
  if (prtChr) Serial.print(dida2Ascii(mChar));  // always output the alpha character (patched)

  // add into the running average 'pAveraging' (but not ones from left field)
  avgdit = dit;
  //    if (minDit>0 && minDit<ditPeriod/3) {Serial.print("  ");Serial.print(minDit); Serial.print(" > "); minDit=ditPeriod/3;  Serial.println(minDit);}
  //    if (maxDah>0 && maxDah>ditPeriod*8) {Serial.print("  ");Serial.print(maxDah); Serial.print(" < ");  maxDah=ditPeriod*8; Serial.println(maxDah);}
  if (minDit > 0 && minDit < ditPeriod / 3) {
    minDit = ditPeriod / 3;
  }
  if (maxDah > 0 && maxDah > ditPeriod * 9) {
    maxDah = ditPeriod * 9;
  }

  if (pAdj) {    // ditdah period adjment enabled
    if (minDit && ditCnt > 2 && minDit < (3 * ditPeriod / 2)) { // adjust pAveraging per recent dits
      pAveraging = (7 * pAveraging / 8) + minDit;
      avgdit = !avgdit; // average in = # of dits & dahs
      //      Serial.print("  diT  "); Serial.println(minDit);
      minDit = ditCnt = 0;
    }
    if (maxDah && dahCnt > 2 && maxDah / 2 > ditPeriod) { // adjust pAveraging per recent dahs
      pAveraging = (7 * pAveraging / 8) + (maxDah / 3);
      avgdit = !avgdit;
      //      Serial.print("  dahT "); Serial.println(maxDah/3);
      maxDah = dahCnt = 0;
    }
  }

  //  if (DEBUG) {
  //    Serial.print("   Morse Char: ");  Serial.print(mChar);
  //    Serial.print("  0x"); Serial.print(mChar, HEX);
  //    Serial.print("    ave Period: ");  Serial.println(pAveraging/4);
  //  }

  return (mChar);
}

// -----------------------
byte fetchMorse_2Key(bool echo) { // using Two button keying (or a dual lever paddle)
  return (fetchMorse_22Key(echo));
}
// -----------------------
byte fetchMorse_22Key(bool echo) { // using Two button keying (or a dual lever paddle)
  // dit & dah differ in sound, but duration is simply based on button press duration.
  // TODO: Not throurghly valided, may need/want adjustments to period timing
  //  static short int ditPeriod;
  byte mChar = 0;
  int  p = 0, p2 = 0, pCnt = 0;
  bool dit;
  byte msn;
  bool toggle = false;
  byte ditdaCnt = 0;

  avgPeriod = pAveraging / 4; // the running average maintains 4*resolution desired
  ditPeriod = avgPeriod / 2;  // avgPeriod=2*ditPeriod  aka  the average between dit and dah period

  while (!Btn) {
    scanBtns();  // wait for button press
    refreshWait(1);
  }
  while (btn1 || btn2) {
    ditdaCnt++;
    p += p2;
    dit = btn1;
    t0 = millis();
    while (Btn) {
      scanBtns();
      if (btn1) {   // produce sound while pressing
        brightOne = 7;
        ditAudio(1);
      } else {
        brightOne = 7; litLED = 8;
        dahAudio(2);
      }
      toggle = !toggle;
      p++;
      clearDisp();
    }

    if (mChar == 0) { // setup first bit
      mChar = (dit) ? 0x01 : 0xFE;
    } else {    // shift in other bits
      mChar = (mChar << 1) | dit;
    }
    // count it into the running average 'pAveraging'
    if (dit) pCnt += 2;
    else pCnt += 3; // compromise, as it takes 2 to key but 4 to reproduce

    p2 = 0;
    while (Btn == 0) { // wait for additional dit-dahs
      scanBtns();
      delay(1);
      if (++p2 > (2 * ditPeriod)) {
        break;      // done here, end of a char
      }
    }
    if (dit) {  // just had a dit + an inter period
      pAveraging = (3 * pAveraging / 4) + (millis() - t0); // provide tracking of avg. for 2p
      avgPeriod = pAveraging / 4;
      ditPeriod = avgPeriod / 2;
    }
  }

  if (btn3 || btn4) return 0;
  if (echo) {
    set_didaStr(mChar, didaStr);
    Serial.print("  ");
    Serial.print(didaStr); Serial.print(" ");
  }
  if (prtChr) Serial.print(dida2Ascii(mChar));  // always output the alpha character

  if (DEBUG) {
    Serial.print("   Morse Char: ");  Serial.print(mChar);
    Serial.print("  0x"); Serial.print(mChar, HEX);
    Serial.print("    ave Period: ");  Serial.println(2 * ditPeriod);
    Serial.print(" Traking Period: ");  Serial.println(pAveraging / 4);
  }

  return (mChar);
}

// -----------------------
char dida2Ascii(byte ditdahs) {
  byte ncode, ascii, msb, ndidas, i;
  const char nc2A[]  = "~EAIWRUSJP\nL~FVH~~QZYCXBOGKDMNT~";
  //  const char dida5[] = "09 8   7 (   /=61    + &2   3 45";
  //  const char dida6[] = " '   @ :  . ,\"    )?! $       - ";
  const char dida5[] = "09~8~~~7~(~~~/=61~~~~+~&2~~~3~45";
  const char dida6[] = "~'~~~@~:~~.~,\"~~~~)?!~$~~~~~~~-~";
  static bool lastP = false; // last char processed was a () type

  //Serial.print(" ditdah code: ");  Serial.println(ditdahs);
  if (ditdahs == 0) return 0;

  // strip off all most significant extra 1s bits
  i = 1;
  msb = (ditdahs & 0x80);
  while ((ditdahs << i & 0x80) == msb) {
    i++;
  }
  ndidas = 8 - i;
  ncode  = ditdahs & 0x1F;
  // decode according to its length
  if (ndidas < 5)       ascii = nc2A[ncode];
  else if (ndidas == 5) ascii = dida5[ncode];
  else if (ndidas > 5)  ascii = dida6[ncode];

  if (!lastP) {
    if (ascii == ')' && pOpen) ascii = '(';
    if (ascii == '(') pOpen = false;
    //was    if (ascii==')' || ascii=="\n") pOpen=true;
    if (ascii == ')' || ascii == 0x0A) pOpen = true;
  }
  if (ascii == '(' || ascii == ')') lastP = true;
  else lastP = false;
  return (ascii);
}

// -----------------------
byte Ascii2dida(char Achar) {
  byte ncode, i;
  const char nc2A[] = "~EAIWRUSJP~L~FVH~~QZYCXBOGKDMNT~~";
  int alen;

  Achar = toupper(Achar);
  //  move i-index past padding bits
  for (i = 1; i < 32; i++) {
    if (nc2A[i] == Achar) break;
  }
  if (i < 32) {
    ncode = (i < B10000) ? i : i + B11100000;
  } else {  // handle #s, punc. & spec. chars
    if (Achar == '[' || Achar == '{')  Achar = '(';
    if (Achar == ']' || Achar == '}')  Achar = ')';
    alen = sizeof(nonAlpha);  // sizeof(char)=1
    for (i = 0; i < alen; i++) {
      if (nonAlpha[i] == Achar) break;
    }
    ncode = nonAlpha_dida[i];
  }
  return (ncode);
}

// --------------------------------------------
void set_didaStr(byte ditdahs, char ddStr[]) {  // string returned in global var: 'didaStr'
  // all MSBits that are the same are discarded, whats left is 1=dit 0=dah, MSbit is first in time, LSB last
  byte i = 1;
  byte bitState;

  if (ditdahs == 0) return;

  while ((ditdahs << i & 0x80) == (ditdahs & 0x80)) i++;
  int c = 0;
  for (int b = i; b < 8; b++) {
    if (ditdahs << b & 0x80) { // this bit is a 'dit'
      ddStr[c++] = '.';
    } else {  // its a dah
      ddStr[c++] = '-';
    }
  }
  ddStr[c++] = 0;
}

// --------------------------
// produce a requested number of randomly sorted indexes
void ranIndexes(byte rIndexes[], byte cnt) {
  char wi[80];  // working set of indexes
  int i, indx;
  byte tmp;

  for (int c = 1; c <= cnt; c++) wi[c] = c; // preset the collection of indexes

  for (int s = cnt; s > 0; s--) { // simple fast random sort
    i = random(1, s);
    rIndexes[s] = wi[i];
    wi[i] = wi[s];
  }
  // ensure there are not any consecutive entries, not for randomness but AB_exersice desired behavior
  for (i = 1; i < cnt; i++) {
    if (rIndexes[i] + 1 == rIndexes[i + 1]) {
      tmp = rIndexes[i];
      rIndexes[i] = rIndexes[i + 1];
      rIndexes[i + 1] = tmp;
    }
    //Serial.print(rIndexes[i]); Serial.print(' ');
  }
  //Serial.println(rIndexes[i]);
}

// ================== Morse Code Practice & Exercises applets =================
// -----------------------
#include <avr/pgmspace.h>
char buf[30];

//  words either sized 500 or 900
//const PROGMEM char wordStr[] = "    ,Quick,brown,fox,jumps,over,the,lazy,dog,Pack,my,box,with,five,dozen,liquor,jugs,Cozy,lynx,gives,smart,squid,asks,for,pen,Hive,of,boxing,wizards,jump,quickly,Garden,Gone,Wild,But,that's,too,okay,very,been,guess,time,going,into,those,did,work,other,I've,even,our,any,yes,his,had,were,where,your,she,which,will,say,who,make,man,than,could,state,only,new,year,come,take,these,use,first,may,much,give,over,most,find,day,also,after,way,many,must,look,before,great,back,long,down,should,through,own,two,here,last,want,didn't,something,being,still,put,never,made,went,thought,again,off,why,can't,quite,actually,might,world,life,another,used,night,old,same,always,next,came,doing,week,house,none,each,feel,seem,high,place,little,nation,hand,tell,write,become,show,both,between,need,call,develop,under,move,general,school,begin,while,part,turn,real,leave,point,form,child,few,small,since,against,ask,late,interest,large,person,end,open,public,follow,during,present,without,hold,govern,around,possible,head,consider,word,program,problem,however,lead,system,set,order,eye,plan,run,keep,face,fact,group,play,stand,increase,early,course,change,help,line,send,listen,sound,audio,quality,xmitt,receive,morse,code,electronics,tech,class,STEM,arduino,kids,email,text,phone,in-box,o'clock,wifi,video,games,arm,leg,foot,LOL!,?Who?,(408),34-26-48,KI7NEW,(3+1=4),3.1416,CQCQ,8/27/49,6:15am,Junk,jumper,jewlery,jam,pajamas,adjust,equal,quote,vote,voltage,box,fix,max,tax,six,taxes,zero,lazy,zone,crazy,Amazon,Z-axis,home,page,has,search,free,i10n,site,news,contact,business,web,view,online,click,its,service,price,date,top,list,name,health,music,buy,data,product,post,her,city,add,policy,number,such,please,available,support,message,best,software,info,rights,books,links,review,years,privacy,book,items,company,read,user,said,research,university,mail,full,map,reviews,days,management,united,hotel,item,center,eBay,store,travel,comments,development,report,member,details,terms,hotels,type,local,using,results,office,education,national,car,design,posted,internet,address,community,within,states,area,dvd,shipping,reserved,subject,forum,family,based,black,check,special,prices,website,index,women,sign,file,link,today,tech.,south,case,project,pages,version,section,found,sports,related,security,county,American,photo,game,members,power,care,network,computer,three,total,following,download,him,per,access,north,resource,current,posts,big,media,law,control,water,history,pictures,size,art,personal,including,guide,shop,directory,board,location,white,rating,rate,government,children,USA,return,students,shopping,account,times,sites,level,digital,profile,previous,events,love,john,main,hours,image,department,title,descripe,non,insurance,shall,property,money,every,listing,content,country,private,visit,save,tools,low,reply,customer,compare,movies,include,college,value,article,york,card,jobs,provide,food,source,author,different,press,learn,sale,print,job,Canada,process,teen,room,stock,training,credit,join,science,men,category,advanced,west,sales,english,left,team,estate,condition,select,windows,photos,party,";
const PROGMEM char wordStr[] = "    ,that's,too,okay,very,been,guess,time,going,into,those,did,work,other,I've,even,our,any,yes,his,had,were,where,your,she,which,will,say,who,make,man,than,could,state,only,new,year,come,take,these,use,first,may,much,give,over,most,find,day,also,after,way,many,must,look,before,great,back,long,down,should,through,own,two,here,last,want,didn't,something,being,still,put,never,made,went,thought,again,off,why,can't,quite,actually,might,world,life,another,used,night,old,same,always,next,came,doing,week,house,none,each,feel,seem,high,place,little,nation,hand,tell,write,become,show,both,between,need,call,develop,under,move,general,school,begin,while,part,turn,real,leave,point,form,child,few,small,since,against,ask,late,interest,large,person,end,open,public,follow,during,present,without,hold,govern,around,possible,head,consider,word,program,problem,however,lead,system,set,order,eye,plan,run,keep,face,fact,group,play,stand,increase,early,course,change,help,line,send,listen,sound,audio,quality,xmitt,receive,morse,code,electronics,tech,class,STEM,arduino,kids,email,text,phone,in-box,o'clock,wifi,video,games,arm,leg,foot,LOL!,?Who?,(408),34-26-48,KI7NEW,(3+1=4),3.1416,CQCQ,8/27/49,6:15am,Junk,jumper,jewlery,jam,pajamas,adjust,equal,quote,vote,voltage,box,fix,max,tax,six,taxes,zero,lazy,zone,crazy,Amazon,Z-axis,BTW,R=roger,TU:thank you,RTU:R+TU,TNX:thanks,QRA:name?,RST:Sig.Report,CQ:contact call,AGN=again,ANT=antenna,DX:distant signal,FB:fine business,PSE=please,GA:go-ahead,GM[good morning],GE:good evening,HI:laughter,NR:number,OM:old-man,PWR=power,WX:weather,73:best regards,QRL \"BUSY\",QRM \"INTERFERE\",QRN \"NOISE\",QRQ (FASTER),QRS (SLOWER),QRZ WHO calling?,QTH \"LOCATION\",QSB \"FADING\",QSY [CHANGE FREQ],QRP, home,page,has,search,free,i10n,site,news,contact,business,web,view,online,click,its,service,price,date,top,list,name,health,music,buy,data,product,post,her,city,add,policy,number,such,please,available,support,message,best,software,info,rights,books,links,review,years,privacy,book,items,company,read,user,said,research,university,mail,full,map,reviews,days,management,united,hotel,item,center,eBay,store,travel,comments,development,report,member,details,terms,hotels,type,local,using,results,office,education,national,car,design,posted,internet,address,community,within,states,area,dvd,shipping,reserved,subject,forum,family,based,black,check,special,prices,website,index,women,sign,file,link,today,tech.,south,case,project,pages,version,section,found,sports,related,security,county,American,photo,game,members,power,care,network,computer,three,total,following,download,him,per,access,north,resource,current,posts,big,media,law,control,water,history,pictures,size,art,personal,including,guide,shop,directory,board,location,white,rating,rate,government,children,USA,return,students,shopping,account,times,sites,level,digital,profile,previous,events,love,john,main,hours,image,department,title,descripe,non,insurance,shall,property,money,every,listing,content,country,private,visit,save,tools,low,reply,customer,compare,movies,include,college,value,article,york,card,jobs,provide,food,source,author,different,press,learn,sale,print,job,Canada,process,teen,room,stock,training,credit,join,science,men,category,advanced,west,sales,english,left,team,estate,condition,select,windows,photos,party,thread,category,note,live,gallery,table,register,market,library,action,start,series,model,features,air,industry,human,provided,required,second,hot,accessory,cost,movie,forums,better,question,yahoo,medical,Army,Navy,doctor,nurse,police,fireman,bit,byte,road,months,sort,moment,street,town,Mexico,door,side,mother,street,Jan.,Feb.,March,April,May,June,Jul.,Aug.,Sept.,Oct.,Nov.,Dec.,Monday,Tue.,Wed.,Thur.,Fri.,Saturday,Sunday,Europe,Asia,Africa,Mars,HAM,Balun,dipole,end feed,OCF,long wire,J-pole,half-wave,rag chew,Yaesu,Baofeng,Kenwood,ICOM,PTT,band,Chirp,SDR,FT-8,FCC,amplifier,tuner,SSB,LSB,Walkie-Talkie,80M,40M,VHF,UHF,Morse,Novice,General,Extra,license,Net,SWR,coax,cable,rope,leader,ladder,12V,USB,Blue Tooth,Laptop,cell ph.,coil,caps,xister,LEDs,ICs,filter,oscillate,impedance,50Ohm,SO239,PL259,BNC,SMA,RG8x,RG174,resistance,solder,circuit,bought,sold,sell,built,DIY,kits,measure,GRM,FRS,Solar,wait,field,bench,break,bend,emit,radiate,BFO,AGC,AFC,VCO,comm,package,bank,chock,switch,flash,portable,friend,LiFePo4,H2O,CO2,inverter,UPS,panel,shack,voice,analog,rig,beam,yrs,temp,loop,Zepp,Yagi,G5RV,quad,speaker,microphone,Heath,Windom,Doublet,sloper,horizontal,PSK31,MFJ,599,EndFed,Inv-V,vertical,roof,tree,snack,kilogram,pound,meter,inch,feet,seconds,minutes,sleep,drive,lunch,drink,beverage,amateur,hail,snow,rain,cold,warm,cool,clear,sunny,windy,cloudy,storms,overcast,freezing,dark,station,channel,operator,Amplitude,modulation,frequency,band-pass,beacon,chassis,repeater,conduce,iron-core,crystal,detector,element,dummy,earth,emergency,emission,energy,signal,noise,overload,fuse,harmonic,inductor,insulator,ionize,lightning,metric,micro-,milli-,Network,primary,traffic,propagation,RTTY,reflection,resistor,safety,skip,sky wave,sunspots,cycle,third-party,transmission,variable,ARES,balanced,trap,shield,equipment,azimuth,bandwidth,Polarity,carrier,components,breaker,controlled,connector,ferrite,toroid,procedure,exam,cutoff,privilege,spectrum,manual,error,feed line,display,simplex,coverage,plane,identify,permit,EMI,keyboard,keyer,technician,logbook,loose,magnetic,matching,maximum,military,negative,positive,node,operating,packet,scatter,radio,sending,speed,reflected,schematic,thermal,density,rotor,satellite,shunt,sine wave,device,speech,splatter,stable,straight,stray,terminal,tower,tubes,tuned,length,width,height,club,";

void MC_freeForm() {  //  -------- allow free form Morse code practice until btn4 pressed
  // mostly for Beginers, and 2 Key M.C. operation is not supported
  byte ditdahs;
  char chrs = 0;
  int p;
  float wpm;

  while (!btn4) {
    if (btn1) {
      ditdahs = fetchMorseChar(0);  //echo);   // get dits & dahs appearing to form the MC of a character
      if (ditdahs != 0) {
        chrs++;
        p = 0;
      }
      litLED = (chrs - 1) % 8 + 1;
    }
    if (btn2Pressed || btn3Pressed) {
      if (btn3) ditPeriod = (6 * ditPeriod + 3) / 7; // pickup the pace 16+% faster
      else      ditPeriod = (7 * ditPeriod + 2) / 6; // slow down the pace 16+% slower
      wpm = ((1000.0 * 60.0) / (240.0 * ditPeriod / 5.0));
      Serial.print(" Keying rate: "); Serial.println(wpm);
      playStr("R", 0);  // .-.  2p for each
    }

    // ---------- checking for end of word
    if (p < (5 * ditPeriod)) {
      if (++p == (5 * ditPeriod)) Serial.print(" "); // give 3+5 more than 7p
    }
    scanBtns();
    refreshWait(1);
  }
  pAveraging = 8 * ditPeriod; // update their standard rate
  reportRate();
}

void reportRate() {
  float wpm;
  wpm = ((1000.0 * 60.0) / (240.0 * ditPeriod / 5.0));
  //Serial.print(" avg. Period: "); Serial.println(pAveraging/4);
  Serial.println();
  Serial.print(" Keying rate (in WPM): "); Serial.print(wpm);
  if (farns == 1) Serial.println();
  else Serial.println("  + Farnsworth spacing");
  showLevel(wpm + 0.5);
  refreshWait(2000);
  clearDisp();
}

// -----------------------   A-Z Morse Code Please
void A2Z_Morse_Exercise() {
  //  char alpha[] = "ZABCDEFGHIJKLMNOPQRSTUVWXYZA";
  byte ditdahs;  // all MSBits that are the same are discarded, whats left is 1=dit 0=dah, MSbit is first in time, LSB last
  byte dida;  // expected
  char cinput;
  byte correct = 0;
  static bool echo = true;

  pAdj = false; // maintain Keying rate
  // display & btn press is cleared prior
  delay(100);
  for (int i = 1; i <= 26; i++) {
    Serial.println(); Serial.print(alpha[i]);
    dimFlash = (i - 1) % nleds + 1;
    while (1) {
      refreshWait(1);
      if (btnPressed() && (btn1 || btn2)) {    // Button has been pushed
        ditdahs = fetchMorseChar(true);     // get one char response
        refreshWait(200);  // ?

        cinput = dida2Ascii(ditdahs);
        dida = Ascii2dida(alpha[i]);
        if (cinput == alpha[i]) { // he got it right
          correct++;
          if (echo) playMorseCode(ditdahs, 0);
        } else {
          while (ditdahs != dida && ditdahs != 0) { // give them the correct ans, & do so until they get it right
            brt[7] = brt[8] = true;
            buZZ(500, 1000);
            brt[7] = brt[8] = false;  refreshDisp();
            playMorseCode(dida, 0);
            ditdahs = fetchMorseChar(true);
            scanBtns();
            if (btn4) return;
          }
          if (echo) playMorseCode(ditdahs, 0);
        }
        clearDisp();
        //            delay(500);  // ? whats advisable here
        break;
      }
      if (btn3Pressed) echo = !echo;
      if (ESC || btn4) return;
    }
  }

  Serial.print("\ncorrect: "); Serial.print(correct);
  Serial.print("  out of  "); Serial.println(26);
  showLevel(correct);
  refreshWait(3000); clearDisp();
}

// -----------------------
void Morse_AB_test() {
  //  const char alpha[] = "ZABCDEFGHIJKLMNOPQRSTUVWXYZA";
  byte clist[28], ri[28];
  int i, ci;
  int correct;
  char cinput;
  byte dida, ditdahs;
  byte ntests = 12; // how many to do
  byte glowLED;

  glowEnable = true;
  pAdj = false; // maintain Keying rate
  while (1) {
    refreshWait(1000);
    correct = 0;
    ranIndexes(ri, 26); // get randomized set of indexes, to chars of alphabet
    for (int i = 1; i <= ntests; i++) {
      glowLED = (i - 1) % nleds + 1;
      glow[glowLED] = true;
      dimIt = glowLED; // dimIt softer

      while (Btn) scanBtns(); // wait out old btn presses
      ci = ri[i];
      dida = Ascii2dida(alpha[ci]);
      set_didaStr(dida, didaStr);
      playMorseCode(dida, 0);              // ---- sound one out
      Serial.println();
      Serial.print(didaStr);  Serial.print(" ");  Serial.print(alpha[ci]);
      ditdahs = fetchMorseChar(true);  // ----  get their response
      if (btn4) return;
      if (btn3) {
        i--;        // replay the prompt char
        refreshWait(750);
        continue;  // go back and do it again
      }
      cinput = dida2Ascii(ditdahs);
      if (cinput == alpha[ci + 1]) { // he got it right
        correct++;
      } else {
        while (cinput != alpha[ci + 1]) { // rePrompt them, until they get it right
          if (ditdahs != 0) {
            brt[7] = brt[8] = true;
            buZZ(500, 1000);
            brt[7] = brt[8] = false; refreshDisp();
          }
          if (btn3) delay(750);
          while (Btn) scanBtns();
          playMorseCode(dida, 0);
          ditdahs = fetchMorseChar(true);
          if (btn4) return;
          cinput = dida2Ascii(ditdahs);
        }
      }
      clearDisp();
      refreshWait(750);
      if (ESC || btn4) return;
    }
    Serial.print("\ncorrect: "); Serial.print(correct);
    Serial.print("  out of  "); Serial.println(ntests);
    showLevel(correct);
    refreshWait(3000); clearDisp();
  }
}

// -----------------------
void MC_Relay_exercise() {
  const char SpChar[] = "01234567890.,?/'-.,?/'-:\"+!=@$)&\n";
  char dd[10];  // stores ditdah given out
  byte ci, li, wlen, numW, mini, maxi;
  int i, p, dl;
  static int opt = 1;
  char dida;
  char c, cinput;
  byte correct;
  static int ditPace;
  static bool adjd = false;
  float wpm;

  if (!adjd) ditPace = 5 * ditPeriod / 6; // use 5/6 his current dit period, play at 1.2x keying speed
  pAdj = false; // maintain Keying rate
  // note: once manually overriden, new speed will get used
  wpm = ((1000.0 * 60.0) / (240.0 * ditPace / 5.0));
  Serial.print(" Audio out rate to be used: "); Serial.println(wpm);

  opt = get_Option(5, opt);
  if (ESC || opt == 0) return;
  mini = 1; // default
  //  determin the char set to be used
  if      (opt == 1) {
    mini = 1;  // Alpha
    maxi = 26;
  }
  else if (opt == 2) {
    mini = 1;  // Alpha + Nums + punctation . thru +
    maxi = 51;
  }
  else if (opt == 3) {
    mini = 27;  // Nums + punctation . thru +
    maxi = 51;
  }
  else if (opt == 4) {
    mini = 37;  // just punctation-special chars
    maxi = 58;
  }

  wlen = (opt == 5) ? 5 : 3; // was 5:4
  numW = (opt == 5) ? 4 : 4; // was 4:3

  clearDisp();
  while (!btn4) {
    correct = 0;
    //    clearDisp();
    while (btnDown()) refreshWait(1);

    // ------  wait for two second, mean while btn2&3 can be used to adjust speed
    dl = 2000; // delay for 2secs
    while (dl--) {
      scanBtns(); refreshWait(1);
      if (btn4) return;
      if (btn2Pressed || btn3Pressed) {
        if (btn3) ditPace = (5 * ditPace + 3) / 6; // pickup the pace 20% faster
        else      ditPace = (6 * ditPace + 2) / 5; // slow down the pace 20% slower
        adjd = true;
        wpm = ((1000.0 * 60.0) / (240.0 * ditPace / 5.0));
        Serial.print(" Audio out rate: "); Serial.println(wpm);
      }
      if (BtnPressed) {
        dl = 3000; // extend waiting
        playMorseCode(Ascii2dida('P'), ditPace); // give a 'P' for reference
      }
    }

    for (int w = 1; w <= numW; w++) {
      clearDisp();
      if (opt == 5) maxi = (random(3) == 0) ? 26 : 42;
      for (li = 0; li < wlen; li++) { // generate random letters of a psuedo word
        ci = random(mini, maxi + 1); // get the character form the set he selected
        if (opt == 5 && li == 4) ci = random(37, 58); // add an extra spc. char
        if (ci < 27) {
          c = alpha[ci];
          Serial.print(c);
          dida = Ascii2dida(c);
        } else {
          i = ci - 26;
          c = SpChar[i];
          Serial.print(c);
          if (i > 16) i -= 6; // adjust index to skip over repeated chars in SpChar[]
          dida = nonAlpha_dida[i];
        }
        dd[li] = dida;

        playMorseCode(dida, ditPace);
        if (btn4) return;
        if (ditPace > 120)  delay(2 * ditPace); // std timing between char pause
        else delay(2 * ditPace * farns); // give students +time to recallet with farnsworth
        if (farns == 1) delay(3 * ditPace); // still give +time to recallet std speed chars, they just heard

        set_didaStr(dida, didaStr);
        Serial.print(" "); Serial.print(didaStr); Serial.print("    ");
      }

      Serial.println();
      dimFlash = 1;
      while (!(btn1 || btn2)) {  // wait for start of input
        refreshWait(1);
        scanBtns();
        if (btn4) return;
        if (btn3Pressed) {
          refreshWait(1000);
          for (li = 0; li < wlen; li++) { // replay the 'word'
            playMorseCode(dd[li], ditPace);
            refreshWait(2 * ditPace * farns);
          }
        }
      }
      for (li = 0; li < wlen; li++) { // get their reponse to the psuedo word given
        dimFlash = li + 1;
        dida = fetchMorseChar(true);
        if (ESC || btn4) return;
        Serial.print("  ");
        if (dida == dd[li]) {
          correct++;
          dim[li + 1] = true;
        }
        else buZZ(150);
      }
      dimFlash = 0;
      refreshWait(1000);
      Serial.println();
    }
    Serial.print("\ncorrect: "); Serial.print(correct);
    Serial.print("  out of ");  Serial.println(wlen * numW);
    if (correct == 0) correct = 1; // credit for participation
    showLevel(correct);
    refreshWait(3000); //clearDisp();
  }
}

// -------------
void MC_Receive_Practice() {  // came from 'MC_Practice'
  byte opt, mode;
  char wrd[30], tmpw[30];
  char *cptr;
  int strInx;
  byte ci, wlen, errs, lErrs, ccnt;
  bool errW;
  int i, ri, wi, w, p;
  byte ditdahs;
  char cinput, c;
  float wpm, wpmTkr;
  unsigned long t, t0;
  const int hiSpd = 19; // to use simplier words when working at lower speeds
  bool test = true; // normally its a prompted test, else its free form sending (CW keying)
  bool fini;
  int ditPace;

  // Modes:  1) RCV word at a time   2) RCV a series of words
  // words are all randomly selected
  Serial.print(" Mode #");
  mode = get_Mode(2); Serial.println(mode);
  if (mode == 0) return;

  //numWrds=strlen(wordStr)/5;  // for approximate
  numWrds = 500;

  while (1) { // provide back to back operation, for long practice sessions
    ditPace = 5 * ditPeriod / 6; // set audio out higher than their keying rate
    if (ditPace > 102) ditPace = 102; // maximum of 102 for minimum. 102 for 12 WPM (92 for 13+, 77 for 16 WPM)
    wpm = ((1000.0 * 60.0) / (240.0 * ditPace / 5.0));
    Serial.print("\nStarting char output WPM rate: "); Serial.println(wpm);

    fini = false;
    ci = 0;
    ccnt = 0;
    errs = lErrs = 0;
    w = 0;
    wpmTkr = 0;
    delay(1000);

    while (ccnt < 60) { // practice test is at leasr 60 chars
      pOpen = true;
      dimFlash = 1 + (w & 0x07);
      refreshDisp();

      do {  //    randomly select a 'word'
        strInx = random(0, strlen(wordStr) - 26); // randomly select an area
        strncpy_P(wrd, (char *) & (wordStr[strInx]), 26 );
        wrd[26] = 0;

        strcpy(tmpw, strchr(wrd, ',') + 1); // pull out a word
        cptr = strchr(tmpw, ',');
        if (cptr == NULL) continue;
        wlen = (int) (cptr - tmpw);
        strncpy(wrd, tmpw, wlen);
        wrd[wlen] = 0;
        //        Serial.print("  wlen is "); Serial.print(wlen);  Serial.print(" : "); Serial.println(wrd);
        //        Serial.print(" strInx is "); Serial.print(strInx);  Serial.print(" : "); Serial.println(wrd);
      } while ( wlen < 1 || (wpm < hiSpd && wlen > 6) || (wpm < 4 + hiSpd && wlen > 8));
      // in the 900 element array there are 635 entries with 3-6 characters, 812 with 8 or less characters, 88 words more than 8

      ccnt += wlen;

      wpm = ((1000.0 * 60.0) / (240.0 * ditPeriod / 5.0));
      if (wpmTkr == 0) wpmTkr = 10 * wpm;
      wpmTkr = (9 * wpmTkr / 10 + wpm); // track & average over a practice round

      if (test) {
        Serial.print(" ");
        Serial.print(wrd);
      }

      for (i = 0; i < strlen(wrd); i++) {
        c = wrd[i];   // use the same ditdah code for all brackets & braces
        if ( c == '[' || c == '{' || c == '(' || c == '}' || c == ']' ) wrd[i] = ')';
      }

      if (w == 0) t0 = millis();
      playStr(wrd, ditPace);
      if (mode == 2) refreshWait(3 * ditPace * farns); // add 3+p more time to make up 7p between word period

      btnsNow();  // scan for new button state
      if (btn4) fini = true;
      else if (mode == 2 && Btn) {           // go into Pause state
        set_LEDs(0x30, 3);
        while (Btn) {
          refreshWait(25);
          scanBtns();
        }
        while (!BtnPressed) scanBtns(); // wait for next press to continue
        set_LEDs(0x30, 0);
        refreshWait(1500);   // give time to return to paper & pencil
      }

      if (mode == 1) { // handle one word at a time
        while (!(btn1)) { // ---------- wait to CW keying to begin
          refreshWait(1);
          scanBtns();
          if (btn4) {
            fini = true;
            break;
          }
          if (btn3Pressed || (btn2Pressed && mode == 1)) {
            delay(1000);
            playStr(wrd, ditPace);
          }
        }
      } else {
        scanBtns();
        if (btn4 && mode == 2) return;
      }

      if (fini) break;
      dimFlash = 0;
      litLED = 0; refreshDisp();
      if (mode == 1) refreshWait(750); // moment to get ready to write it down
      w++;
    }

    t = (millis() - t0);
    //Serial.print("\n Char cnt: "); Serial.print(ccnt); Serial.print("   time(msecs): "); Serial.print(t); Serial.print("   mins: "); Serial.print(t/60000.0);
    if (mode == 1) t = t - 8; // -(0.75*11) for the waits the code injected
    wpm = ((ccnt - 1 + w * farns) / 5.0) / (t / 1000.0 / 60.0); // +12*farns is adj for alowing extra time between letters & words
    Serial.println(); Serial.print("   over all WPM: "); Serial.println(wpm);
    showLevel(wpm);
    refreshWait(2000);
    while (!btnPressed()) {
      delay(1); // wait for button press
    }
    if (btn4) return;  // B4->exit, else go again
    while (btnDown()) ;        // then wait for release
  }
}

// -----------------------
void Tx_Line_test() {
  // btn 3 toggles Easy mode, where bad spacing is tollerated
  // each time you complete a line with no errors 10 characters is added to the test lines (upto 80 chars)
  char wrd[30], tmpw[30];
  char line[100];
  static const char ending[] = ".,?!-/+.,?";
  byte ci, wc, wlen, wcnt, errs, ccnt, llen;
  int wi;
  byte i, pspace, xt;
  int p;
  byte ditdahs;
  char cinput;
  float wpm, wpmTkr;
  byte chrs;
  bool lerr, ptime;
  static bool easy = true; // true= don't penalize poor spacing
  static byte maxC = 10;
  unsigned long t, pauseT;
  char testC;
  char *cptr;
  int strInx;
  //  const byte hiSpd=18;  // to use simplier words when working at lower speeds
  const byte hiSpd = 14;

  randomSeed(randSeed = micros() / 4); // micros() resolution is ~ 4us
  Serial.print("random Seed:  "); Serial.print( randSeed);


  prtChr = false; // prevents double printing MC chars

  wpm = ((1000.0 * 60.0) / (240.0 * ditPeriod / 5.0));
  if (wpm > 17) easy = false;
  //  Serial.print("Starting char WPM rate: "); Serial.println(wpm);
  wpmTkr = 6 * wpm;
  delay(200);  // avoid button bounce on entry
  while (!btn4) {
    i = 0; ci = 0;
    ccnt = 0;
    wcnt = 0;
    chrs = 0;
    pauseT = 0;
    while (ccnt < maxC) {  // predetermine 'word's for line totaling <80 chars

      do {  //    randomly select a 'word'
        strInx = random(0, strlen(wordStr) - 26); // randomly select an area
        strncpy_P(wrd, (char *) & (wordStr[strInx]), 26 );
        wrd[26] = 0;

        strcpy(tmpw, strchr(wrd, ',') + 1); // pull out a word
        cptr = strchr(tmpw, ',');
        if (cptr == NULL) continue;
        wlen = (int) (cptr - tmpw);
        strncpy(wrd, tmpw, wlen);
        wrd[wlen] = 0;
        //        Serial.print("  wlen is "); Serial.print(wlen);  Serial.print(" : "); Serial.println(wrd);
        //        Serial.print(" strInx is "); Serial.print(strInx);  Serial.print(" : "); Serial.println(wrd);
      } while ( wlen < 1 || (wpm < hiSpd && wlen > 6) || (wpm < 4 + hiSpd && wlen > 8));
      // in the 900 element array there are 635 entries with 3-6 characters, 812 with 8 or less characters, 88 words more than 8

      for (i = 0; i < wlen; i++) {
        line[ci + i] = wrd[i]; // crude string copy
      }
      ci = ci + wlen;
      line[ci++] = ' ';
      ccnt += wlen;
      wcnt++;
    }


    line[ci - 1] = ending[random(0, strlen(ending))];
    line[ci] = 0x00; // mark end of string
    llen = strlen(line);
    Serial.print("\n "); Serial.println(line); Serial.print(" ");

    ci = 0;
    wi = 0;
    ccnt = 0;
    t0 = 0;
    errs = 0;
    xt = 0;
    lerr = false;
    ptime = false;

    ptime = true; // allow time to pass before keying begins
    while (ci < llen) {
      cinput = '#';
      p = 0;
      while (cinput == '#') { // ---------- get next MC character from user
        dimFlash = 1 + (wi & 0x07);
        refreshWait(1);
        if (btnPressed()) {
          if (btn1 || btn2) {   // ---- they're keying in MC
            if (t0 == 0) t0 = millis();
            ditdahs = fetchMorseChar(false);   // get dits & dahs appearing to form the MC of a character
            t1 = millis();
            pauseT = millis();
            cinput = dida2Ascii(ditdahs);
          }
          if (btn3) easy = !easy;
        } else {    // ---- space between words detection
          delay(1);
          if (!ptime) p++;
          //        if (p>(4*ditPeriod)) {  // time for a space between words (3p after last char + 4p = 7p)
          pspace = (line[ci] == ' ') ? 3 : 5; // adj. to tolarate speed variations, at least
          if (p > ((pspace + xt)*ditPeriod)) { // xt normally =0 is for eXtra Time
            cinput = ' ';
          }
        }
        //  OK now?    if (cinput==' ' && line[ci-1]==' ') cinput='#';  // allow extra space at begining of words
        // If last or next char is puncuation allow for a space in time.
        //      if (cinput==' ' && (isPunct(line[ci-1]) || isPunct(line[ci]))) cinput='#';
        if (cinput == ' ' && (isPunct(line[ci]))) cinput = '#';
        if (ESC || btn4) return;
      }
      xt = 0;

      // -----------  did we get the char we expected
      testC = line[ci];
      if (testC == '[' || testC == '{')  testC = '(';
      if (testC == ']' || testC == '}')  testC = ')';
      if (cinput == ')' && testC == '(')  cinput == '(';
      if (easy && line[ci] == ' ' && cinput != ' ') {
        ci++;
        Serial.print('_');
      }
      if (cinput == toupper(testC)) { // ---- got it as expected
        ci++;
        if (cinput != ' ') chrs++;
        ptime = (cinput == ' ');
        Serial.print(cinput);
        if (line[ci] == ' ' || line[ci] == 0x0) { // update WPM tracking after each word
          wi++;
          wpm = ((1000.0 * 60.0) / (240.0 * ditPeriod / 5.0));
          wpmTkr = (5 * wpmTkr / 6 + wpm); // track & average over a practice round
          //        Serial.print("\n ditP WPM: "); Serial.print(wpm);
          //        Serial.print("   Trkd WPM: "); Serial.print(wpmTkr/6);
        }
        if (lerr) xt = 3; // if they are just righting a wrong allow x-time to get going again
        lerr = false;

      } else {                       // ---- Not as expected
        if (easy && cinput == ' ') continue;
        buZZ(25);
        if (!lerr) {
          errs++;
          buZZ(75, 150);
        }
        lerr = true;
        ptime = true; // let them have time to regroup
        //      ptime=(line[ci]!=' ');  // if they gave wrong Letter-Symble let them have time to regroup
        if (line[ci] == ' ') {
          ci++;
          Serial.print("_");
          continue;
        }
      }
    }

    t = t1 - t0;
    wpm = ((1000.0 * 60.0) / (240.0 * ditPeriod / 5.0));
    Serial.print("\n current ditdah Keying rate in WPM: "); Serial.print(wpm);
    // note: avg char code takes 12p, 4 char is 48p, 5* that is 240p, the std for 5WPM

    Serial.print("\n Trending ditdah Keying rate in WPM: "); Serial.print(wpmTkr / 6);
    //Serial.print("\nchrs: "); Serial.print(chrs); Serial.print("   t: "); Serial.println(t);

    wpm = (chrs / 4.5) / (t / (1000.0 * 60.0)); // adj. for the char distribution in our word databaase
    if (wpm < 1) wpm = 1;
    Serial.print("    over all ~ Words/Minute: "); Serial.println(wpm);
    //  showLevel((wpmTkr+3)/6);
    //  refreshWait(2000); clearDisp();
    delay(750);
    if (errs > 0) {
      Serial.print("   Number of Incorrect characters: "); Serial.println(errs);
      buZZ(200 + 50 * errs);
      //    showLevel(errs);
      //    refreshWait(1000); clearDisp();
    } else {
      Serial.println("    No errors were detected.");
      if (maxC < 80) {
        maxC += 10;
        set_LEDs(0xFF, 3); delay(250); // congratulations Flash
        set_LEDs(0xFF, 0); delay(100);
        set_LEDs(0xFF, 3); delay(250);
        showLevel(maxC / 10);
        Serial.print("    Advance minimum line length to "); Serial.println(maxC);
        refreshWait(2000); clearDisp();
      }
    }
  }
}

// ================== Local Morse Code Support Utility Functions =================

// -----------------------
bool isPuntc(byte charCode) { // identifies punctuation as we can be tolerant of delays keying them in (added 0-9 as well)
  //  return (charCode>32 && !(toupper(charCode)>64 && toupper(charCode)<91) && !(toupper(charCode)>47 && toupper(charCode)<58));
  return (charCode > 32 && !(toupper(charCode) > 64 && toupper(charCode) < 91));
}

void debugTest() {
  /***  char c;
    Serial.println(" Char,  isPuntc(c),  dida2Ascii(Ascii2dida(c))");
    for (byte b=32; b<=127; b++) {c=b; Serial.print(c);}
    Serial.println();
    for (byte b=32; b<=127; b++) { Serial.print(isPuntc(b));}
    Serial.println();
    Serial.print(' '); // dida2Ascii(Ascii2dida(' '))=0   so print a ' ' holder for it
    for (byte b=33; b<=127; b++) Serial.print(dida2Ascii(Ascii2dida(b)));  Serial.println();
  ***/

  // List/Test all of the words in our vocabulary
  int wi, wlen ;
  char wrd[30], tmpw[30];
  char *cptr;
  int strInx;
  int deltInx;

  strInx = 0;
  for (wi = 0; wi < 900; wi++) {
    //strcpy_P(wrd, (char *)pgm_read_word(&(words[wi])));
    do {  //    randomly select a 'word'
      //        strInx = random(0,strlen(wordStr)-8); // randomly select an area (// out for listing in order)
      strncpy_P(wrd, (char *) & (wordStr[strInx]), 26 );
      wrd[26] = 0;
      strcpy(tmpw, strchr(wrd, ',') + 1); // pull out a word
      deltInx = strlen(wrd) - strlen(tmpw);
      cptr = strchr(tmpw, ',');
      if (cptr == NULL) continue;
      wlen = (int) (cptr - tmpw);
      strncpy(wrd, tmpw, wlen);
      wrd[wlen] = 0;
      //        Serial.print("  wlen is "); Serial.print(wlen);  Serial.print(" : "); Serial.println(wrd);
      //        Serial.print(" strInx is "); Serial.print(strInx);  Serial.print(" : "); Serial.println(wrd);
    } while (wlen < 1);
    strInx = strInx + deltInx;
    if (strInx > (strlen(wordStr) - 4)) return;

    wlen = strlen(wrd);
    Serial.print(wi); Serial.print("\t"); Serial.print(wlen);
    Serial.print("\t*"); Serial.print(wrd); Serial.println("*");
  }
}

void DebounceTest() { // user watches for lite skipping ahead of button press count
  unsigned int t;
  byte bb = 0;
  litLED = 1;
  while (!ESC) {
    scanBtns();
    if (BtnChanged) { // print time between state chgs
      t = millis() - t0;
      t0 = millis();
      Serial.print(" "); Serial.print(t);
      if ((!btns) && t < 20) { // count # of state time periods <20ms
        Serial.print(" _@"); Serial.print(++bb);
      }
      if (BtnPressed) litLED = (litLED % nleds) + 1; // move lit LED each time btn pressed
    }
    refreshWait(1);
  }
}

// -----------------------
void dit_dah(unsigned short int delayms) { // Used for ditdah audio while user has key pressed
  // higher freq & longer duty cycle for piezo
  delayms = delayms - (delayms / 5);
  for (int i = 0; i < delayms; i++) {
    digitalWrite ( AUDIO_OUT, HIGH);
    if (Loud)  digitalWrite ( AUDIO_NEG, LOW);
    delayMicroseconds(840);   // 750 840
    digitalWrite ( AUDIO_OUT, LOW);
    if (Loud)  digitalWrite ( AUDIO_NEG, HIGH);
    delayMicroseconds(560);   // 500 560
  }
  //digitalWrite ( AUDIO_OUT, LOW);;
  if (Loud)  digitalWrite ( AUDIO_NEG, LOW);
}

// -----------------------
void ditAudio(unsigned short int delayms) { //  over duty cycle adjusted for freq.
  // ON duration extra needed for good piezo sound
  //  delayms = delayms + (delayms/2);  // for use with 400/266
  delayms = delayms - (delayms / 5); // for use with 700/550
  for (int i = 0; i < delayms; i++) {     // wait for 'delayms' millisecs
    digitalWrite ( AUDIO_OUT, HIGH);
    if (Loud)  digitalWrite ( AUDIO_NEG, LOW);
    delayMicroseconds(840);    // tried: 700 600 400 456 best? 700 825 840
    digitalWrite ( AUDIO_OUT, LOW);
    if (Loud)  digitalWrite ( AUDIO_NEG, HIGH);
    delayMicroseconds(560);   // tried: 550 400 266 210 best? 500 550 560
    refreshDisp();  // just spent ~1 ms, lets refreshDisp now
  }
  digitalWrite ( AUDIO_OUT, LOW);;
  if (Loud)  digitalWrite ( AUDIO_NEG, LOW);
}

// -----------------------
void dahAudio(unsigned short int delayms) { // lower freq ( w/ longer ON duty cycle for Piezo)
  delayms = (2 * delayms) / 3;
  for (int i = 0; i < delayms; i++) {     // wait for delayms ms
    digitalWrite ( AUDIO_OUT, HIGH);
    if (Loud)  digitalWrite ( AUDIO_NEG, LOW);
    delayMicroseconds(870);    // 900
    digitalWrite ( AUDIO_OUT, LOW);
    if (Loud)  digitalWrite ( AUDIO_NEG, HIGH);
    delayMicroseconds(580);    // 600
    refreshDisp();  // just spent ~1 ms, lets refreshDisp now
  }
  digitalWrite ( AUDIO_OUT, LOW);;
  if (Loud) digitalWrite ( AUDIO_NEG, LOW);
}


// -----------------------
void playMorseCode(byte ditdahs, int pace) {
  // all MSBits that are the same are discarded, whats left is 1=dit 0=dah, MSbit is first in time, LSB last
  short int ditPeriod;
  byte i = 1;
  byte bitState;

  ditPeriod = pAveraging / 8;  // set according to their current pace
  ditPeriod = 8 * ditPeriod / 10; // but 80% timing, for 1.25 the speed
  if (pace) ditPeriod = pace;  // unless passed in

  if (ditdahs == 0) return;
  while ((ditdahs << i & 0x80) == (ditdahs & 0x80)) i++;
  for (int b = i; b < 8; b++) {
    if (ditdahs << b & 0x80) { // this bit is a 'dit'
      brightOne = 7;
      ditAudio(ditPeriod);
      brightOne = 0;
      refreshWait(ditPeriod);
    } else {  // its a dah
      dim[7] = dim[8] = true;
      dahAudio(3 * ditPeriod);
      dim[7] = dim[8] = false;
      refreshWait(ditPeriod);
    }
    scanBtns();  // needed to keep button state upto date (for btn4 esc)
  }
}


// -----------------------
void playStr(char *wrd, int pace) {          // ment for messsage announcements
  float ff = farns;
  char c;
  byte ditdahs;
  int ditPace;
  byte dimF;

  dimF = dimFlash;
  dimFlash = 0; // stop position flash in MC_Practice during this word output
  refreshWait(10);

  ditPace = ditPeriod;
  if (pace) ditPace = pace;  // unless passed in

  for (int i = 0; i < strlen(wrd); i++) {
    ditdahs = Ascii2dida(wrd[i]);
    if (wrd[i] == '_') ditdahs = 0;
    //Serial.print(wrd[i]); Serial.print(ditdahs);
    if (ditdahs == 0) delay(2 * ditPace * ff); // handle timing for the likes of a space
    playMorseCode(ditdahs, ditPace);
    if (i+1<strlen(wrd)) {  // more coming
      //    refreshWait(3*ditPace*ff-ditPace);   // +2p for between character
      refreshWait(2*ditPace*ff);   // +2p for between character
    }
    scanBtns();
    if (btn4) return;
  }
  //  refreshWait(3*ditPace*ff);  // add 3+p more time to make up 7p between word period
  //  now caller's responsibility
  dimFlash = dimF;
}

// -----------------------
//void stableBtns() { // get a stable state of the buttons post debouncing
//  refreshWait(1);
//  scanBtns();
//  while(currState != btnState) {
//    refreshWait(1);
//    scanBtns();
//  }
//}


// =====================================================================
// ===========================  Group 0  ===============================
// -------------- Diagnostics Tools & Testing Utilities ----------------

void btnReporting() {
  // =================================  Button - LED exercise & Reporting
  int icnt = 0; // iteration count, lsb~=1msec
  clearDisp();
  while (1) {
    refreshWait(1);
    icnt++;
    scanBtns();
    // ------------------- button reporting
    if (BtnChanged) {
      Serial.println(" ");
      if (btn1) Serial.print("[Btn1] "); else Serial.print("       ");
      if (btn2) Serial.print("[Btn2] "); else Serial.print("       ");
      if (btn3) Serial.print("[Btn3] "); else Serial.print("       ");
      if (btn4) Serial.print("[Btn4] "); else Serial.print("       ");
      //Serial.print(btnState);

      // ------------------ LED reflection of button state
      lit[1] = lit[2] = btn1; // set Red LEDs
      lit[3] = lit[4] = btn2; // set Yel LEDs
      lit[5] = lit[6] = btn3; // set Blu LEDs
      lit[7] = lit[8] = btn4; // set Grn LEDs
      brightOne = 0;

      beep(15); // make a key press click, for up&down
      icnt = 0;
    } else if (icnt > 10000) { // if ignorned for 10 seconds
      icnt = 0;
      brightOne = 1;
      beep(15, 200);
      clearDisp();
    }
    if (ESC && brightOne == 0 && btn4 ) Serial.print(" [ESC] ");
    brightOne = (ESC) ? 8 : 0;
    if (ESC && btn1) break;    // only leave Btn Reporting if btn1 is also held.
  }
}

void Color_lights() {   //  turns the 4 sets of LEDs On-Off  (exersises 'ledsAll()')
  int leds = 0;
  byte coLvl[5] = {0, 0, 0, 0, 0}; // level for each color LED set
  byte col_leds[] = {0, 1, 2, 4, 8};
  byte col_pat[] = {0, 0b00000011, 0b00001100, 0b00110000, 0b11000000};
  byte colNum;

  Serial.println("Exercize and test operation of set_LEDs(), ledsAll(), showColor() & setColor() utils.");
  Beep(100);
  clearDisp();

  wait_BtnRelease(); // wait for no button
  blinkLed = 1;
  while (!catchBtn()) {
    ;
  }
  blinkLed = 0;

  while (1) {
    if (BtnPressed) {
      colNum = btnNum;
      coLvl[colNum] = (coLvl[colNum] + 1) % 5;
      set_LEDs(col_pat[colNum], 0);  // clear old color set's levels, so not additive
      // update the lit state of each set of color LEDs
      switch (coLvl[colNum]) {
        case 1: set_LEDs(col_pat[colNum], 1); break;  // sets to dim level
        case 2:
          //ledsAll(col_leds[colNum]); // sets to mid lit level for 1-4 colors & clears 'lit[]' for others
          //showColor(colNum);         // shows only requested color (via 'lit[]') clears the rest of lit[]
          set_LEDs(col_pat[colNum], 2); // additively sets to mid lit level, requested color(s)
          break;
        case 3: set_LEDs(col_pat[colNum], 3); break;  // sets to HI lite level
        case 4:
          setColor(colNum); // overrides till cleared, does not relay on refreshDisp() resources
          wait_BtnRelease();
          setColor(0);  // clear setColor override, settings supported in refreshDisp will be seen again
          coLvl[colNum] = 0; // & reset lvl to 0
          break;
        default:  break;
      }
    }

    // highlight any change with the onBrd Led
    digitalWrite(ledOnBrd, BtnChanged);
    if (BtnChanged) refreshWait(10);  // else ledOnBrd will be on for only 2 ms, sight unseen

    refreshWait(2);  // extra time extends ESC timeout period (1.5 -> 3 secs)
    scanBtns();
    if (ESC) return;
  }
}

void GlowBeepTest() {
  byte code[5];
  byte i, s, e, loc = 1, cType = 1, audio = 0;
  byte litLevel = 0;
  byte dimItLoc = 0; //=2; // was for test viewing of cursors + dimIt

  clearDisp();
  while (Btn) scanBtns(); // wait for btns to clear
  dimFlash = 1;

  glowEnable = true; // we here-in use glowing LEDs
  dimIt = dimItLoc;
  dim[3] = lit[4] = brt[5] = 1;
  arc[6] = glow[7] = 1;

  // handle btn presses
  while (!ESC) {
    if (BtnPressed) {
      if (btn1 && btn4) return;  // fast exit
      if (btn1Pressed) audio = audio % 4 + 1; // 1-4
      if (btn2Pressed) loc = (loc == 1) ?  nleds : loc - 1; // -- move left Allow 0:debugging
      if (btn3Pressed) loc = (loc % nleds) + 1; // ++ moves right
      if (btn4Pressed) cType = (cType % 7) + 1; // switch curror type

      if (BtnPressed && btn1 && btn2) {  // switches litLevels 1-8 & 9-16
        clearDisp();
        litLevel = (litLevel + 1) % 3;
        if (litLevel == 0) {
          dim[3] = lit[4] = brt[5] = 1;
          arc[6] = glow[7] = 1;
        } else if (litLevel == 1) {
          levelOn[1] = 1; levelOn[2] = 2; levelOn[3] = 3; levelOn[4] = 4;
          levelOn[5] = 5; levelOn[6] = 6; levelOn[7] = 7; levelOn[8] = 8;
        } else {
          levelOn[1] = 9; levelOn[2] = 10; levelOn[3] = 11; levelOn[4] = 12;
          levelOn[5] = 13; levelOn[6] = 14; levelOn[7] = 15; levelOn[8] = 16;
        }
      }

      // testing cursors over each of the possible LED states
      dimFlash = flashLed = blinkLed = dimLED = litLED = spriteOne = 0;
      if (cType == 1) dimFlash = loc;
      if (cType == 2) flashLed = loc;
      if (cType == 3) blinkLed = loc;
      if (cType == 4) dimIt = loc; else dimIt = dimItLoc; // works but subtle & not over blank cells, so can confusing
      //      if (cType==4) cType++;
      if (cType == 5) dimLED = loc;
      if (cType == 6) litLED = loc;
      if (cType == 7) spriteOne = loc;

      while (btnNum == 1 && !btns) {
        if      (audio == 1) {
          boop(50);
          refreshWait(200);
          scanBtns();
        }
        else if (audio == 2) {
          beep(50);
          refreshWait(200);
          scanBtns();
        }
        else if (audio == 3) {
          Beep(50);
          refreshWait(200);
          scanBtns();
        }
        else if (audio == 4) {
          buZZ(50);
          refreshWait(200);
          scanBtns();
        }
      }
    }
    refreshWait(1);
    scanBtns();
  }
}


// ================== Local Support Utility Functions =================

void showColor(byte color) {   //  0: none, 1:RED, 2:YEL, 3:GRN, 4:BLUE, 5:ALL
  //if (color==0) return;
  for (int i = 1; i <= nleds; i++) lit[i] = false;
  if (color == 1 || color == 5)  lit[1] = lit[2] = true; // set Red LEDs
  if (color == 2 || color == 5)  lit[3] = lit[4] = true; // set Yel LEDs
  if (color == 3 || color == 5)  lit[5] = lit[6] = true; // set Blu LEDs
  if (color == 4 || color == 5)  lit[7] = lit[8] = true; // set Grn LEDs
}

// simular to 'showColor'. but does not use or effect std refreshDisp()
void setColor(byte color) {   // replaced 'allColor=' use
  bool on[9];                //  0: none, 1:RED, 2:YEL, 3:GRN, 4:BLUE
  onSet = false;
  if (color == 0) {
    refreshDisp();
    return;
  }
  for (int i = 1; i <= nleds; i++) on[i] = 0; // init working array
  if (color == 1 || color == 5)  on[1] = on[2] = true; // set Red LEDs
  if (color == 2 || color == 5)  on[3] = on[4] = true; // set Yel LEDs
  if (color == 3 || color == 5)  on[5] = on[6] = true; // set Blu LEDs
  if (color == 4 || color == 5)  on[7] = on[8] = true; // set Grn LEDs
  // -------------- turn the resulting Leds ON
  for (int i = 0; i < nleds; i++) {
    digitalWrite(lites[i], (on[i + 1] == ON_STATE));
  }
  onSet = true;
}

void set_LEDs(byte led_pattern, byte state) {
  // led_pattern: bit decoded pattern, eg: 9= (00001001b)left most led + forth (led#1 & #4)
  // state: 1=dim, 2=lit, 3=bright, 0=clear
  byte b;
  for (int i = 1; i <= 8; i++) {
    b = led_pattern & 1;
    led_pattern = led_pattern >> 1;
    if (b) {
      dim[i] = 0; lit[i] = 0; brt[i] = 0; // default state cleared
      if (state == 1)  dim[i] = true;
      else if (state == 2)  lit[i] = true;
      else if (state == 3)  brt[i] = true;
    }
  }
}

// -----------------------
void showLevel(byte level) {
  int n;
  clearDisp();
  refreshWait(250);
  for (int i = 0; i < level; i++) {
    beep(10);
    n = (i % nleds) + 1; // n wraps around as i exceeds nleds
    dim[n] = true;
    brightOne = n;
    refreshWait(100);
  }
}

void announceLevel(byte level) {
  onSet = true;
  for (int i = 1; i <= nleds; i++) {
    digitalWrite(lites[i - 1], level != i);
  }
  if (level > nleds) digitalWrite(lites[nleds - 1], ON_STATE);
  genTone(300 * level, 100);
  delay(100);
  while (btnDown()) ; // wait for button release
  for (int i = 1; i <= nleds; i++) {
    digitalWrite(lites[i - 1], !ON_STATE);
  }
  delay(200);
  onSet = false;
}

// -----------------------
byte get_Mode(byte modeCnt) {  // modeCnt is the max selectable mode #
  byte mode;
  byte ledSet = 0;
  byte ledMask = 0;
  int cylceTime = 650;

  clearDisp();
  refreshWait(100);
  if (modeCnt == 0 || modeCnt > 4) {
    buZZ(500);
    return (0);
  }
  for (int i = 4; i >= 1; i--) { // ---- recognizable intro
    showColor(i); refreshWait(150);
    dim[2 * i] = dim[2 * i - 1] = (i <= modeCnt);
  }
  showColor(0);

  while (btnDown()) ; // wait for button release
  while (! Btn || btnNum > modeCnt) {
    if (ESC) return (0);
    if (0 == (msCnt % cylceTime)) { // every 1/2 sec highlight a posible selection
      ledSet = (ledSet + 1) % modeCnt; // cycles 0 1 2 3
      ledMask = 0x03 << 2 * (ledSet);
      set_LEDs(ledMask, 2);
    } if (200 == (msCnt % cylceTime)) {
      set_LEDs(ledMask, 1); // undo last lit
    }
    refreshWait(1);
    scanBtns();
    if (ESC || (btn4 && modeCnt < 4)) return (0);
  }
  set_LEDs(ledMask, 1); // undo last lit
  mode = btnNum;
  // highlight bright their selection
  ledMask = 0x03 << 2 * (mode - 1);
  set_LEDs(ledMask, 3);
  while (btnDown()) ; // wait for button release
  clearDisp();
  return mode;
}


byte getOption(byte optCnt) {
  //  return(get_Option(optCnt,1));
  return (get_Mode(optCnt)); // I like this better as part of Mega_Games
}

// -----------------------
byte get_Option(byte optCnt, int dflt) {  // optCnt is the max # option, dflt is the default value
  clearDisp();
  for (int i = nleds; i>0; i--) { // ---- recognizable intro
    spriteOne = i; refreshWait(100);
    dim[i] = (i <= optCnt);
    if (dflt==i) spriteTwo = dflt;
  }
  spriteOne = spriteTwo = 0;

  flashLed = dflt;
  while (btnDown()) ; // wait for button release
  while (! btn1) {
    refreshWait(1);
    if (btnPressed()) {
      if (btn2Pressed) flashLed = (flashLed == 0) ?  optCnt : flashLed - 1; // -- move left Allow 0:debugging
      if (btn3Pressed) flashLed = (flashLed % optCnt) + 1;            // ++ moves right
      if (btn1 && btn4) return (0); // ESCape hatch
      // at this time allow volumn control (here as it did't operate in loop code as it calls here at start up)
      // --------------- cycle: quiet-normal-Loud volume
      if (btn4Pressed) {
        if (Loud) {
          Loud = false;
          quiet = true;
        }
        else if (quiet) quiet = false;
        else Loud = true; // with AC audio vs pulsing DC
        if (Loud && !spkr) {
          Loud = false;
          quiet = true;
        }
        if (quiet) pinMode(AUDIO_NEG, INPUT);  // silent or low volume with a 150-220 ohm to ground
        else pinMode(AUDIO_NEG, OUTPUT);
        while (btn4) {
          dit_dah(20);
          scanBtns();
        }
      }
    }
  }
  //clearDisp(); this will reset flashLed !
  return (flashLed);
}

// -----------------------
bool catchBtn() {
  if (!Btn) scanBtns();
  return Btn;
}

// -----------------------
bool scanWait(int msec) {  // scan for button upto n msecs
  while (msec > 0) {
    if (ESC) break;
    scanBtns();
    refreshWait(1);
    msec--;
  }
  return Btn;
}
// -----------------------
bool scanBtnWait(int msec) {
  while (msec > 0) {
    refreshWait(1);
    scanBtns();
    if (Btn) break;
    msec--;
  }
  return Btn;
}

// -----------------------
bool btnPressed() { // check for a new button pressing
  btnsNow();
  return (BtnChanged && Btn != 0);
}

// -----------------------
bool btnDown() { // check if button is still down
  refreshWait(1);
  scanBtns();
  return (Btn != 0);
}

int noBtnDelay(int msec) {  // spend time while there is No Button pressed
  ///?? if (Btn)  return(Btn);  causes an issue somewhere
  if (BtnPressed)  return (Btn); // don't lose a btn press event
  do {
    //delay(1);  // preferred if ISR does LED refresh
    refreshWait(1);
    scanBtns();
  } while (!Btn && msec-- > 0);
  return (Btn);
}

int DispScanWait(int msecs)  {   // wait given amount of time, while updating display and capturing btn presses
  int ms;
  for (ms = msecs; ms > 0; ms--) {
    refreshWait(1);
    if (btnPressed() && BtnChanged)  break;
  }
  refreshWait(ms);
  return (Btn);
}

// -----------------------
void wait_BtnRelease() {
  bool wasESC;

  wasESC = ESC;
  do {
    //delay(1);
    refreshWait(1);
    scanBtns();
    if (!wasESC && ESC) return;
  } while (Btn || BtnChanged);
}

// -----------------------
void standby(int msec) { // standby refreshing Disp for 'msec' or btn press event
  do {
    scanBtns();
    if (Btn) return;
    refreshDisp();
    if (msec > 0) delay(1);
    msec--;
  } while (msec > 0);
}


void LOUDbeep(int delayms) { // works for both Buzzers & Speakers
  for (int i = 0; i < delayms; i++) {     // wait for a delayms ms
    digitalWrite ( AUDIO_OUT, HIGH);
    digitalWrite ( AUDIO_NEG, LOW);
    delayMicroseconds(500);
    digitalWrite ( AUDIO_OUT, LOW);
    digitalWrite ( AUDIO_NEG, HIGH);
    delayMicroseconds(500);
  }
  //digitalWrite ( AUDIO_OUT, LOW);;
  digitalWrite ( AUDIO_NEG, LOW);
}

void LOUDbuZZ(int delayms) { // works for both Buzzers & Speakers
  for (int i = 0; i < delayms; i += 5) { // wait for >= delayms ms
    LOUDbeep(2);
    if (random(2)) refreshDisp();  // flicker LEDs during LOUD buZZ
    digitalWrite ( AUDIO_OUT, HIGH);
    digitalWrite ( AUDIO_NEG, LOW);
    delayMicroseconds(1500);
    digitalWrite ( AUDIO_OUT, LOW);
    digitalWrite ( AUDIO_NEG, HIGH);
    delayMicroseconds(1500);
    if (random(2)) refreshDisp();  // flicker LEDs during LOUD buZZ
  }
  //digitalWrite ( AUDIO_OUT, LOW);;
  digitalWrite ( AUDIO_NEG, LOW);
}

//const int RedSet=4, YelSet=3, GrnSet=1;  // used by readySetGo() to cycle Red/Yellow/!GREEN
int readySetGo(int maxDelay) {  // -------  in support of Head-2-head Games:
  bool falseStarts = false;
  int goDelay;

  while (Btn) scanBtns();     // start with btns clear

  if (maxDelay != 0) falseStarts = true; // pentalize false starts
  setColor(RedSet); noBtnDelay(500);   // Red    - ready?
  setColor(0); noBtnDelay(200);
  setColor(YelSet); noBtnDelay(500);   // Yellow - Set

  goDelay = (maxDelay > 0) ? random(100, maxDelay) : 200;
  setColor(0); noBtnDelay(goDelay);

  if (Btn && falseStarts) return 1;
  if (ESC || btn3) {
    return 1;
  }

  setColor(GrnSet);     // ...                 // Green - GO!
  beep(25);  // the gun shot
  return 0;
  // calling code must do a 'clearDisp()' soon (to clear the setColor Grn)
}

// -------------- display the desired set of Colored LEDs
void dspColors() {
  byte set, n;

  for (int i = 0; i <= nleds; i++) lit[i] = false; // pre clear the 'lit' array
  for (int i = 1; i <= 4; i++) {
    if (colorLeds[i]) {
      set = i - 1;
      n = sizeof(colorSet[0]) / sizeof(colorSet[0][0]); // the number of leds per set
      for (int i = 0; i < n; i++) {
        lit[ colorSet[set][i] ] = true;
      }
    }
  }
  //refreshDisp();  // can result in a race condition seen at times in 'LED_lights'
  refreshWait(1);
}


void taDah() {
  setColor(0);
  setColor(5); beep(50);                    // Tah
  setColor(0); refreshDisp(); delay(150);  // ...
  setColor(5); beep(200);                   // Dah
  if (level > 0) { // for games using 'level' show it
    showLevel(level);
    while (!btnPressed()) {
      refreshDisp();
    }
  }
  setColor(0); refreshDisp();
}

void winner(int player) {    // acknowledge the "Win-ner!"
  clearDisp();
  refreshWait(100);
  if (player == 1) {
    brt[2] = 1;
    beep(150);
    delay(100);
    brt[1] = 1;
    beep(500);
  }
  else if (player == -1) {
    buZZ(150, 100);
  }
  else          {
    brt[nleds - 1] = 1;
    beep(150);
    delay(100);
    brt[nleds] = 1;
    beep(500);
  }
  clearDisp();
  refreshWait(250);
  playing = false;
}

void  lost()  {   // player has lost, give the 'Raspberry'
  clearDisp();  refreshWait(300);
  showColor(1);  buZZ(100);
  showColor(4);  LOUDbuZZ(300);
  clearDisp();  refreshDisp();
}


// ---------------------  tone() type Audio Output
//                        true tone audio with a Speaker;  fine with Passive buzzer;
//                        only almost, sort of, ok with Active Buzzer, due to Moiré interference patterns

//   Generate Tone frequencies without using "tone()" (saving 1500 program bytes & 21 ram bytes)
//     sounding almost as good
void genTone(int toneFreq, int delayms) {      // Generate a specified TONE
  byte pinState = 0;       // 0 turns it off
  byte mss = 0, cnt = 0;

  if (toneFreq == 0) {
    refreshWait(delayms);
    return;
  }
  pinMode(AUDIO_OUT, OUTPUT);

  int msCnt = ( (long) delayms * toneFreq) / 1000;
  int halfcycle = (1000000L / toneFreq) / 2;
  int nhCycles = 2 * msCnt; // keep calcs out of loop for accuracy sake
  if (nhCycles < 2) nhCycles = 2;
  int cms = 1000 / halfcycle; // calc cnt/ms for disp upd
  for (int i = 0; i < nhCycles; i++) {     // wait for a delayms ms
    if (volHigh) digitalWrite ( AUDIO_NEG, pinState); // set NEG lead to opposite of + lead
    pinState = 1 - pinState;
    digitalWrite ( AUDIO_OUT, pinState);
    delayMicroseconds(halfcycle);
    // may sound better & more accurate, if no LED refreshing is done ! or do each ~2ms, using min. CPU
    if (++cnt >= cms && (pinState == 0 || halfcycle>500)) {
      refreshDisp();
      cnt = 0;
    }
  }
  digitalWrite ( AUDIO_OUT, LOW);
  digitalWrite ( AUDIO_NEG, LOW);
}

// ----------------------------------
void playCord(int tone1, int tone2, int tone3, int delayms) {      // generate a Cord of three tones
  int cnt = ((long) delayms * 1000L) / (1000000L / tone1 + 1000000L / tone2 + 1000000L / tone3) + 1;
  //int cnt = ((long) delayms * 1000L) / (1000000L/tone2+1000000L/tone3) +1;
  for (int c = 0; c < cnt; c++) {
    digitalWrite ( SPEAKERPIN, HIGH);
    delayMicroseconds((1000000L / tone1) / 2);
    digitalWrite ( SPEAKERPIN, LOW);
    delayMicroseconds((1000000L / tone2) / 2);
    digitalWrite ( SPEAKERPIN, HIGH);
    delayMicroseconds((1000000L / tone3) / 2);
    digitalWrite ( SPEAKERPIN, LOW);
    delayMicroseconds((1000000L / tone1) / 2);
    digitalWrite ( SPEAKERPIN, HIGH);
    delayMicroseconds((1000000L / tone2) / 2);
    digitalWrite ( SPEAKERPIN, LOW);
    delayMicroseconds((1000000L / tone3) / 2);
  }
  digitalWrite ( SPEAKERPIN, LOW);
  digitalWrite ( SPEAKERNEG, LOW);
}

//   How to do it with out using "tone()" (saving 1500 program bytes & 21 ram bytes)
//     sounding almost as good
void beepTone(int toneVal, int msecs) {
  byte pinState = 0;       // 0 turns it off
  pinMode(BEEPPIN, OUTPUT);

  //  t1=millis();
  int cnt = 0;
  int hcpms = 2 * toneVal / 1000; // half cycles / ms
  int cycles = ( (long) msecs * toneVal) / 1000;
  if (cycles < 1) cycles = 1;
  for (int i = 0; i < (2 * cycles); i++) {
    pinState = 1 - pinState;
    digitalWrite ( BEEPPIN, pinState);
    delayMicroseconds((1000000L / toneVal) / 2); // 1/2 a cycle wait
    //    if (t1!=millis() && pinState==0) {refreshDisp(); t1=millis();}
    if (++cnt >= hcpms && pinState == 0) {
      refreshDisp();
      cnt = 0;
    }
  }
  digitalWrite ( BEEPPIN, LOW);
}

void playNote(int note, int delayms) {
  genTone(basicKeyNotes[note], delayms);
}

void playNote(int note) {
  playNote(note, 500);
}


void boOop(int delayms) { // 250Hz boop tone with Vibrado
  bool up = true;
  unsigned int p = 2000;   // 1/2 cycle period=2000us for 250hz
  int stP = p - (p / 100); // -1% for bottom freq's ~1/8 cP
  int endP = p + (p / 100);
  // note: vibrado scales up&down for 80 msecs each
  unsigned int ms;

  for (int i = 0; i < delayms; i += 4) {   // wait for a delayms ms
    ms = millis();
    up = (ms / 80) % 2;
    if (up) p = stP  + (endP - stP) * (ms % 80) / 80;
    else    p = endP - (endP - stP) * (ms % 80) / 80;
    //Serial.println(p);
    digitalWrite ( BEEPPIN, HIGH);
    delayMicroseconds(p);
    refreshDisp();
    digitalWrite ( BEEPPIN, LOW);
    delayMicroseconds(p);
    refreshDisp();
  }
}

void Beep(unsigned short int delayms) { // higher freq & longer dury cycle, tobe more effective
  delayms = 2 * delayms / 3;
  for (int i = 0; i < delayms; i++) {     // wait for a delayms ms
    digitalWrite ( BEEPPIN, HIGH);
    delayMicroseconds(666);
    digitalWrite ( BEEPPIN, LOW);
    delayMicroseconds(333);
    digitalWrite ( BEEPPIN, HIGH);
    delayMicroseconds(333);
    digitalWrite ( BEEPPIN, LOW);
    delayMicroseconds(166);
    refreshDisp();
  }
  digitalWrite ( BEEPPIN, LOW);;
  //  delay(delayms/2);          // wait for a delayms ms
}

void bEEp(int msecs) { // higher freq bEEp  (2KHz)
  for (int i = 0; i < (2 * msecs); i++) {   // loop for so many 'msecs'
    digitalWrite ( AUDIO_OUT, HIGH);
    delayMicroseconds(250);
    digitalWrite ( AUDIO_OUT, LOW);
    delayMicroseconds(250);
    if (i % 2 == 1) refreshDisp(); // just spent 1 ms, lets refreshDisp now
  }
}


void beeps(unsigned short int delayms, int cnt) {
  for (int i = 0; i < cnt; i++) {
    beep(delayms, delayms / 3);
  }
}

// ----------------------------------
//   Generate a Sound Effect segment
//     using frequency & volume vectors (a start & end Freq. and volume) plus duration
//     Freq.s are in cycle/sec,  Volumes is 0-20 (10 is normal fill on), duration is in msecs.
void genSndEffect(int stFreq, int endFreq, int stVol, int endVol, int dur ) {
  int d, deltaD = 10;
  int FreqStep, toneVal;
  float VolStep, vol;
  byte pinState = 0;       // 0 turns it off
  int numSteps;
  int volCnt;
  bool volLow, vHigh, oldvHigh; // note: gobal 'volHigh' not used within this utility

  if (stVol == 0 and endVol == 0) {
    delay(dur);  // simply mark time
    return;
  }
  pinMode(SPEAKERPIN, OUTPUT);
  numSteps = dur / deltaD;
  FreqStep = (endFreq - stFreq) / numSteps;
  VolStep = float (endVol - stVol) / numSteps;
  toneVal = stFreq;
  vol = stVol;
  volCnt = 0;
  for (d = 0; d < dur; d += deltaD) {
    int msCnt = ( (long) deltaD * toneVal) / 1000;
    for (int i = 0; i < (2 * msCnt); i++) {   // wait for a delayms ms
      oldvHigh = vHigh;
      vHigh = (volCnt < (vol - 10));
      if (vHigh) digitalWrite ( SPEAKERNEG, pinState); // set NEG lead to opposite of + lead
      pinState = 1 - pinState;
      volLow = (volCnt > vol);
      if (!volLow) {
        digitalWrite ( SPEAKERPIN, pinState);
      } else digitalWrite ( SPEAKERPIN, 0);
      delayMicroseconds((1000000L / toneVal) / 2);
      //refreshDisp(); // ill effects on audio quality
      if (++volCnt > 10) volCnt = 0;
      if (oldvHigh && !vHigh) digitalWrite ( SPEAKERNEG, LOW);
    }
    toneVal += FreqStep;
    vol += VolStep;
  }
  digitalWrite ( SPEAKERPIN, LOW);
  digitalWrite ( SPEAKERNEG, LOW);
}

void phoneTone(byte keyCode) {
  int freq;
  freq = 1100 + 110 * keyCode;
  genTone(freq, 150);
}
